@@ -51,6 +51,7 @@ const char
     * const GridBase::META_VECTOR_TYPE = "vector_type",
     * const GridBase::META_FILE_BBOX_MIN = "file_bbox_min",
     * const GridBase::META_FILE_BBOX_MAX = "file_bbox_max",
+    * const GridBase::META_FILE_COMPRESSION = "file_compression",
     * const GridBase::META_FILE_MEM_BYTES = "file_mem_bytes",
     * const GridBase::META_FILE_VOXEL_COUNT = "file_voxel_count";
 
@@ -446,6 +447,10 @@ void
 GridBase::addStatsMetadata()
 {
     const CoordBBox bbox = this->evalActiveVoxelBoundingBox();
+    this->removeMeta(META_FILE_BBOX_MIN);
+    this->removeMeta(META_FILE_BBOX_MAX);
+    this->removeMeta(META_FILE_MEM_BYTES);
+    this->removeMeta(META_FILE_VOXEL_COUNT);
     this->insertMeta(META_FILE_BBOX_MIN,    Vec3IMetadata(bbox.min().asVec3i()));
     this->insertMeta(META_FILE_BBOX_MAX,    Vec3IMetadata(bbox.max().asVec3i()));
     this->insertMeta(META_FILE_MEM_BYTES,   Int64Metadata(this->memUsage()));
