@@ -34,7 +34,9 @@
 #include <iostream>
 #include <set>
 #include <vector>
+#include <boost/static_assert.hpp>
 #include <boost/type_traits/remove_const.hpp>
+#include <boost/type_traits/is_floating_point.hpp>
 #include <openvdb/Types.h>
 #include <openvdb/util/Name.h>
 #include <openvdb/math/Transform.h>
@@ -75,6 +77,25 @@ template<typename TreePtrType>
 inline typename Grid<typename TreePtrType::element_type>::Ptr createGrid(TreePtrType);
 
 
+/// @brief Create a new grid of type @c GridType classified as a "Level Set",
+/// i.e., a narrow-band level set.
+///
+/// @note @c GridType::ValueType must be a floating-point scalar.
+///
+/// @param voxelSize  the size of a voxel in world units
+/// @param halfWidth  the half width of the narrow band in voxel units
+///
+/// @details The voxel size and the narrow band half width define the grid's
+/// background value as halfWidth*voxelWidth.  The transform is linear
+/// with a uniform scaling only corresponding to the specified voxel size.
+///
+/// @note It is generally advisable to specify a half-width of the narrow band
+/// that is larger than one voxel unit, otherwise zero crossings are not guaranteed.
+template<typename GridType>
+typename GridType::Ptr createLevelSet(
+    double voxelSize = 1.0, double halfWidth = LEVEL_SET_HALF_WIDTH);
+
+
 ////////////////////////////////////////
 
 
@@ -206,11 +227,9 @@ public:
     static std::string gridClassToString(GridClass);
     /// Return a formatted string version of the grid class.
     static std::string gridClassToMenuName(GridClass);
-
     /// @brief Return the class of volumetric data specified by the given string.
-    ///
-    /// If the string is not one of the ones returned by gridClassToString(),
-    /// return GRID_UNKNOWN.
+    /// @details If the string is not one of the ones returned by gridClassToString(),
+    /// return @c GRID_UNKNOWN.
     static GridClass stringToGridClass(const std::string&);
 
     /// @brief Return the type of vector data (invariant, covariant, etc.) stored
@@ -228,8 +247,7 @@ public:
     /// (e.g., "Gradient/Normal", given VEC_COVARIANT).
     static std::string vecTypeExamples(VecType);
     /// @brief Return a string describing how the given type of vector data is affected
-    /// by transformations.
-    /// (e.g., "Does not transform", given VEC_INVARIANT).
+    /// by transformations (e.g., "Does not transform", given VEC_INVARIANT).
     static std::string vecTypeDescription(VecType);
     static VecType stringToVecType(const std::string&);
 
@@ -242,7 +260,7 @@ public:
 
     // Standard metadata field names
     // (These fields should normally not be accessed directly, but rather
-    // via the accessor methods above.)
+    // via the accessor methods above, when available.)
     // Note: Visual C++ requires these declarations to be separate statements.
     static const char* const META_GRID_CLASS;
     static const char* const META_GRID_CREATOR;
@@ -252,6 +270,7 @@ public:
     static const char* const META_VECTOR_TYPE;
     static const char* const META_FILE_BBOX_MIN;
     static const char* const META_FILE_BBOX_MAX;
+    static const char* const META_FILE_COMPRESSION;
     static const char* const META_FILE_MEM_BYTES;
     static const char* const META_FILE_VOXEL_COUNT;
 
@@ -310,11 +329,13 @@ public:
     /// or constTransform().
     void setTransform(math::Transform::Ptr);
 
-    /// Return this grid's voxel dimensions.
-    Vec3d voxelDimensions() const { return transform().voxelDimensions(); }
-    /// @brief Return this grid's voxel dimensions at position (x, y, z).
-    /// @note Frustum and perspective transforms have position-dependent voxel dimensions.
-    Vec3d voxelDimensions(const Vec3d& xyz) const { return transform().voxelDimensions(xyz); }
+    /// Return the size of this grid's voxels.
+    Vec3d voxelSize() const { return transform().voxelSize(); }
+    OPENVDB_DEPRECATED Vec3d voxelDimensions() const { return transform().voxelSize(); }
+    /// @brief Return the size of this grid's voxel at position (x, y, z).
+    /// @note Frustum and perspective transforms have position-dependent voxel size.
+    Vec3d voxelSize(const Vec3d& xyz) const { return transform().voxelSize(xyz); }
+    OPENVDB_DEPRECATED Vec3d voxelDimensions(const Vec3d& xyz) const { return transform().voxelSize(xyz); }
     /// Return true if the voxels in world space are uniformly sized cubes
     bool hasUniformVoxels() const { return mTransform->hasUniformScale(); }
     //@{
@@ -457,7 +478,17 @@ public:
     typedef typename _TreeType::ValueAllIter              ValueAllIter;
     typedef typename _TreeType::ValueAllCIter             ValueAllCIter;
 
-
+    /// @brief ValueConverter<T>::Type is the type of a grid having the same
+    /// hierarchy as this grid but a different value type, T.
+    ///
+    /// For example, FloatGrid::ValueConverter<double>::Type is equivalent to DoubleGrid.
+    /// @note If the source grid type is a template argument, it might be necessary
+    /// to write "typename SourceGrid::template ValueConverter<T>::Type".
+    template<typename OtherValueType>
+    struct ValueConverter {
+        typedef Grid<typename TreeType::template ValueConverter<OtherValueType>::Type> Type;
+    };
+    
     /// Return a new grid with the given background value.
     static Ptr create(const ValueType& background);
     /// Return a new grid with background value zero.
@@ -566,7 +597,9 @@ public:
     /// operation for optimal sparseness.
     void fill(const CoordBBox& bbox, const ValueType& value, bool active = true);
 
-    /// @brief Propagate the sign from a narrow-band level set into inactive voxels and tiles.
+    /// @brief Set the values of all inactive voxels of a narrow-band level set
+    /// from the signs of the active values, setting outside values to +background
+    /// and inside values to -background.
     /// @note This method should only be used on closed, narrow-band level sets!
     void signedFloodFill() { tree().signedFloodFill(); }
 
@@ -592,7 +625,9 @@ public:
     /// as active and set to their original value in this grid.
     template<typename OtherTreeType>
     void topologyUnion(const Grid<OtherTreeType>& other) { tree().topologyUnion(other.tree()); }
-
+    /// @todo topologyIntersection
+    /// @todo topologyDifference
+    
     //
     // Statistics
     //
@@ -1132,6 +1167,23 @@ createGrid(TreePtrType tree)
     return Grid<TreeType>::create(tree);
 }
 
+
+template<typename GridType>
+typename GridType::Ptr
+createLevelSet(double voxelSize, double halfWidth)
+{
+    typedef typename GridType::ValueType ValueType;
+
+    // GridType::ValueType is required to be a floating-point scalar.
+    BOOST_STATIC_ASSERT(boost::is_floating_point<ValueType>::value);
+
+    typename GridType::Ptr grid = GridType::create(
+        /*background=*/static_cast<ValueType>(voxelSize * halfWidth));
+    grid->setTransform(math::Transform::createLinearTransform(voxelSize));
+    grid->setGridClass(GRID_LEVEL_SET);
+    return grid;
+}
+
 } // namespace OPENVDB_VERSION_NAME
 } // namespace openvdb
 
