@@ -97,6 +97,8 @@ typedef math::Mat3<Real>    Mat3R;
 
 // Four-dimensional matrix types
 typedef math::Mat4<Real>    Mat4R;
+typedef math::Mat4<double>  Mat4d;
+typedef math::Mat4<float>   Mat4s;
 
 // Compressed Hermite data
 typedef math::Hermite       Hermite;
@@ -104,6 +106,16 @@ typedef math::Hermite       Hermite;
 // Quaternions
 typedef math::Quat<Real>    QuatR;
 
+
+////////////////////////////////////////
+
+
+template<typename T> struct VecTraits { static const bool IsVec = false; };
+template<typename T> struct VecTraits<math::Vec2<T> > { static const bool IsVec = true; };
+template<typename T> struct VecTraits<math::Vec3<T> > { static const bool IsVec = true; };
+template<typename T> struct VecTraits<math::Vec4<T> > { static const bool IsVec = true; };
+
+
 ////////////////////////////////////////
 
 
@@ -116,6 +128,7 @@ enum GridClass {
 };
 enum { NUM_GRID_CLASSES = GRID_STAGGERED + 1 };
 
+static const Real LEVEL_SET_HALF_WIDTH = 3;
 
 /// The type of a vector determines how transforms are applied to it:
 /// <dl>
@@ -149,7 +162,8 @@ enum { NUM_VEC_TYPES = VEC_CONTRAVARIANT_ABSOLUTE + 1 };
 
 ////////////////////////////////////////
 
-
+//Deprecated
+/*
 class ValueProbe
 {
 public:
@@ -192,7 +206,7 @@ public:
     void setState(bool state) {mState = state;}
     bool getState() const {return mState;}
 };
-
+*/
 
 ////////////////////////////////////////
 
@@ -212,7 +226,9 @@ template<> inline const char* typeNameAsString<Vec3i>()       { return "vec3i";
 template<> inline const char* typeNameAsString<Vec3f>()       { return "vec3s"; }
 template<> inline const char* typeNameAsString<Vec3d>()       { return "vec3d"; }
 template<> inline const char* typeNameAsString<std::string>() { return "string"; }
-
+template<> inline const char* typeNameAsString<Mat4s>()       { return "mat4s"; }
+template<> inline const char* typeNameAsString<Mat4d>()       { return "mat4d"; }
+    
 
 ////////////////////////////////////////
 
