@@ -38,6 +38,7 @@
 #include <boost/uuid/uuid_generators.hpp>
 #include <boost/uuid/uuid_io.hpp>
 #include <openvdb/Exceptions.h>
+#include <openvdb/Metadata.h>
 #include <openvdb/util/logging.h>
 #include "GridDescriptor.h"
 
@@ -47,20 +48,26 @@ OPENVDB_USE_VERSION_NAMESPACE
 namespace OPENVDB_VERSION_NAME {
 namespace io {
 
-/// Allocate storage for per-stream file format and library version numbers.
+/// Allocate storage for per-stream file format and library version numbers
+/// and other values of use to readers and writers.
 const int
     Archive::sFormatVersionIndex = std::ios_base::xalloc(),
     Archive::sLibraryMajorVersionIndex = std::ios_base::xalloc(),
     Archive::sLibraryMinorVersionIndex = std::ios_base::xalloc(),
-    Archive::sDataCompression = std::ios_base::xalloc(),
-    Archive::sWriteGridStatsMetadata = std::ios_base::xalloc();
+    Archive::sDataCompressionIndex = std::ios_base::xalloc(),
+    Archive::sWriteGridStatsMetadataIndex = std::ios_base::xalloc(),
+    Archive::sGridBackgroundIndex = std::ios_base::xalloc(),
+    Archive::sGridClassIndex = std::ios_base::xalloc();
+
+const uint32_t Archive::DEFAULT_COMPRESSION_FLAGS = (COMPRESS_ZIP | COMPRESS_ACTIVE_MASK);
+
 
 Archive::Archive():
     mFileVersion(OPENVDB_FILE_VERSION),
     mUuid(boost::uuids::nil_uuid()),
     mInputHasGridOffsets(false),
     mEnableInstancing(true),
-    mEnableCompression(true),
+    mCompression(DEFAULT_COMPRESSION_FLAGS),
     mEnableGridStats(true)
 {
     mLibraryVersion.first = OPENVDB_LIBRARY_MAJOR_VERSION;
@@ -160,24 +167,65 @@ Archive::version() const
 ////////////////////////////////////////
 
 
-bool
-getDataCompression(std::istream& is)
+uint32_t
+getDataCompression(std::ios_base& strm)
 {
-    return is.iword(Archive::sDataCompression) != 0;
-}
-
-
-bool
-getDataCompression(std::ostream& os)
-{
-    return os.iword(Archive::sDataCompression) != 0;
+    return uint32_t(strm.iword(Archive::sDataCompressionIndex));
 }
 
 
 void
 Archive::setDataCompression(std::istream& is)
 {
-    is.iword(Archive::sDataCompression) = mEnableCompression;
+    is.iword(Archive::sDataCompressionIndex) = mCompression;
+}
+
+
+bool
+Archive::isCompressionEnabled() const
+{
+    return (mCompression & COMPRESS_ZIP);
+}
+
+
+void
+Archive::setCompressionEnabled(bool b)
+{
+    if (b) mCompression |= COMPRESS_ZIP;
+    else mCompression &= ~COMPRESS_ZIP;
+}
+
+
+void
+Archive::setGridCompression(std::ostream& os, const GridBase& grid) const
+{
+    // Start with the options that are enabled globally for this archive.
+    uint32_t compression = compressionFlags();
+
+    // Disable options that are inappropriate for the given grid.
+    switch (grid.getGridClass()) {
+        case GRID_LEVEL_SET:
+        case GRID_FOG_VOLUME:
+            // Zip compression is not used on level sets or fog volumes.
+            compression = compression & ~COMPRESS_ZIP;
+            break;
+        default:
+            break;
+    }
+    os.iword(Archive::sDataCompressionIndex) = compression;
+
+    os.write(reinterpret_cast<const char*>(&compression), sizeof(uint32_t));
+}
+
+
+void
+Archive::readGridCompression(std::istream& is)
+{
+    if (getFormatVersion(is) >= OPENVDB_FILE_VERSION_NODE_MASK_COMPRESSION) {
+        uint32_t compression = COMPRESS_NONE;
+        is.read(reinterpret_cast<char*>(&compression), sizeof(uint32_t));
+        is.iword(Archive::sDataCompressionIndex) = compression;
+    }
 }
 
 
@@ -187,14 +235,40 @@ Archive::setDataCompression(std::istream& is)
 bool
 getWriteGridStatsMetadata(std::ostream& os)
 {
-    return os.iword(Archive::sWriteGridStatsMetadata) != 0;
+    return os.iword(Archive::sWriteGridStatsMetadataIndex) != 0;
 }
 
 
 void
 Archive::setWriteGridStatsMetadata(std::ostream& os)
 {
-    os.iword(Archive::sWriteGridStatsMetadata) = mEnableGridStats;
+    os.iword(Archive::sWriteGridStatsMetadataIndex) = mEnableGridStats;
+}
+
+
+////////////////////////////////////////
+
+
+uint32_t
+getGridClass(std::ios_base& strm)
+{
+    const uint32_t val = strm.iword(Archive::sGridClassIndex);
+    if (val >= NUM_GRID_CLASSES) return GRID_UNKNOWN;
+    return val;
+}
+
+
+const void*
+getGridBackgroundValuePtr(std::ios_base& strm)
+{
+    return strm.pword(Archive::sGridBackgroundIndex);
+}
+
+
+void
+setGridBackgroundValuePtr(std::ios_base& strm, const void* background)
+{
+    strm.pword(Archive::sGridBackgroundIndex) = const_cast<void*>(background);
 }
 
 
@@ -245,12 +319,15 @@ Archive::readHeader(std::istream& is)
         mInputHasGridOffsets = hasGridOffsets;
     }
 
-    // 5) Read the flag that indicates whether leaf data is compressed.
-    mEnableCompression = true;
-    if (mFileVersion >= OPENVDB_FILE_VERSION_SELECTIVE_COMPRESSION) {
+    // 5) Read the flag that indicates whether data is compressed.
+    //    (From version 222 on, compression information is stored per grid.)
+    mCompression = DEFAULT_COMPRESSION_FLAGS;
+    if (mFileVersion >= OPENVDB_FILE_VERSION_SELECTIVE_COMPRESSION &&
+        mFileVersion < OPENVDB_FILE_VERSION_NODE_MASK_COMPRESSION)
+    {
         char isCompressed;
         is.read(&isCompressed, sizeof(char));
-        mEnableCompression = (isCompressed != 0);
+        mCompression = (isCompressed != 0 ? COMPRESS_ZIP : COMPRESS_NONE);
     }
 
     // 6) Read the 16-byte (128-bit) uuid.
@@ -293,8 +370,7 @@ Archive::writeHeader(std::ostream& os, bool seekable) const
     os.write(&hasGridOffsets, sizeof(char));
 
     // 5) Write a flag indicating that this stream contains compressed leaf data.
-    char isCompressed = mEnableCompression;
-    os.write(&isCompressed, sizeof(char));
+    //    (Omitted as of version 222)
 
     // 6) Generate a new random 16-byte (128-bit) uuid and write it to the stream.
     boost::mt19937 ran;
@@ -354,7 +430,24 @@ Archive::connectInstance(const GridDescriptor& gd, const NamedGridMap& grids) co
 void
 Archive::readGrid(GridBase::Ptr grid, const GridDescriptor& gd, std::istream& is)
 {
+    // Read the compression settings for this grid and tag the stream with them
+    // so that downstream functions can reference them.
+    readGridCompression(is);
+
+    is.iword(Archive::sGridClassIndex) = GRID_UNKNOWN;
+    is.pword(Archive::sGridBackgroundIndex) = NULL;
+
     grid->readMeta(is);
+
+    // Add a description of the compression settings to the grid as metadata.
+    /// @todo Would this be useful?
+    //const uint32_t compression = getDataCompression(is);
+    //grid->insertMeta(GridBase::META_FILE_COMPRESSION,
+    //    StringMetadata(compressionToString(compression)));
+
+    const GridClass gridClass = grid->getGridClass();
+    is.iword(Archive::sGridClassIndex) = gridClass;
+
     if (getFormatVersion(is) >= OPENVDB_FILE_VERSION_GRID_INSTANCING) {
         grid->readTransform(is);
         if (!gd.isInstance()) {
@@ -389,9 +482,9 @@ void
 Archive::write(std::ostream& os, const GridCPtrVec& grids, bool seekable,
     const MetaMap& metadata) const
 {
-    // Set stream flags so that downstream functions can query for them.
-    os.iword(Archive::sDataCompression) = isCompressionEnabled();
-    os.iword(Archive::sWriteGridStatsMetadata) = isGridStatsMetadataEnabled();
+    // Set stream flags so that downstream functions can reference them.
+    os.iword(Archive::sDataCompressionIndex) = compressionFlags();
+    os.iword(Archive::sWriteGridStatsMetadataIndex) = isGridStatsMetadataEnabled();
 
     this->writeHeader(os, seekable);
 
@@ -451,6 +544,10 @@ Archive::write(std::ostream& os, const GridCPtrVec& grids, bool seekable,
                 treeMap[treePtr] = gd;
             }
         }
+
+        // Some compression options (e.g., mask compression) are set per grid.
+        // Restore the original settings before writing the next grid.
+        os.iword(Archive::sDataCompressionIndex) = compressionFlags();
     }
 }
 
@@ -473,6 +570,9 @@ Archive::writeGrid(GridDescriptor& gd, GridBase::ConstPtr grid,
     // Now we know the starting grid storage position.
     if (seekable) gd.setGridPos(os.tellp());
 
+    // Save the compression settings for this grid.
+    setGridCompression(os, *grid);
+
     // Save the grid's metadata and transform.
     if (!getWriteGridStatsMetadata(os)) {
         grid->writeMeta(os);
@@ -527,6 +627,9 @@ Archive::writeGridInstance(GridDescriptor& gd, GridBase::ConstPtr grid,
     // Now we know the starting grid storage position.
     if (seekable) gd.setGridPos(os.tellp());
 
+    // Save the compression settings for this grid.
+    setGridCompression(os, *grid);
+
     // Save the grid's metadata and transform.
     grid->writeMeta(os);
     grid->writeTransform(os);
