@@ -40,6 +40,7 @@
 #include <openvdb/Grid.h>
 #include <openvdb/metadata/MetaMap.h>
 #include <openvdb/version.h> // for VersionId
+#include "Compression.h" // for COMPRESS_ZIP, etc.
 
 
 class TestFile;
@@ -67,26 +68,44 @@ OPENVDB_API std::string getVersion(std::istream&);
 /// Associate the current file format and library version numbers with the given input stream.
 OPENVDB_API void setCurrentVersion(std::istream&);
 
-/// Return @c true if the data should be stored in a compressed format.
-OPENVDB_API bool getDataCompression(std::istream&);
-/// Return @c true if the data should be compressed on output.
-OPENVDB_API bool getDataCompression(std::ostream&);
-
 /// Return @c true if grid statistics (active voxel count and bounding box, etc.)
 /// should be computed and stored as grid metadata on output to the given stream.
 OPENVDB_API bool getWriteGridStatsMetadata(std::ostream&);
 
+/// @brief Return a bitwise OR of compression option flags (COMPRESS_ZIP,
+/// COMPRESS_ACTIVE_MASK, etc.) specifying whether and how input data is compressed
+/// or output data should be compressed.
+OPENVDB_API uint32_t getDataCompression(std::ios_base&);
+
+/// @brief Return the class (GRID_LEVEL_SET, GRID_UNKNOWN, etc.) of the grid
+/// currently being read from or written to the given stream.
+OPENVDB_API uint32_t getGridClass(std::ios_base&);
+
+/// @brief Return a pointer to the background value of the grid
+/// currently being read from or written to the given stream.
+OPENVDB_API const void* getGridBackgroundValuePtr(std::ios_base&);
+
+/// @brief Specify (a pointer to) the background value of the grid
+/// currently being read from or written to the given stream.
+/// @note The pointer must remain valid until the entire grid has been read or written.
+OPENVDB_API void setGridBackgroundValuePtr(std::ios_base&, const void* background);
+
 
 /// Grid serializer/unserializer
 class OPENVDB_API Archive
 {
 public:
-    // VC9 requires these declarations to be on separate lines.
+    // Indices into a stream's internal extensible array of values of use to readers and writers
+    // (VC9 requires these declarations to be on separate lines.)
     static const int sFormatVersionIndex;
     static const int sLibraryMajorVersionIndex;
     static const int sLibraryMinorVersionIndex;
-    static const int sDataCompression;
-    static const int sWriteGridStatsMetadata;
+    static const int sDataCompressionIndex;
+    static const int sWriteGridStatsMetadataIndex;
+    static const int sGridBackgroundIndex;
+    static const int sGridClassIndex;
+
+    static const uint32_t DEFAULT_COMPRESSION_FLAGS;
 
     Archive();
     virtual ~Archive();
@@ -114,12 +133,21 @@ public:
     /// @note Instancing is enabled by default.
     void setInstancingEnabled(bool b) { mEnableInstancing = b; }
 
-    /// Return @c true if the data stream is compressed.
-    bool isCompressionEnabled() const { return mEnableCompression; }
-    /// @brief Specify whether the data stream should be compressed.
-    /// @details Enabling compression makes I/O slower, but saves space.
+    /// Return @c true if the data stream is Zip-compressed.
+    bool isCompressionEnabled() const;
+    /// @brief Specify whether the data stream should be Zip-compressed.
+    /// @details Enabling Zip compression makes I/O slower, but saves space.
     /// Disable it only if raw I/O speed is a concern.
-    void setCompressionEnabled(bool b) { mEnableCompression = b; }
+    void setCompressionEnabled(bool);
+
+    /// Return a bit mask specifying compression options for the data stream.
+    uint32_t compressionFlags() const { return mCompression; }
+    /// @brief Specify whether and how the data stream should be compressed.
+    /// [Mainly for internal use]
+    /// @param c bitwise OR (e.g., COMPRESS_ZIP | COMPRESS_ACTIVE_MASK) of
+    ///     compression option flags (see Compression.h for the available flags)
+    /// @note Not all combinations of compression options are supported.
+    void setCompressionFlags(uint32_t c) { mCompression = c; }
 
     /// @brief Return @c true if grid statistics (active voxel count and
     /// bounding box, etc.) are computed and written as grid metadata.
@@ -147,10 +175,17 @@ protected:
     /// @sa getLibraryVersion()
     void setLibraryVersion(std::istream&);
 
-    /// @brief Tag the given input stream with a flag indicating whether
-    /// the input stream contains compressed data.
+    /// @brief Tag the given input stream with flags indicating whether
+    /// the input stream contains compressed data and how it is compressed.
     void setDataCompression(std::istream&);
 
+    /// @brief Tag an output stream with flags specifying only those
+    /// compression options that are applicable to the given grid.
+    void setGridCompression(std::ostream&, const GridBase&) const;
+    /// @brief Read in the compression flags for a grid and
+    /// tag the given input stream with those flags.
+    static void readGridCompression(std::istream&);
+
     /// @brief Tag the given output stream with a flag indicating whether
     /// to compute and write grid statistics metadata.
     void setWriteGridStatsMetadata(std::ostream&);
@@ -207,8 +242,8 @@ private:
     /// Flag indicating whether a tree shared by multiple grids should be
     /// written out only once (true) or once per grid (false)
     bool mEnableInstancing;
-    /// Flag indicating whether the data stream is compressed
-    bool mEnableCompression;
+    /// Flags indicating whether and how the data stream is compressed
+    uint32_t mCompression;
     /// Flag indicating whether grid statistics metadata should be written
     bool mEnableGridStats;
 }; // class Archive
