@@ -30,6 +30,7 @@
 
 #include "Compression.h"
 
+#include <boost/algorithm/string/join.hpp>
 #include <boost/shared_array.hpp>
 #include <zlib.h>
 #include <openvdb/Exceptions.h>
@@ -41,6 +42,23 @@ OPENVDB_USE_VERSION_NAMESPACE
 namespace OPENVDB_VERSION_NAME {
 namespace io {
 
+std::string
+compressionToString(uint32_t flags)
+{
+    if (flags == COMPRESS_NONE) return "none";
+
+    std::vector<std::string> words;
+    if (flags & COMPRESS_ZIP) words.push_back("zipped");
+    if (flags & COMPRESS_ACTIVE_MASK) {
+        words.push_back("active values");
+    }
+    return boost::join(words, " ");
+}
+
+
+////////////////////////////////////////
+
+
 namespace {
 const int ZIP_COMPRESSION_LEVEL = Z_DEFAULT_COMPRESSION; ///< @todo use Z_BEST_SPEED?
 }
