@@ -72,10 +72,10 @@ private:
 
 public:
     /// Generates an uninitialized (i.e in-valid) Frustum
-    Frustum() {}
+OPENVDB_DEPRECATED Frustum() {}
 
     /// Simple copy constructor
-    Frustum(const Frustum& other)
+OPENVDB_DEPRECATED Frustum(const Frustum& other)
     {
         mFace[0]=other.mFace[0];
         mFace[1]=other.mFace[1];
@@ -89,11 +89,11 @@ public:
     /// GridType (or TransformType) is assumed to have a
     /// indexToWorld(Vector) method! A complicating aspect is that
     /// linear- and frustrum-transforms employ different handedness!
-    Frustum(const Transform& t) { this->init(t); }
+OPENVDB_DEPRECATED Frustum(const Transform& t) { this->init(t); }
 
     /// Constructor takes the six planes of the Frustum. Note the
     /// ordering: front,back,left,right,up,down
-    Frustum(const PlaneType &front, const PlaneType &back,
+OPENVDB_DEPRECATED Frustum(const PlaneType &front, const PlaneType &back,
             const PlaneType &left,  const PlaneType &right,
             const PlaneType &up,    const PlaneType &down)
     {
@@ -114,7 +114,7 @@ public:
     ///     direction, up and left should form an orthogonal basis for the frustum.
     /// @param zNear,zFar the distance from position along direction to the
     ///     near and far planes of the frustum.
-    Frustum(const Point  &position, //origin = camera position
+OPENVDB_DEPRECATED Frustum(const Point  &position, //origin = camera position
             const Vector &direction,//z-axis = unit-vector from origin to center
             const Vector &up, //x-axis (=center - upper edge of near plane)
             const Vector &left,//y-axis = center - left edge of near plane
@@ -124,11 +124,11 @@ public:
     }
 
     /// @brief Constructor for a Frustum from a NonlinearFrustumMap
-    Frustum(const NonlinearFrustumMap& frustum) {
+OPENVDB_DEPRECATED Frustum(const NonlinearFrustumMap& frustum) {
 
         // an index-space bounding box defines the region transfromed into the frustum
-        Vector min = frustum.getBBox().min().asVec3d();
-        Vector max = frustum.getBBox().max().asVec3d();
+        Vector min = frustum.getBBox().min();
+        Vector max = frustum.getBBox().max();
         // compute the four corners of the front face
         Vector ll = frustum.applyMap(min);
         Vector lr = frustum.applyMap(Vec3d(max.x(),min.y(),min.z()));
