@@ -146,8 +146,50 @@ protected:
 };
 
 
+template<typename T> class Quat;
 template<typename T> class Vec3;
 
+/// Returns rotation matrix specified by the quaternion
+/// The quaternion is normalized and used to construct the matrix
+/// Note that the matrix is transposed to match post-multiplication
+/// symantics.
+template<class MatType>
+MatType rotation(const Quat<typename MatType::value_type> &q,
+                 typename MatType::value_type eps = 1.0e-8)
+{
+    typedef typename MatType::value_type T;
+
+    T qdot(q.dot(q));
+    T s(0);
+
+    if (!isApproxEqual(qdot, T(0.0),eps)) {
+        s = 2.0 / qdot;
+    }
+
+    T x  = s*q.x();
+    T y  = s*q.y();
+    T z  = s*q.z();
+    T wx = x*q.w();
+    T wy = y*q.w();
+    T wz = z*q.w();
+    T xx = x*q.x();
+    T xy = y*q.x();
+    T xz = z*q.x();
+    T yy = y*q.y();
+    T yz = z*q.y();
+    T zz = z*q.z();
+
+    MatType r;
+    r[0][0]=T(1) - (yy+zz); r[0][1]=xy + wz;        r[0][2]=xz - wy;
+    r[1][0]=xy - wz;        r[1][1]=T(1) - (xx+zz); r[1][2]=yz + wx;
+    r[2][0]=xz + wy;        r[2][1]=yz - wx;        r[2][2]=T(1) - (xx+yy);
+
+    if(MatType::numColumns() == 4) padMat4(r);
+    return r;
+}
+
+
+
 /// @brief Set the matrix to a rotation about the given axis.
 /// @param axis   The axis (one of X, Y, Z) to rotate about.
 /// @param angle  The rotation angle, in radians.
@@ -277,17 +319,18 @@ Vec3<typename MatType::value_type> eulerAngles(
     RotationOrder rotationOrder,
     typename MatType::value_type eps=1.0e-8)
 {
-    typedef Vec3<typename MatType::value_type> V;
-    typename MatType::value_type phi, theta, psi;
+    typedef typename MatType::value_type ValueType;
+    typedef Vec3<ValueType> V;
+    ValueType phi, theta, psi;
 
     switch(rotationOrder)
     {
     case XYZ_ROTATION:
-        if (isApproxEqual(mat[2][0], 1.0, eps)) {
+        if (isApproxEqual(mat[2][0], ValueType(1.0), eps)) {
             theta = M_PI_2;
             phi = 0.5 * atan2(mat[1][2], mat[1][1]);
             psi = phi;
-        } else if (isApproxEqual(mat[2][0], -1.0, eps)) {
+        } else if (isApproxEqual(mat[2][0], ValueType(-1.0), eps)) {
             theta = -M_PI_2;
             phi = 0.5 * atan2(mat[1][2], mat[1][1]);
             psi = -phi;
@@ -300,11 +343,11 @@ Vec3<typename MatType::value_type> eulerAngles(
         }
         return V(phi, theta, psi);
     case ZXY_ROTATION:
-        if (isApproxEqual(mat[1][2], 1.0, eps)) {
+        if (isApproxEqual(mat[1][2], ValueType(1.0), eps)) {
             theta = M_PI_2;
             phi = 0.5 * atan2(mat[0][1], mat[0][0]);
             psi = phi;
-        } else if (isApproxEqual(mat[1][2], -1.0, eps)) {
+        } else if (isApproxEqual(mat[1][2], ValueType(-1.0), eps)) {
             theta = -M_PI/2;
             phi = 0.5 * atan2(mat[0][1],mat[2][1]);
             psi = -phi;
@@ -318,11 +361,11 @@ Vec3<typename MatType::value_type> eulerAngles(
         return V(theta, psi, phi);
 
     case YZX_ROTATION:
-        if (isApproxEqual(mat[0][1], 1.0, eps)) {
+        if (isApproxEqual(mat[0][1], ValueType(1.0), eps)) {
             theta = M_PI_2;
             phi = 0.5 * atan2(mat[2][0], mat[2][2]);
             psi = phi;
-        } else if (isApproxEqual(mat[0][1], -1.0, eps)) {
+        } else if (isApproxEqual(mat[0][1], ValueType(-1.0), eps)) {
             theta = -M_PI/2;
             phi = 0.5 * atan2(mat[2][0], mat[1][0]);
             psi = -phi;
@@ -337,11 +380,11 @@ Vec3<typename MatType::value_type> eulerAngles(
 
     case XZX_ROTATION:
 
-        if (isApproxEqual(mat[0][0], 1.0, eps)) {
+        if (isApproxEqual(mat[0][0], ValueType(1.0), eps)) {
             theta = 0.0;
             phi = 0.5 * atan2(mat[1][2], mat[1][1]);
             psi = phi;
-        } else if (isApproxEqual(mat[0][0], -1.0, eps)) {
+        } else if (isApproxEqual(mat[0][0], ValueType(-1.0), eps)) {
             theta = M_PI;
             psi = 0.5 * atan2(mat[2][1], -mat[1][1]);
             phi = - psi;
@@ -356,11 +399,11 @@ Vec3<typename MatType::value_type> eulerAngles(
 
     case ZXZ_ROTATION:
 
-        if (isApproxEqual(mat[2][2], 1.0, eps)) {
+        if (isApproxEqual(mat[2][2], ValueType(1.0), eps)) {
             theta = 0.0;
             phi = 0.5 * atan2(mat[0][1], mat[0][0]);
             psi = phi;
-        } else if (isApproxEqual(mat[2][2], -1.0, eps)) {
+        } else if (isApproxEqual(mat[2][2], ValueType(-1.0), eps)) {
             theta = M_PI;
             phi = 0.5 * atan2(mat[0][1], mat[0][0]);
             psi = -phi;
@@ -375,11 +418,11 @@ Vec3<typename MatType::value_type> eulerAngles(
 
     case YXZ_ROTATION:
 
-        if (isApproxEqual(mat[2][1], 1.0, eps)) {
+        if (isApproxEqual(mat[2][1], ValueType(1.0), eps)) {
             theta =  - M_PI_2;
             phi = 0.5 * atan2(-mat[1][0], mat[0][0]);
             psi = phi;
-        } else if (isApproxEqual(mat[2][1], -1.0, eps)) {
+        } else if (isApproxEqual(mat[2][1], ValueType(-1.0), eps)) {
             theta = M_PI_2;
             phi = 0.5 * atan2(mat[1][0], mat[0][0]);
             psi = -phi;
@@ -394,11 +437,11 @@ Vec3<typename MatType::value_type> eulerAngles(
 
     case ZYX_ROTATION:
 
-        if (isApproxEqual(mat[0][2], 1.0, eps)) {
+        if (isApproxEqual(mat[0][2], ValueType(1.0), eps)) {
             theta =  -M_PI_2;
             phi = 0.5 * atan2(-mat[1][0], mat[1][1]);
             psi = phi;
-        } else if (isApproxEqual(mat[0][2], -1.0, eps)) {
+        } else if (isApproxEqual(mat[0][2], ValueType(-1.0), eps)) {
             theta = M_PI_2;
             phi = 0.5 * atan2(mat[2][1], mat[2][0]);
             psi = - phi;
@@ -413,11 +456,11 @@ Vec3<typename MatType::value_type> eulerAngles(
 
     case XZY_ROTATION:
 
-        if (isApproxEqual(mat[1][0], -1.0, eps)) {
+        if (isApproxEqual(mat[1][0], ValueType(-1.0), eps)) {
             theta = M_PI_2;
             psi = 0.5 * atan2(mat[2][1], mat[2][2]);
             phi = - psi;
-        } else if (isApproxEqual(mat[1][0], 1.0, eps)) {
+        } else if (isApproxEqual(mat[1][0], ValueType(1.0), eps)) {
             theta = - M_PI_2;
             psi = 0.5 * atan2(- mat[2][1], mat[2][2]);
             phi = psi;
