@@ -46,7 +46,7 @@ namespace math {
 
 template<typename T> class Vec3;
 template<typename T> class Mat4;
-
+template<typename T> class Quat; 
 
 /// @class Mat3 Mat3.h
 /// @brief 3x3 matrix class.
@@ -55,12 +55,18 @@ class Mat3: public Mat<3, T>
 {
 public:
     /// Data type held by the matrix.
-    typedef T value_type;
-    typedef T ValueType;
-
+    typedef T         value_type;
+    typedef T         ValueType;
+    typedef Mat<3, T> MyBase;
     /// Trivial constructor, the matrix is NOT initialized
     Mat3() {}
 
+    /// Constructor given the quaternion rotation, e.g.    Mat3f m(q);
+    /// The quaternion is normalized and used to construct the matrix
+    Mat3(const Quat<T> &q)
+    { setToRotation(q); }
+    
+
     /// Constructor given array of elements, the ordering is in row major form:
     /** @verbatim
         a b c
@@ -69,18 +75,18 @@ public:
         @endverbatim */
     template<typename Source>
     Mat3(Source a, Source b, Source c,
-        Source d, Source e, Source f,
-        Source g, Source h, Source i)
+         Source d, Source e, Source f,
+         Source g, Source h, Source i)
     {
-        this->mm[0] = a;
-        this->mm[1] = b;
-        this->mm[2] = c;
-        this->mm[3] = d;
-        this->mm[4] = e;
-        this->mm[5] = f;
-        this->mm[6] = g;
-        this->mm[7] = h;
-        this->mm[8] = i;
+        MyBase::mm[0] = a;
+        MyBase::mm[1] = b;
+        MyBase::mm[2] = c;
+        MyBase::mm[3] = d;
+        MyBase::mm[4] = e;
+        MyBase::mm[5] = f;
+        MyBase::mm[6] = g;
+        MyBase::mm[7] = h;
+        MyBase::mm[8] = i;
     } // constructor1Test
 
     /// Construct matrix given basis vectors (columns)
@@ -95,15 +101,15 @@ public:
     template<typename Source>
     Mat3(Source *a)
     {
-        this->mm[0] = a[0];
-        this->mm[1] = a[1];
-        this->mm[2] = a[2];
-        this->mm[3] = a[3];
-        this->mm[4] = a[4];
-        this->mm[5] = a[5];
-        this->mm[6] = a[6];
-        this->mm[7] = a[7];
-        this->mm[8] = a[8];
+        MyBase::mm[0] = a[0];
+        MyBase::mm[1] = a[1];
+        MyBase::mm[2] = a[2];
+        MyBase::mm[3] = a[3];
+        MyBase::mm[4] = a[4];
+        MyBase::mm[5] = a[5];
+        MyBase::mm[6] = a[6];
+        MyBase::mm[7] = a[7];
+        MyBase::mm[8] = a[8];
     } // constructor1Test
 
     /// Copy constructor
@@ -111,7 +117,7 @@ public:
     {
         for (int i=0; i<3; ++i) {
             for (int j=0; j<3; ++j) {
-                this->mm[i*3 + j] = m[i][j];
+                MyBase::mm[i*3 + j] = m[i][j];
             }
         }
     }
@@ -122,7 +128,7 @@ public:
     {
         for (int i=0; i<3; ++i) {
             for (int j=0; j<3; ++j) {
-                this->mm[i*3 + j] = m[i][j];
+                MyBase::mm[i*3 + j] = m[i][j];
             }
         }
     }
@@ -132,7 +138,7 @@ public:
     {
         for (int i=0; i<3; ++i) {
             for (int j=0; j<3; ++j) {
-                this->mm[i*3 + j] = m[i][j];
+                MyBase::mm[i*3 + j] = m[i][j];
             }
         }
     }
@@ -153,9 +159,9 @@ public:
         // assert(i>=0 && i<3);
         int i3 = i * 3;
 
-        this->mm[i3+0] = v[0];
-        this->mm[i3+1] = v[1];
-        this->mm[i3+2] = v[2];
+        MyBase::mm[i3+0] = v[0];
+        MyBase::mm[i3+1] = v[1];
+        MyBase::mm[i3+2] = v[2];
     } // rowColumnTest
 
     /// Get ith row, e.g.    Vec3d v = m.row(1);
@@ -169,9 +175,9 @@ public:
     void setCol(int j, const Vec3<T>& v)
     {
         // assert(j>=0 && j<3);
-        this->mm[0+j] = v[0];
-        this->mm[3+j] = v[1];
-        this->mm[6+j] = v[2];
+        MyBase::mm[0+j] = v[0];
+        MyBase::mm[3+j] = v[1];
+        MyBase::mm[6+j] = v[2];
     } // rowColumnTest
 
     /// Get jth column, e.g.    Vec3d v = m.col(0);
@@ -188,29 +194,12 @@ public:
     //@{
     /// Array style reference to ith row
     /// e.g.    m[1][2] = 4;
-    T* operator[](int i) { return &(this->mm[i*3]); }
-    const T* operator[](int i) const { return &(this->mm[i*3]); }
+    T* operator[](int i) { return &(MyBase::mm[i*3]); }
+    const T* operator[](int i) const { return &(MyBase::mm[i*3]); }
     //@}
 
-    /// For certain legacy APIs, we need to treat the contents as a pointer.
-    /// @remark This was originally designed as an "operator T*()", but that
-    /// automatic pointer cast would be applied in unexpected cases, such as
-    /// std::min/max.
-#ifdef  ALLOW_CAST_TO_POINTER
-    /// Cast to T*
-    operator T*()
-#if (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
-            __attribute__ ((deprecated))
-#endif
-            { return this->mm; } // trivial
-    operator const T*() const
-#if (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
-            __attribute__ ((deprecated))
-#endif
-            { return this->mm; } // trivial
-#endif
-    T* asPointer() {return this->mm;}
-    const T* asPointer() const {return this->mm;}
+    T* asPointer() {return MyBase::mm;}
+    const T* asPointer() const {return MyBase::mm;}
 
 
     /// Alternative indexed reference to the elements
@@ -220,7 +209,7 @@ public:
     {
         // assert(i>=0 && i<3);
         // assert(j>=0 && j<3);
-        return this->mm[3*i+j];
+        return MyBase::mm[3*i+j];
     } // trivial
 
     /// Alternative indexed constant reference to the elements,
@@ -230,35 +219,35 @@ public:
     {
         // assert(i>=0 && i<3);
         // assert(j>=0 && j<3);
-        return this->mm[3*i+j];
+        return MyBase::mm[3*i+j];
     } // trivial
 
     /// Set the columns of "this" matrix to the vectors v1, v2, v3
     void setBasis(const Vec3<T> &v1, const Vec3<T> &v2, const Vec3<T> &v3)
     {
-        this->mm[0] = v1[0];
-        this->mm[1] = v1[1];
-        this->mm[2] = v1[2];
-        this->mm[3] = v2[0];
-        this->mm[4] = v2[1];
-        this->mm[5] = v2[2];
-        this->mm[6] = v3[0];
-        this->mm[7] = v3[1];
-        this->mm[8] = v3[2];
+        MyBase::mm[0] = v1[0];
+        MyBase::mm[1] = v1[1];
+        MyBase::mm[2] = v1[2];
+        MyBase::mm[3] = v2[0];
+        MyBase::mm[4] = v2[1];
+        MyBase::mm[5] = v2[2];
+        MyBase::mm[6] = v3[0];
+        MyBase::mm[7] = v3[1];
+        MyBase::mm[8] = v3[2];
     } // setBasisTest
 
     /// Set diagonal and symmetric triangular components
     void setSymmetric(const Vec3<T> &vdiag, const Vec3<T> &vtri)
     {
-        this->mm[0] = vdiag[0];
-        this->mm[1] = vtri[0];
-        this->mm[2] = vtri[1];
-        this->mm[3] = vtri[0];
-        this->mm[4] = vdiag[1];
-        this->mm[5] = vtri[2];
-        this->mm[6] = vtri[1];
-        this->mm[7] = vtri[2];
-        this->mm[8] = vdiag[2];
+        MyBase::mm[0] = vdiag[0];
+        MyBase::mm[1] = vtri[0];
+        MyBase::mm[2] = vtri[1];
+        MyBase::mm[3] = vtri[0];
+        MyBase::mm[4] = vdiag[1];
+        MyBase::mm[5] = vtri[2];
+        MyBase::mm[6] = vtri[1];
+        MyBase::mm[7] = vtri[2];
+        MyBase::mm[8] = vdiag[2];
     } // setSymmetricTest
 
     /// Returns matrix with prescribed diagonal and symmetric triangular
@@ -266,15 +255,23 @@ public:
     static Mat3 symmetric(const Vec3<T> &vdiag, const Vec3<T> &vtri)
     {
         return Mat3(
-            vdiag[0], vtri[0], vtri[1],
-            vtri[0], vdiag[1], vtri[2],
-            vtri[1], vtri[2], vdiag[2]);
+                    vdiag[0], vtri[0], vtri[1],
+                    vtri[0], vdiag[1], vtri[2],
+                    vtri[1], vtri[2], vdiag[2]
+                    );
     }
 
     /// Set the matrix as cross product of the given vector
     void setSkew(const Vec3<T> &v)
     {*this = skew(v);}
 
+    /// Set "this" matrix to the rotation matrix specified by the quaternion
+    /// The quaternion is normalized and used to construct the matrix
+    /// Note that the matrix is transposed to match post-multiplication
+    /// symantics 
+    void setToRotation(const Quat<T> &q)
+    {*this = rotation<Mat3<T> >(q);}
+    
     /// Set "this" matrix to the rotation specified by axis and angle
     /// The axis must be unit vector
     void setToRotation(const Vec3<T> &axis, T angle)
@@ -283,29 +280,29 @@ public:
     // Set "this" matrix to zero
     void setZero()
     {
-        this->mm[0] = 0;
-        this->mm[1] = 0;
-        this->mm[2] = 0;
-        this->mm[3] = 0;
-        this->mm[4] = 0;
-        this->mm[5] = 0;
-        this->mm[6] = 0;
-        this->mm[7] = 0;
-        this->mm[8] = 0;
+        MyBase::mm[0] = 0;
+        MyBase::mm[1] = 0;
+        MyBase::mm[2] = 0;
+        MyBase::mm[3] = 0;
+        MyBase::mm[4] = 0;
+        MyBase::mm[5] = 0;
+        MyBase::mm[6] = 0;
+        MyBase::mm[7] = 0;
+        MyBase::mm[8] = 0;
     } // trivial
 
     /// Set "this" matrix to identity
     void setIdentity()
     {
-        this->mm[0] = 1;
-        this->mm[1] = 0;
-        this->mm[2] = 0;
-        this->mm[3] = 0;
-        this->mm[4] = 1;
-        this->mm[5] = 0;
-        this->mm[6] = 0;
-        this->mm[7] = 0;
-        this->mm[8] = 1;
+        MyBase::mm[0] = 1;
+        MyBase::mm[1] = 0;
+        MyBase::mm[2] = 0;
+        MyBase::mm[3] = 0;
+        MyBase::mm[4] = 1;
+        MyBase::mm[5] = 0;
+        MyBase::mm[6] = 0;
+        MyBase::mm[7] = 0;
+        MyBase::mm[8] = 1;
     } // trivial
 
     /// Assignment operator
@@ -315,31 +312,32 @@ public:
         const Source *src = m.asPointer();
 
         // don't suppress type conversion warnings
-        std::copy(src, (src + this->numElements()), this->mm);
+        std::copy(src, (src + this->numElements()), MyBase::mm);
         return *this;
     } // opEqualToTest
 
     /// Test if "this" is equivalent to m with tolerance of eps value
     bool eq(const Mat3 &m, T eps=1.0e-8) const
     {
-        return (isApproxEqual(this->mm[0],m.mm[0],eps) &&
-                isApproxEqual(this->mm[1],m.mm[1],eps) &&
-                isApproxEqual(this->mm[2],m.mm[2],eps) &&
-                isApproxEqual(this->mm[3],m.mm[3],eps) &&
-                isApproxEqual(this->mm[4],m.mm[4],eps) &&
-                isApproxEqual(this->mm[5],m.mm[5],eps) &&
-                isApproxEqual(this->mm[6],m.mm[6],eps) &&
-                isApproxEqual(this->mm[7],m.mm[7],eps) &&
-                isApproxEqual(this->mm[8],m.mm[8],eps));
+        return (isApproxEqual(MyBase::mm[0],m.mm[0],eps) &&
+                isApproxEqual(MyBase::mm[1],m.mm[1],eps) &&
+                isApproxEqual(MyBase::mm[2],m.mm[2],eps) &&
+                isApproxEqual(MyBase::mm[3],m.mm[3],eps) &&
+                isApproxEqual(MyBase::mm[4],m.mm[4],eps) &&
+                isApproxEqual(MyBase::mm[5],m.mm[5],eps) &&
+                isApproxEqual(MyBase::mm[6],m.mm[6],eps) &&
+                isApproxEqual(MyBase::mm[7],m.mm[7],eps) &&
+                isApproxEqual(MyBase::mm[8],m.mm[8],eps));
     } // trivial
 
     /// Negation operator, for e.g.   m1 = -m2;
     Mat3<T> operator-() const
     {
         return Mat3<T>(
-            -this->mm[0], -this->mm[1], -this->mm[2],
-            -this->mm[3], -this->mm[4], -this->mm[5],
-            -this->mm[6], -this->mm[7], -this->mm[8]);
+                       -MyBase::mm[0], -MyBase::mm[1], -MyBase::mm[2],
+                       -MyBase::mm[3], -MyBase::mm[4], -MyBase::mm[5],
+                       -MyBase::mm[6], -MyBase::mm[7], -MyBase::mm[8]
+                       );
     } // trivial
 
     /// Multiplication operator, e.g.   M = scalar * M;
@@ -347,122 +345,19 @@ public:
     //     return m*scalar;
     // }
 
-    /// this = m1 + m2
-    /// m1 and m2 need not be distinct objects than "this", e.g. m1.add(m1,m2);
-    template<typename T0, typename T1>
-    const Mat3 &add(const Mat3<T0> &m1, const Mat3<T1> &m2)
-    {
-        const T0 *src1 = m1.asPointer();
-        const T1 *src2 = m2.asPointer();
-
-        this->mm[0] = src1[0] + src2[0];
-        this->mm[1] = src1[1] + src2[1];
-        this->mm[2] = src1[2] + src2[2];
-        this->mm[3] = src1[3] + src2[3];
-        this->mm[4] = src1[4] + src2[4];
-        this->mm[5] = src1[5] + src2[5];
-        this->mm[6] = src1[6] + src2[6];
-        this->mm[7] = src1[7] + src2[7];
-        this->mm[8] = src1[8] + src2[8];
-
-        return *this;
-    } // opPlusTest
-
-    /// this = m1 - m2
-    /// m1 and m2 need not be distinct objects than "this", e.g. m1.sub(m1,m2);
-    template<typename T0, typename T1>
-    const Mat3 &sub(const Mat3<T0> &m1, const Mat3<T1> &m2)
-    {
-        const T0 *src1 = m1.asPointer();
-        const T1 *src2 = m2.asPointer();
-
-        this->mm[0] = src1[0] - src2[0];
-        this->mm[1] = src1[1] - src2[1];
-        this->mm[2] = src1[2] - src2[2];
-        this->mm[3] = src1[3] - src2[3];
-        this->mm[4] = src1[4] - src2[4];
-        this->mm[5] = src1[5] - src2[5];
-        this->mm[6] = src1[6] - src2[6];
-        this->mm[7] = src1[7] - src2[7];
-        this->mm[8] = src1[8] - src2[8];
-
-        return *this;
-    } // opMinusTest
-
-    /// this = m * scalar
-    template<typename T0, typename T1>
-    const Mat3 &scale(T0 scalar, const Mat3<T1> &m)
-    {
-        const T1 *src = m.asPointer();
-
-        this->mm[0] = scalar * src[0];
-        this->mm[1] = scalar * src[1];
-        this->mm[2] = scalar * src[2];
-        this->mm[3] = scalar * src[3];
-        this->mm[4] = scalar * src[4];
-        this->mm[5] = scalar * src[5];
-        this->mm[6] = scalar * src[6];
-        this->mm[7] = scalar * src[7];
-        this->mm[8] = scalar * src[8];
-
-        return *this;
-    }
-
-    /// this = m1 * m2
-    /// @note m1 and m2 must not be the same object as this.
-    template<typename T0, typename T1>
-    const Mat3 &mult(const Mat3<T0> &m1, const Mat3<T1> &m2)
-    {
-        const T0 *src1 = m1.asPointer();
-        const T1 *src2 = m2.asPointer();
-        // assert(this!=&m1);
-        // assert(this!=&m2);
-
-        this->mm[0] = static_cast<T>(src1[0] * src2[0] +
-                            src1[1] * src2[3] +
-                            src1[2] * src2[6]);
-        this->mm[1] = static_cast<T>(src1[0] * src2[1] +
-                            src1[1] * src2[4] +
-                            src1[2] * src2[7]);
-        this->mm[2] = static_cast<T>(src1[0] * src2[2] +
-                            src1[1] * src2[5] +
-                            src1[2] * src2[8]);
-
-        this->mm[3] = static_cast<T>(src1[3] * src2[0] +
-                            src1[4] * src2[3] +
-                            src1[5] * src2[6]);
-        this->mm[4] = static_cast<T>(src1[3] * src2[1] +
-                            src1[4] * src2[4] +
-                            src1[5] * src2[7]);
-        this->mm[5] = static_cast<T>(src1[3] * src2[2] +
-                            src1[4] * src2[5] +
-                            src1[5] * src2[8]);
-
-        this->mm[6] = static_cast<T>(src1[6] * src2[0] +
-                            src1[7] * src2[3] +
-                            src1[8] * src2[6]);
-        this->mm[7] = static_cast<T>(src1[6] * src2[1] +
-                            src1[7] * src2[4] +
-                            src1[8] * src2[7]);
-        this->mm[8] = static_cast<T>(src1[6] * src2[2] +
-                            src1[7] * src2[5] +
-                            src1[8] * src2[8]);
-        return *this;
-    } // multTest
-
     /// @brief Returns m, where \f$m_{i,j} *= scalar\f$ for \f$i, j \in [0, 2]\f$
     template <typename S>
     const Mat3<T>& operator*=(S scalar)
     {
-        this->mm[0] *= scalar;
-        this->mm[1] *= scalar;
-        this->mm[2] *= scalar;
-        this->mm[3] *= scalar;
-        this->mm[4] *= scalar;
-        this->mm[5] *= scalar;
-        this->mm[6] *= scalar;
-        this->mm[7] *= scalar;
-        this->mm[8] *= scalar;
+        MyBase::mm[0] *= scalar;
+        MyBase::mm[1] *= scalar;
+        MyBase::mm[2] *= scalar;
+        MyBase::mm[3] *= scalar;
+        MyBase::mm[4] *= scalar;
+        MyBase::mm[5] *= scalar;
+        MyBase::mm[6] *= scalar;
+        MyBase::mm[7] *= scalar;
+        MyBase::mm[8] *= scalar;
         return *this;
     }
 
@@ -472,15 +367,15 @@ public:
     {
         const S *s = m1.asPointer();
 
-        this->mm[0] += s[0];
-        this->mm[1] += s[1];
-        this->mm[2] += s[2];
-        this->mm[3] += s[3];
-        this->mm[4] += s[4];
-        this->mm[5] += s[5];
-        this->mm[6] += s[6];
-        this->mm[7] += s[7];
-        this->mm[8] += s[8];
+        MyBase::mm[0] += s[0];
+        MyBase::mm[1] += s[1];
+        MyBase::mm[2] += s[2];
+        MyBase::mm[3] += s[3];
+        MyBase::mm[4] += s[4];
+        MyBase::mm[5] += s[5];
+        MyBase::mm[6] += s[6];
+        MyBase::mm[7] += s[7];
+        MyBase::mm[8] += s[8];
         return *this;
     }
 
@@ -490,15 +385,15 @@ public:
     {
         const S *s = m1.asPointer();
 
-        this->mm[0] -= s[0];
-        this->mm[1] -= s[1];
-        this->mm[2] -= s[2];
-        this->mm[3] -= s[3];
-        this->mm[4] -= s[4];
-        this->mm[5] -= s[5];
-        this->mm[6] -= s[6];
-        this->mm[7] -= s[7];
-        this->mm[8] -= s[8];
+        MyBase::mm[0] -= s[0];
+        MyBase::mm[1] -= s[1];
+        MyBase::mm[2] -= s[2];
+        MyBase::mm[3] -= s[3];
+        MyBase::mm[4] -= s[4];
+        MyBase::mm[5] -= s[5];
+        MyBase::mm[6] -= s[6];
+        MyBase::mm[7] -= s[7];
+        MyBase::mm[8] -= s[8];
         return *this;
     }
 
@@ -510,36 +405,36 @@ public:
         const T* s0 = m0.asPointer();
         const S* s1 = m1.asPointer();
 
-        this->mm[0] = static_cast<T>(s0[0] * s1[0] +
-                                     s0[1] * s1[3] +
-                                     s0[2] * s1[6]);
-        this->mm[1] = static_cast<T>(s0[0] * s1[1] +
-                                     s0[1] * s1[4] +
-                                     s0[2] * s1[7]);
-        this->mm[2] = static_cast<T>(s0[0] * s1[2] +
-                                     s0[1] * s1[5] +
-                                     s0[2] * s1[8]);
-
-        this->mm[3] = static_cast<T>(s0[3] * s1[0] +
-                                     s0[4] * s1[3] +
-                                     s0[5] * s1[6]);
-        this->mm[4] = static_cast<T>(s0[3] * s1[1] +
-                                     s0[4] * s1[4] +
-                                     s0[5] * s1[7]);
-        this->mm[5] = static_cast<T>(s0[3] * s1[2] +
-                                     s0[4] * s1[5] +
-                                     s0[5] * s1[8]);
-
-        this->mm[6] = static_cast<T>(s0[6] * s1[0] +
-                                     s0[7] * s1[3] +
-                                     s0[8] * s1[6]);
-        this->mm[7] = static_cast<T>(s0[6] * s1[1] +
-                                     s0[7] * s1[4] +
-                                     s0[8] * s1[7]);
-        this->mm[8] = static_cast<T>(s0[6] * s1[2] +
-                                     s0[7] * s1[5] +
-                                     s0[8] * s1[8]);
-
+        MyBase::mm[0] = static_cast<T>(s0[0] * s1[0] +
+                                       s0[1] * s1[3] +
+                                       s0[2] * s1[6]);
+        MyBase::mm[1] = static_cast<T>(s0[0] * s1[1] +
+                                       s0[1] * s1[4] +
+                                       s0[2] * s1[7]);
+        MyBase::mm[2] = static_cast<T>(s0[0] * s1[2] +
+                                       s0[1] * s1[5] +
+                                       s0[2] * s1[8]);
+        
+        MyBase::mm[3] = static_cast<T>(s0[3] * s1[0] +
+                                       s0[4] * s1[3] +
+                                       s0[5] * s1[6]);
+        MyBase::mm[4] = static_cast<T>(s0[3] * s1[1] +
+                                       s0[4] * s1[4] +
+                                       s0[5] * s1[7]);
+        MyBase::mm[5] = static_cast<T>(s0[3] * s1[2] +
+                                       s0[4] * s1[5] +
+                                       s0[5] * s1[8]);
+        
+        MyBase::mm[6] = static_cast<T>(s0[6] * s1[0] +
+                                       s0[7] * s1[3] +
+                                       s0[8] * s1[6]);
+        MyBase::mm[7] = static_cast<T>(s0[6] * s1[1] +
+                                       s0[7] * s1[4] +
+                                       s0[8] * s1[7]);
+        MyBase::mm[8] = static_cast<T>(s0[6] * s1[2] +
+                                       s0[7] * s1[5] +
+                                       s0[8] * s1[8]);
+        
         return *this;
     }
 
@@ -547,24 +442,24 @@ public:
     Mat3 adjoint() const
     {
         return Mat3<T>(
-        this->mm[4]*this->mm[8] - this->mm[5]*this->mm[7],
-        this->mm[2]*this->mm[7] - this->mm[1]*this->mm[8],
-        this->mm[1]*this->mm[5] - this->mm[2]*this->mm[4],
-        this->mm[5]*this->mm[6] - this->mm[3]*this->mm[8],
-        this->mm[0]*this->mm[8] - this->mm[2]*this->mm[6],
-        this->mm[2]*this->mm[3] - this->mm[0]*this->mm[5],
-        this->mm[3]*this->mm[7] - this->mm[4]*this->mm[6],
-        this->mm[1]*this->mm[6] - this->mm[0]*this->mm[7],
-        this->mm[0]*this->mm[4] - this->mm[1]*this->mm[3]);
+        MyBase::mm[4] * MyBase::mm[8] - MyBase::mm[5] * MyBase::mm[7],
+        MyBase::mm[2] * MyBase::mm[7] - MyBase::mm[1] * MyBase::mm[8],
+        MyBase::mm[1] * MyBase::mm[5] - MyBase::mm[2] * MyBase::mm[4],
+        MyBase::mm[5] * MyBase::mm[6] - MyBase::mm[3] * MyBase::mm[8],
+        MyBase::mm[0] * MyBase::mm[8] - MyBase::mm[2] * MyBase::mm[6],
+        MyBase::mm[2] * MyBase::mm[3] - MyBase::mm[0] * MyBase::mm[5],
+        MyBase::mm[3] * MyBase::mm[7] - MyBase::mm[4] * MyBase::mm[6],
+        MyBase::mm[1] * MyBase::mm[6] - MyBase::mm[0] * MyBase::mm[7],
+        MyBase::mm[0] * MyBase::mm[4] - MyBase::mm[1] * MyBase::mm[3]);
     } // adjointTest
 
     /// returns transpose of this
     Mat3 transpose() const
     {
         return Mat3<T>(
-            this->mm[0], this->mm[3], this->mm[6],
-            this->mm[1], this->mm[4], this->mm[7],
-            this->mm[2], this->mm[5], this->mm[8]);
+            MyBase::mm[0], MyBase::mm[3], MyBase::mm[6],
+            MyBase::mm[1], MyBase::mm[4], MyBase::mm[7],
+            MyBase::mm[2], MyBase::mm[5], MyBase::mm[8]);
 
     } // transposeTest
 
@@ -574,7 +469,7 @@ public:
     {
         Mat3<T> inv(adjoint());
 
-        T det = inv.mm[0]*this->mm[0] + inv.mm[1]*this->mm[3] + inv.mm[2]*this->mm[6];
+        T det = inv.mm[0]*MyBase::mm[0] + inv.mm[1]*MyBase::mm[3] + inv.mm[2]*MyBase::mm[6];
 
         // If the determinant is 0, m was singular and "this" will contain junk.
         if (isApproxEqual(det,0.0,tolerance))
@@ -587,17 +482,17 @@ public:
     /// Determinant of matrix
     T det() const
     {
-        T co00 = this->mm[4]*this->mm[8] - this->mm[5]*this->mm[7];
-        T co10 = this->mm[5]*this->mm[6] - this->mm[3]*this->mm[8];
-        T co20 = this->mm[3]*this->mm[7] - this->mm[4]*this->mm[6];
-        T d    = this->mm[0]*co00  + this->mm[1]*co10 + this->mm[2]*co20;
+        T co00 = MyBase::mm[4]*MyBase::mm[8] - MyBase::mm[5]*MyBase::mm[7];
+        T co10 = MyBase::mm[5]*MyBase::mm[6] - MyBase::mm[3]*MyBase::mm[8];
+        T co20 = MyBase::mm[3]*MyBase::mm[7] - MyBase::mm[4]*MyBase::mm[6];
+        T d    = MyBase::mm[0]*co00  + MyBase::mm[1]*co10 + MyBase::mm[2]*co20;
         return d;
     } // determinantTest
 
     /// Trace of matrix
     T trace() const 
     {
-        return this->mm[0]+this->mm[4]+this->mm[8];
+        return MyBase::mm[0]+MyBase::mm[4]+MyBase::mm[8];
     }
     
     /// This function snaps a specific axis to a specific direction,
@@ -635,6 +530,130 @@ public:
         return snapBasis(*this, axis, direction);
     }
 
+
+    /// For certain legacy APIs, we need to treat the contents as a pointer.
+    /// @remark This was originally designed as an "operator T*()", but that
+    /// automatic pointer cast would be applied in unexpected cases, such as
+    /// std::min/max.
+#ifdef  ALLOW_CAST_TO_POINTER
+    /// Cast to T*
+OPENVDB_DEPRECATED operator T*()
+#if (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
+            __attribute__ ((deprecated))
+#endif
+            { return MyBase::mm; } // trivial
+OPENVDB_DEPRECATED operator const T*() const
+#if (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
+            __attribute__ ((deprecated))
+#endif
+            { return MyBase::mm; } // trivial
+#endif
+
+
+    /// this = m1 + m2
+    /// m1 and m2 need not be distinct objects than "this", e.g. m1.add(m1,m2);
+    template<typename T0, typename T1>
+OPENVDB_DEPRECATED const Mat3 &add(const Mat3<T0> &m1, const Mat3<T1> &m2)
+    {
+        const T0 *src1 = m1.asPointer();
+        const T1 *src2 = m2.asPointer();
+
+        MyBase::mm[0] = src1[0] + src2[0];
+        MyBase::mm[1] = src1[1] + src2[1];
+        MyBase::mm[2] = src1[2] + src2[2];
+        MyBase::mm[3] = src1[3] + src2[3];
+        MyBase::mm[4] = src1[4] + src2[4];
+        MyBase::mm[5] = src1[5] + src2[5];
+        MyBase::mm[6] = src1[6] + src2[6];
+        MyBase::mm[7] = src1[7] + src2[7];
+        MyBase::mm[8] = src1[8] + src2[8];
+
+        return *this;
+    } // opPlusTest
+
+    /// this = m1 - m2
+    /// m1 and m2 need not be distinct objects than "this", e.g. m1.sub(m1,m2);
+    template<typename T0, typename T1>
+OPENVDB_DEPRECATED const Mat3 &sub(const Mat3<T0> &m1, const Mat3<T1> &m2)
+    {
+        const T0 *src1 = m1.asPointer();
+        const T1 *src2 = m2.asPointer();
+
+        MyBase::mm[0] = src1[0] - src2[0];
+        MyBase::mm[1] = src1[1] - src2[1];
+        MyBase::mm[2] = src1[2] - src2[2];
+        MyBase::mm[3] = src1[3] - src2[3];
+        MyBase::mm[4] = src1[4] - src2[4];
+        MyBase::mm[5] = src1[5] - src2[5];
+        MyBase::mm[6] = src1[6] - src2[6];
+        MyBase::mm[7] = src1[7] - src2[7];
+        MyBase::mm[8] = src1[8] - src2[8];
+
+        return *this;
+    } // opMinusTest
+
+    /// this = m * scalar
+    template<typename T0, typename T1>
+OPENVDB_DEPRECATED const Mat3 &scale(T0 scalar, const Mat3<T1> &m)
+    {
+        const T1 *src = m.asPointer();
+
+        MyBase::mm[0] = scalar * src[0];
+        MyBase::mm[1] = scalar * src[1];
+        MyBase::mm[2] = scalar * src[2];
+        MyBase::mm[3] = scalar * src[3];
+        MyBase::mm[4] = scalar * src[4];
+        MyBase::mm[5] = scalar * src[5];
+        MyBase::mm[6] = scalar * src[6];
+        MyBase::mm[7] = scalar * src[7];
+        MyBase::mm[8] = scalar * src[8];
+
+        return *this;
+    }
+
+    /// this = m1 * m2
+    /// @note m1 and m2 must not be the same object as this.
+    template<typename T0, typename T1>
+OPENVDB_DEPRECATED const Mat3 &mult(const Mat3<T0> &m1, const Mat3<T1> &m2)
+    {
+        const T0 *src1 = m1.asPointer();
+        const T1 *src2 = m2.asPointer();
+        // assert(this!=&m1);
+        // assert(this!=&m2);
+
+        MyBase::mm[0] = static_cast<T>(src1[0] * src2[0] +
+                                       src1[1] * src2[3] +
+                                       src1[2] * src2[6]);
+        MyBase::mm[1] = static_cast<T>(src1[0] * src2[1] +
+                                       src1[1] * src2[4] +
+                                       src1[2] * src2[7]);
+        MyBase::mm[2] = static_cast<T>(src1[0] * src2[2] +
+                                       src1[1] * src2[5] +
+                                       src1[2] * src2[8]);
+        
+        MyBase::mm[3] = static_cast<T>(src1[3] * src2[0] +
+                                       src1[4] * src2[3] +
+                                       src1[5] * src2[6]);
+        MyBase::mm[4] = static_cast<T>(src1[3] * src2[1] +
+                                       src1[4] * src2[4] +
+                                       src1[5] * src2[7]);
+        MyBase::mm[5] = static_cast<T>(src1[3] * src2[2] +
+                                       src1[4] * src2[5] +
+                                       src1[5] * src2[8]);
+        
+        MyBase::mm[6] = static_cast<T>(src1[6] * src2[0] +
+                                       src1[7] * src2[3] +
+                                       src1[8] * src2[6]);
+        MyBase::mm[7] = static_cast<T>(src1[6] * src2[1] +
+                                       src1[7] * src2[4] +
+                                       src1[8] * src2[7]);
+        MyBase::mm[8] = static_cast<T>(src1[6] * src2[2] +
+                                       src1[7] * src2[5] +
+                                       src1[8] * src2[8]);
+        return *this;
+    } // multTest
+
+
 private:
     static const Mat3<T> sIdentity;
     static const Mat3<T> sZero;
