@@ -85,7 +85,7 @@ struct ISGradient
     result(const StencilT& stencil)
     {
         typedef typename StencilT::ValueType  ValueType;
-        typedef Vec3<ValueType>              Vec3Type;
+        typedef Vec3<ValueType>               Vec3Type;
         return Vec3Type( D1< DiffScheme>::inX(stencil),
                          D1< DiffScheme>::inY(stencil),
                          D1< DiffScheme>::inZ(stencil) );
@@ -228,7 +228,7 @@ struct ISGradientNormSqrd
 
         Vec3Type up   = ISGradient<FD>::result(stencil);
         Vec3Type down = ISGradient<BD>::result(stencil);
-        return math::GudonovsNormSqrd(stencil.template get< 0, 0, 0>()>0, down, up);
+        return math::GudonovsNormSqrd(stencil.template getValue< 0, 0, 0>()>0, down, up);
     }
 };
 
@@ -281,28 +281,28 @@ struct ISGradientNormSqrd<HJWENO5_BIAS>
 
         // SSE optimized
         const simd::Float4
-            v1(s.template get<-2, 0, 0>() - s.template get<-3, 0, 0>(),
-               s.template get< 0,-2, 0>() - s.template get< 0,-3, 0>(),
-               s.template get< 0, 0,-2>() - s.template get< 0, 0,-3>(), 0),
-            v2(s.template get<-1, 0, 0>() - s.template get<-2, 0, 0>(),
-               s.template get< 0,-1, 0>() - s.template get< 0,-2, 0>(),
-               s.template get< 0, 0,-1>() - s.template get< 0, 0,-2>(), 0),
-            v3(s.template get< 0, 0, 0>() - s.template get<-1, 0, 0>(),
-               s.template get< 0, 0, 0>() - s.template get< 0,-1, 0>(),
-               s.template get< 0, 0, 0>() - s.template get< 0, 0,-1>(), 0),
-            v4(s.template get< 1, 0, 0>() - s.template get< 0, 0, 0>(),
-               s.template get< 0, 1, 0>() - s.template get< 0, 0, 0>(),
-               s.template get< 0, 0, 1>() - s.template get< 0, 0, 0>(), 0),
-            v5(s.template get< 2, 0, 0>() - s.template get< 1, 0, 0>(),
-               s.template get< 0, 2, 0>() - s.template get< 0, 1, 0>(),
-               s.template get< 0, 0, 2>() - s.template get< 0, 0, 1>(), 0),
-            v6(s.template get< 3, 0, 0>() - s.template get< 2, 0, 0>(),
-               s.template get< 0, 3, 0>() - s.template get< 0, 2, 0>(),
-               s.template get< 0, 0, 3>() - s.template get< 0, 0, 2>(), 0),
+            v1(s.template getValue<-2, 0, 0>() - s.template getValue<-3, 0, 0>(),
+               s.template getValue< 0,-2, 0>() - s.template getValue< 0,-3, 0>(),
+               s.template getValue< 0, 0,-2>() - s.template getValue< 0, 0,-3>(), 0),
+            v2(s.template getValue<-1, 0, 0>() - s.template getValue<-2, 0, 0>(),
+               s.template getValue< 0,-1, 0>() - s.template getValue< 0,-2, 0>(),
+               s.template getValue< 0, 0,-1>() - s.template getValue< 0, 0,-2>(), 0),
+            v3(s.template getValue< 0, 0, 0>() - s.template getValue<-1, 0, 0>(),
+               s.template getValue< 0, 0, 0>() - s.template getValue< 0,-1, 0>(),
+               s.template getValue< 0, 0, 0>() - s.template getValue< 0, 0,-1>(), 0),
+            v4(s.template getValue< 1, 0, 0>() - s.template getValue< 0, 0, 0>(),
+               s.template getValue< 0, 1, 0>() - s.template getValue< 0, 0, 0>(),
+               s.template getValue< 0, 0, 1>() - s.template getValue< 0, 0, 0>(), 0),
+            v5(s.template getValue< 2, 0, 0>() - s.template getValue< 1, 0, 0>(),
+               s.template getValue< 0, 2, 0>() - s.template getValue< 0, 1, 0>(),
+               s.template getValue< 0, 0, 2>() - s.template getValue< 0, 0, 1>(), 0),
+            v6(s.template getValue< 3, 0, 0>() - s.template getValue< 2, 0, 0>(),
+               s.template getValue< 0, 3, 0>() - s.template getValue< 0, 2, 0>(),
+               s.template getValue< 0, 0, 3>() - s.template getValue< 0, 0, 2>(), 0),
             down = math::WENO5(v1, v2, v3, v4, v5),
             up   = math::WENO5(v6, v5, v4, v3, v2);
 
-        return math::GudonovsNormSqrd(s.template get< 0, 0, 0>()>0, down, up);
+        return math::GudonovsNormSqrd(s.template getValue< 0, 0, 0>()>0, down, up);
     }
 };
 #endif //DWA_OPENVDB  // for SIMD - note will do the computations in float
@@ -341,10 +341,10 @@ struct ISLaplacian< CD_SECOND>
     template<typename StencilT>
     static typename StencilT::ValueType result(const StencilT& stencil)
     {
-        return  stencil.template get< 1, 0, 0>() + stencil.template get<-1, 0, 0>() +
-                stencil.template get< 0, 1, 0>() + stencil.template get< 0,-1, 0>() +
-                stencil.template get< 0, 0, 1>() + stencil.template get< 0, 0,-1>()
-                                                   - 6*stencil.template get< 0, 0, 0>();
+        return  stencil.template getValue< 1, 0, 0>() + stencil.template getValue<-1, 0, 0>() +
+                stencil.template getValue< 0, 1, 0>() + stencil.template getValue< 0,-1, 0>() +
+                stencil.template getValue< 0, 0, 1>() + stencil.template getValue< 0, 0,-1>()
+                                                   - 6*stencil.template getValue< 0, 0, 0>();
     }
 };
 
@@ -372,14 +372,14 @@ struct ISLaplacian< CD_FOURTH>
     static typename StencilT::ValueType result(const StencilT& stencil)
     {
         return (-1./12.)*(
-                stencil.template get< 2, 0, 0>() + stencil.template get<-2, 0, 0>() +
-                stencil.template get< 0, 2, 0>() + stencil.template get< 0,-2, 0>() +
-                stencil.template get< 0, 0, 2>() + stencil.template get< 0, 0,-2>() )
+                stencil.template getValue< 2, 0, 0>() + stencil.template getValue<-2, 0, 0>() +
+                stencil.template getValue< 0, 2, 0>() + stencil.template getValue< 0,-2, 0>() +
+                stencil.template getValue< 0, 0, 2>() + stencil.template getValue< 0, 0,-2>() )
             + (4./3.)*(
-                stencil.template get< 1, 0, 0>() + stencil.template get<-1, 0, 0>() +
-                stencil.template get< 0, 1, 0>() + stencil.template get< 0,-1, 0>() +
-                stencil.template get< 0, 0, 1>() + stencil.template get< 0, 0,-1>() )
-            - 7.5*stencil.template get< 0, 0, 0>();
+                stencil.template getValue< 1, 0, 0>() + stencil.template getValue<-1, 0, 0>() +
+                stencil.template getValue< 0, 1, 0>() + stencil.template getValue< 0,-1, 0>() +
+                stencil.template getValue< 0, 0, 1>() + stencil.template getValue< 0, 0,-1>() )
+            - 7.5*stencil.template getValue< 0, 0, 0>();
     }
 };
 
@@ -410,18 +410,18 @@ struct ISLaplacian< CD_SIXTH>
     static typename StencilT::ValueType result(const StencilT& stencil)
     {
         return (1./90.)*(
-                stencil.template get< 3, 0, 0>() + stencil.template get<-3, 0, 0>() +
-                stencil.template get< 0, 3, 0>() + stencil.template get< 0,-3, 0>() +
-                stencil.template get< 0, 0, 3>() + stencil.template get< 0, 0,-3>() )
+                stencil.template getValue< 3, 0, 0>() + stencil.template getValue<-3, 0, 0>() +
+                stencil.template getValue< 0, 3, 0>() + stencil.template getValue< 0,-3, 0>() +
+                stencil.template getValue< 0, 0, 3>() + stencil.template getValue< 0, 0,-3>() )
             - (3./20.)*(
-                stencil.template get< 2, 0, 0>() + stencil.template get<-2, 0, 0>() +
-                stencil.template get< 0, 2, 0>() + stencil.template get< 0,-2, 0>() +
-                stencil.template get< 0, 0, 2>() + stencil.template get< 0, 0,-2>() )
+                stencil.template getValue< 2, 0, 0>() + stencil.template getValue<-2, 0, 0>() +
+                stencil.template getValue< 0, 2, 0>() + stencil.template getValue< 0,-2, 0>() +
+                stencil.template getValue< 0, 0, 2>() + stencil.template getValue< 0, 0,-2>() )
             + 1.5 *(
-                stencil.template get< 1, 0, 0>() + stencil.template get<-1, 0, 0>() +
-                stencil.template get< 0, 1, 0>() + stencil.template get< 0,-1, 0>() +
-                stencil.template get< 0, 0, 1>() + stencil.template get< 0, 0,-1>() )
-            - (3*49/18.)*stencil.template get< 0, 0, 0>();
+                stencil.template getValue< 1, 0, 0>() + stencil.template getValue<-1, 0, 0>() +
+                stencil.template getValue< 0, 1, 0>() + stencil.template getValue< 0,-1, 0>() +
+                stencil.template getValue< 0, 0, 1>() + stencil.template getValue< 0, 0,-1>() )
+            - (3*49/18.)*stencil.template getValue< 0, 0, 0>();
     }
 };
 //@}
@@ -580,7 +580,7 @@ struct Gradient
         typedef typename internal::ReturnValue<StencilT>::Vec3Type Vec3Type;
 
         Vec3d iGradient( ISGradient<DiffScheme>::result(stencil) );
-        return Vec3Type(map.applyIJT(iGradient, stencil.getCenter().asVec3d()));
+        return Vec3Type(map.applyIJT(iGradient, stencil.getCenterCoord().asVec3d()));
     }
 };
 
@@ -767,7 +767,7 @@ struct GradientBiased
         typedef math::Vec3<ValueType>            Vec3Type;
 
         Vec3d iGradient( ISGradientBiased<GradScheme, Vec3Type>::result(stencil, V) );
-        return Vec3Type(map.applyIJT(iGradient, stencil.getCenter().asVec3d()));
+        return Vec3Type(map.applyIJT(iGradient, stencil.getCenterCoord().asVec3d()));
     }
 };
 //@}
@@ -805,7 +805,7 @@ struct GradientNormSqrd
 
         Vec3Type up   = Gradient<MapType, FD>::result(map, stencil);
         Vec3Type down = Gradient<MapType, BD>::result(map, stencil);
-        return math::GudonovsNormSqrd(stencil.template get< 0, 0, 0>()>0, down, up);
+        return math::GudonovsNormSqrd(stencil.template getValue< 0, 0, 0>()>0, down, up);
     }
 };
 
@@ -901,7 +901,7 @@ struct Divergence
             Vec3d vec( D1Vec<DiffScheme>::inX(stencil, i),
                        D1Vec<DiffScheme>::inY(stencil, i),
                        D1Vec<DiffScheme>::inZ(stencil, i) );
-            div += ValueType(map.applyIJT(vec, stencil.getCenter().asVec3d())[i]);
+            div += ValueType(map.applyIJT(vec, stencil.getCenterCoord().asVec3d())[i]);
         }
         return div;
     }
@@ -1250,7 +1250,7 @@ struct Curl
                 D1Vec<DiffScheme>::inY(stencil, i),
                 D1Vec<DiffScheme>::inZ(stencil, i));
             // dF_i/dx_j   (x_1 = x, x_2 = y,  x_3 = z)
-            mat[i] = Vec3Type(map.applyIJT(vec, stencil.getCenter().asVec3d()));
+            mat[i] = Vec3Type(map.applyIJT(vec, stencil.getCenterCoord().asVec3d()));
         }
         return Vec3Type(mat[2][1] - mat[1][2], // dF_3/dx_2 - dF_2/dx_3
                         mat[0][2] - mat[2][0], // dF_1/dx_3 - dF_3/dx_1
@@ -1428,7 +1428,7 @@ struct Laplacian
                         D1<CD_2ND>::inY(stencil),
                         D1<CD_2ND>::inZ(stencil) );
 
-            d2_rs = map.applyIJC(d2_is, d1_is, stencil.getCenter().asVec3d());
+            d2_rs = map.applyIJC(d2_is, d1_is, stencil.getCenterCoord().asVec3d());
         }
 
         // the trace of the second derivative (range space) matrix is laplacian
@@ -1628,15 +1628,15 @@ struct CPT
         typedef Vec3<ValueType>              Vec3Type;
 
         // current distance
-        ValueType d = stencil.template get< 0, 0, 0>();
+        ValueType d = stencil.template getValue< 0, 0, 0>();
         // compute gradient in physical space where it is a unit normal
         // since the grid holds a distance level set.
         Vec3d vectorFromSurface(d*Gradient<MapType, DiffScheme>::result(map, stencil));
         if (is_linear<MapType>::value) {
-            Vec3d result = stencil.getCenter().asVec3d() - map.applyInverseMap(vectorFromSurface);
+            Vec3d result = stencil.getCenterCoord().asVec3d() - map.applyInverseMap(vectorFromSurface);
             return Vec3Type(result);
         } else {
-            Vec3d location = map.applyMap(stencil.getCenter().asVec3d());
+            Vec3d location = map.applyMap(stencil.getCenterCoord().asVec3d());
             Vec3d result = map.applyInverseMap(location - vectorFromSurface);
             return Vec3Type(result);
         }
@@ -1674,12 +1674,12 @@ struct CPT_RANGE
         typedef typename StencilT::ValueType ValueType;
         typedef Vec3<ValueType>              Vec3Type;
         // current distance
-        ValueType d = stencil.template get< 0, 0, 0>();
+        ValueType d = stencil.template getValue< 0, 0, 0>();
         // compute gradient in physical space where it is a unit normal
         // since the grid holds a distance level set.
         Vec3Type vectorFromSurface =
             d*Gradient<MapType, DiffScheme>::result(map, stencil);
-        Vec3d result = map.applyMap(stencil.getCenter().asVec3d()) - vectorFromSurface;
+        Vec3d result = map.applyMap(stencil.getCenterCoord().asVec3d()) - vectorFromSurface;
 
         return Vec3Type(result);
     }
@@ -1798,8 +1798,8 @@ struct MeanCurvature
              d2_rs = map.applyIJC(d2_is);
              d1_rs = map.applyIJT(d1_is);
          } else {
-             d2_rs = map.applyIJC(d2_is, d1_is, stencil.getCenter().asVec3d());
-             d1_rs = map.applyIJT(d1_is, stencil.getCenter().asVec3d());
+             d2_rs = map.applyIJC(d2_is, d1_is, stencil.getCenterCoord().asVec3d());
+             d1_rs = map.applyIJT(d1_is, stencil.getCenterCoord().asVec3d());
          }
 
          // assemble the mean curvature
@@ -2030,8 +2030,10 @@ public:
     double determinant() const { return mMap->determinant(); }
     double determinant(const Vec3d& in) const { return mMap->determinant(in); }
 
-    Vec3d voxelDimensions() const { return mMap->voxelDimensions(); }
-    Vec3d voxelDimensions(const Vec3d&v) const { return mMap->voxelDimensions(v); }
+    Vec3d voxelSize() const { return mMap->voxelSize(); }
+    Vec3d voxelSize(const Vec3d&v) const { return mMap->voxelSize(v); }
+    OPENVDB_DEPRECATED Vec3d voxelDimensions() const { return mMap->voxelDimensions(); }
+    OPENVDB_DEPRECATED Vec3d voxelDimensions(const Vec3d&v) const { return mMap->voxelDimensions(v); }
  private:
     MapBase::ConstPtr mMap;
 };
