@@ -348,7 +348,7 @@ public:
     }
 
     /// Test if "this" is equivalent to q with tolerance of eps value
-    bool eq(const Quat &q, T eps=1.0e-8) const
+    bool eq(const Quat &q, T eps=1.0e-7) const
     {
         return isApproxEqual(mm[0],q.mm[0],eps) && isApproxEqual(mm[1],q.mm[1],eps) &&
             isApproxEqual(mm[2],q.mm[2],eps) && isApproxEqual(mm[3],q.mm[3],eps) ;
@@ -508,7 +508,7 @@ public:
     bool normalize(T eps =1.0e-8)
     {
         T d = sqrt(mm[0]*mm[0] + mm[1]*mm[1] + mm[2]*mm[2] + mm[3]*mm[3]);
-        if( isApproxEqual(d,0.0,eps) ) return false;
+        if( isApproxEqual(d, T(0.0), eps) ) return false;
         *this *= ( T(1)/d );
         return true;
     } 
@@ -517,7 +517,7 @@ public:
     Quat unit() const
     {
         T d = sqrt(mm[0]*mm[0] + mm[1]*mm[1] + mm[2]*mm[2] + mm[3]*mm[3]);
-        if( isExactlyEqual(d , 0.0) )
+        if( isExactlyEqual(d , T(0.0) ) )
             OPENVDB_THROW(ArithmeticError,
                 "Normalizing degenerate quaternion");
         return *this / d;
@@ -527,7 +527,7 @@ public:
     Quat inverse(T tolerance = 0)
     {
         T d = mm[0]*mm[0] + mm[1]*mm[1] + mm[2]*mm[2] + mm[3]*mm[3];
-        if( isApproxEqual(d, 0.0, tolerance) )
+        if( isApproxEqual(d, T(0.0), tolerance) )
             OPENVDB_THROW(ArithmeticError,
                 "Cannot invert degenerate quaternion");
         Quat result = *this/-d;
@@ -546,8 +546,7 @@ public:
     /// Return rotated vector by "this" quaternion
     Vec3<T> rotateVector(const Vec3<T> &v) const
     {
-        Mat3<T> m;
-        m.setToRotation(*this);
+        Mat3<T> m(*this);
         return m.transform(v);
     } 
 
@@ -598,45 +597,6 @@ protected:
 template <typename S, typename T>
 Quat<T> operator*(S scalar, const Quat<T> &q) { return q*scalar; }
 
-/// Returns rotation matrix specified by the quaternion
-/// The quaternion is normalized and used to construct the matrix
-/// Note that the matrix is transposed to match post-multiplication
-/// symantics.
-template<class MatType>
-MatType rotation(const Quat<typename MatType::value_type> &q,
-                 typename MatType::value_type eps = 1.0e-8)
-{
-    typedef typename MatType::value_type T;
-
-    T qdot(q.dot(q));
-    T s(0);
-
-    if (!isApproxEqual(qdot,0.0,eps)) {
-        s = 2.0 / qdot;
-    }
-
-    T x  = s*q.x();
-    T y  = s*q.y();
-    T z  = s*q.z();
-    T wx = x*q.w();
-    T wy = y*q.w();
-    T wz = z*q.w();
-    T xx = x*q.x();
-    T xy = y*q.x();
-    T xz = z*q.x();
-    T yy = y*q.y();
-    T yz = z*q.y();
-    T zz = z*q.z();
-
-    MatType r;
-    r[0][0]=T(1) - (yy+zz); r[0][1]=xy + wz;        r[0][2]=xz - wy;
-    r[1][0]=xy - wz;        r[1][1]=T(1) - (xx+zz); r[1][2]=yz + wx;
-    r[2][0]=xz + wy;        r[2][1]=yz - wx;        r[2][2]=T(1) - (xx+yy);
-
-    if(MatType::numColumns() == 4) padMat4(r);
-    return r;
-}
-
 
 /// @brief Interpolate between m1 and m2. 
 /// Converts to quaternion  form and uses slerp
