@@ -61,15 +61,15 @@ public:
     typedef typename BufferType::iterator IterType;
 
     /// Initialize the stencil buffer with the values of voxel (x, y, z)
-    /// and its seven neighbors.
+    /// and its neighbors.
     inline void moveTo(const Coord& ijk)
     {
         mCenter = ijk;
         mStencil[0] = mCache.getValue(ijk);
-        static_cast<StencilType*>(this)->init(mCenter);
+        static_cast<StencilType&>(*this).init(mCenter);
     }
     /// @brief Initialize the stencil buffer with the values of voxel
-    /// (x, y, z) and its seven neighbors.
+    /// (x, y, z) and its neighbors.
     ///
     /// @note This version is slightly faster than the one above, since
     /// the center voxel's value is read directly from the iterator.
@@ -78,16 +78,37 @@ public:
     {
         mCenter = iter.getCoord();
         mStencil[0] = *iter;
-        static_cast<StencilType*>(this)->init(mCenter);
+        static_cast<StencilType&>(*this).init(mCenter);
     }
 
-    /// @brief Return the nth value from the stencil buffer.
+    /// @brief Return the value from the stencil buffer with linear
+    /// offset pos.
     ///
-    /// The default (@a n = 0) corresponds to the center point of the stencil.
-    inline ValueType getValue(unsigned int n = 0) const
+    /// The default (@a pos = 0) corresponds to the center point of the stencil.
+    inline ValueType getValue(unsigned int pos = 0) const
     {
-        assert(n < mStencil.size());
-        return mStencil[n];
+        assert(pos < mStencil.size());
+        return mStencil[pos];
+    }
+    
+    /// Return the value at the specified location relative to the center of the stencil
+    template<int i, int j, int k>
+    const ValueType& getValue() const
+    {
+        return mStencil[static_cast<const StencilType&>(*this).template pos<i,j,k>()];
+    }
+
+    /// Return value at specified the linear off pos
+    OPENVDB_DEPRECATED const ValueType& operator()(int pos) const { return this->getValue(pos); }
+
+    template<int i, int j, int k>
+    OPENVDB_DEPRECATED const ValueType& get() const { return this->template getValue<i,j,k>(); }
+    
+    /// Set the value at the specified location relative to the center of the stencil
+    template<int i, int j, int k>
+    void setValue(const ValueType& value)
+    {
+        mStencil[static_cast<const StencilType&>(*this).template pos<i,j,k>()] = value;
     }
 
     /// Return the size of the stencil buffer.
@@ -127,8 +148,29 @@ public:
     }
 
     /// Return the coordinates of the center point of the stencil.
-    inline const Coord& getCenter() const { return mCenter; }
+    OPENVDB_DEPRECATED inline const Coord& getCenter() const { return mCenter; }
 
+    /// Return the coordinates of the center point of the stencil.
+    inline const Coord& getCenterCoord() const { return mCenter; }
+    
+    /// Return the value at the center of the stencil
+    inline const ValueType& getCenterValue() const
+    {
+        return this->getValue<0,0,0>();
+    }
+
+    /// Return true if the center of the stencil intersects the
+    /// iso-contour specified by the isoValue
+    inline bool intersects(const ValueType &isoValue = zeroVal<ValueType>()) const
+    {
+        const bool less = this->getValue< 0, 0, 0>() < isoValue;
+        return (less  ^  (this->getValue<-1, 0, 0>() < isoValue)) ||
+               (less  ^  (this->getValue< 1, 0, 0>() < isoValue)) ||
+               (less  ^  (this->getValue< 0,-1, 0>() < isoValue)) ||
+               (less  ^  (this->getValue< 0, 1, 0>() < isoValue)) ||
+               (less  ^  (this->getValue< 0, 0,-1>() < isoValue)) ||
+               (less  ^  (this->getValue< 0, 0, 1>() < isoValue))  ;
+    }
 
 protected:
     // Constructor is protected to prevent direct instantiation.
@@ -138,12 +180,9 @@ protected:
     {
     }
 
-    void setCenter(const Coord& ijk) { mCenter = ijk; }
-
     typename GridType::ConstAccessor mCache;
-    BufferType mStencil;
-    Coord mCenter;
-    ValueType  mDx2;
+    BufferType                       mStencil;
+    Coord                            mCenter;
 
 }; // class BaseStencil
 
@@ -174,28 +213,27 @@ public:
     typedef typename BaseType::BufferType   BufferType;
     typedef typename GridType::ValueType    ValueType;
     typedef math::Vec3<ValueType>           Vec3Type;
+    static const int SIZE = 7;
 
-    SevenPointStencil(const GridType& grid): BaseType(grid, /*size=*/7) {}
-
+    SevenPointStencil(const GridType& grid): BaseType(grid, SIZE) {}
+    
+    /// Return linear offset for the specified stencil point relative to its center
     template<int i, int j, int k>
-    const ValueType& get() const { return mStencil[SevenPt<i,j,k>::idx]; }
-
-    const ValueType& operator()(int i) const { return mStencil[ i]; }
+    unsigned int pos() const { return SevenPt<i,j,k>::idx; }
 
 private:
     inline void init(const Coord& ijk)
     {
-        mStencil[SevenPt< 0, 0, 0>::idx] = mCache.getValue(ijk);
+        BaseType::template setValue< 0, 0, 0>(mCache.getValue(ijk));
 
-        mStencil[SevenPt< 1, 0, 0>::idx] = mCache.getValue(ijk.offsetBy( 1,  0,  0));
-        mStencil[SevenPt<-1, 0, 0>::idx] = mCache.getValue(ijk.offsetBy(-1,  0,  0));
+        BaseType::template setValue<-1, 0, 0>(mCache.getValue(ijk.offsetBy(-1, 0, 0)));
+        BaseType::template setValue< 1, 0, 0>(mCache.getValue(ijk.offsetBy( 1, 0, 0)));
 
-        mStencil[SevenPt< 0, 1, 0>::idx] = mCache.getValue(ijk.offsetBy( 0,  1,  0));
-        mStencil[SevenPt< 0,-1, 0>::idx] = mCache.getValue(ijk.offsetBy( 0, -1,  0));
-
-        mStencil[SevenPt< 0, 0, 1>::idx] = mCache.getValue(ijk.offsetBy( 0,  0,  1));
-        mStencil[SevenPt< 0, 0,-1>::idx] = mCache.getValue(ijk.offsetBy( 0,  0, -1));
+        BaseType::template setValue< 0,-1, 0>(mCache.getValue(ijk.offsetBy( 0,-1, 0)));
+        BaseType::template setValue< 0, 1, 0>(mCache.getValue(ijk.offsetBy( 0, 1, 0)));
 
+        BaseType::template setValue< 0, 0,-1>(mCache.getValue(ijk.offsetBy( 0, 0,-1)));
+        BaseType::template setValue< 0, 0, 1>(mCache.getValue(ijk.offsetBy( 0, 0, 1)));
     }
 
     template<typename, typename> friend class BaseStencil; // allow base class to call init()
@@ -249,14 +287,13 @@ public:
     typedef typename GridType::ValueType    ValueType;
     typedef math::Vec3<ValueType>           Vec3Type;
 
-    static const int pointCount = 19;
-
-    SecondOrderDenseStencil(const GridType& grid): BaseType(grid, pointCount) {}
+    static const int SIZE = 19;
 
+    SecondOrderDenseStencil(const GridType& grid): BaseType(grid, SIZE) {}
+    
+    /// Return linear offset for the specified stencil point relative to its center
     template<int i, int j, int k>
-    const ValueType& get() const { return mStencil[DensePt<i,j,k>::idx]; }
-
-    const ValueType& operator()(int i) const { return mStencil[i]; }
+    unsigned int pos() const { return DensePt<i,j,k>::idx; }
 
 private:
     inline void init(const Coord& ijk)
@@ -330,14 +367,13 @@ public:
     typedef typename GridType::ValueType    ValueType;
     typedef math::Vec3<ValueType>           Vec3Type;
 
-    static const int pointCount = 13;
-
-    ThirteenPointStencil(const GridType& grid): BaseType(grid, pointCount) {}
+    static const int SIZE = 13;
 
+    ThirteenPointStencil(const GridType& grid): BaseType(grid, SIZE) {}
+    
+    /// Return linear offset for the specified stencil point relative to its center
     template<int i, int j, int k>
-    const ValueType& get() const { return mStencil[ThirteenPt<i,j,k>::idx]; }
-
-    const ValueType& operator()(int i) const { return mStencil[i]; }
+    unsigned int pos() const { return ThirteenPt<i,j,k>::idx; }
 
 private:
     inline void init(const Coord& ijk)
@@ -462,14 +498,13 @@ public:
     typedef typename GridType::ValueType    ValueType;
     typedef math::Vec3<ValueType>           Vec3Type;
 
-    static const int pointCount = 61;
+    static const int SIZE = 61;
 
-    FourthOrderDenseStencil(const GridType& grid): BaseType(grid, pointCount) {}
+    FourthOrderDenseStencil(const GridType& grid): BaseType(grid, SIZE) {}
 
+    /// Return linear offset for the specified stencil point relative to its center
     template<int i, int j, int k>
-    const ValueType& get() const { return mStencil[FourthDensePt<i,j,k>::idx]; }
-
-    const ValueType& operator()(int i) const { return mStencil[i]; }
+    unsigned int pos() const { return FourthDensePt<i,j,k>::idx; }
 
 private:
     inline void init(const Coord& ijk)
@@ -601,15 +636,14 @@ public:
     typedef typename BaseType::BufferType   BufferType;
     typedef typename GridType::ValueType    ValueType;
     typedef math::Vec3<ValueType>           Vec3Type;
+    
+    static const int SIZE = 19;
 
-    static const int pointCount = 19;
-
-    NineteenPointStencil(const GridType& grid): BaseType(grid, pointCount) {}
+    NineteenPointStencil(const GridType& grid): BaseType(grid, SIZE) {}
 
+    /// Return linear offset for the specified stencil point relative to its center
     template<int i, int j, int k>
-    const ValueType& get() const { return mStencil[NineteenPt<i,j,k>::idx]; }
-
-    const ValueType& operator()(int i) const { return mStencil[i]; }
+    unsigned int pos() const { return NineteenPt<i,j,k>::idx; }
 
 private:
     inline void init(const Coord& ijk)
@@ -817,15 +851,14 @@ public:
     typedef typename BaseType::BufferType   BufferType;
     typedef typename GridType::ValueType    ValueType;
     typedef math::Vec3<ValueType>           Vec3Type;
+    
+    static const int SIZE = 127;
 
-    static const int pointCount = 127;
-
-    SixthOrderDenseStencil(const GridType& grid): BaseType(grid, pointCount) {}
-
+    SixthOrderDenseStencil(const GridType& grid): BaseType(grid, SIZE) {}
+    
+    /// Return linear offset for the specified stencil point relative to its center
     template<int i, int j, int k>
-    const ValueType& get() const { return mStencil[SixthDensePt<i,j,k>::idx]; }
-
-    const ValueType& operator()(int i) const { return mStencil[i]; }
+    unsigned int pos() const { return SixthDensePt<i,j,k>::idx; }
 
 private:
     inline void init(const Coord& ijk)
@@ -1002,15 +1035,17 @@ public:
     typedef typename GridType::ValueType                  ValueType;
     typedef math::Vec3<ValueType>                         Vec3Type;
 
+    static const int SIZE = 7;
+
     GradStencil(const GridType& grid):
-        BaseType(grid, /*size=*/7),
-        mInv2Dx(ValueType(0.5 / grid.voxelDimensions()[0])),
+        BaseType(grid, SIZE),
+        mInv2Dx(ValueType(0.5 / grid.voxelSize()[0])),
         mInvDx2(ValueType(4.0 * mInv2Dx * mInv2Dx))
     {
     }
 
     GradStencil(const GridType& grid, Real dx):
-        BaseType(grid, /*size=*/7),
+        BaseType(grid, SIZE),
         mInv2Dx(ValueType(0.5 / dx)),
         mInvDx2(ValueType(4.0 * mInv2Dx * mInv2Dx))
     {
@@ -1081,7 +1116,7 @@ public:
     /// with uniform scaling.
     inline Vec3Type cpt()
     {
-        const Coord& ijk = BaseType::getCenter();
+        const Coord& ijk = BaseType::getCenterCoord();
         const ValueType d = ValueType(mStencil[0] * 0.5 * mInvDx2); // distance in voxels / (2dx^2)
         return Vec3Type(ijk[0] - d*(mStencil[2] - mStencil[1]),
                         ijk[1] - d*(mStencil[4] - mStencil[3]),
@@ -1125,16 +1160,18 @@ public:
     typedef typename GridType::ValueType                  ValueType;
     typedef math::Vec3<ValueType>                         Vec3Type;
 
+    static const int SIZE = 19;
+
     WenoStencil(const GridType& grid):
-        BaseType(grid, /*size=*/19),
-        mDx2(ValueType(math::Pow2(grid.voxelDimensions()[0]))),
-        mInv2Dx(ValueType(0.5 / grid.voxelDimensions()[0])),
+        BaseType(grid, SIZE),
+        mDx2(ValueType(math::Pow2(grid.voxelSize()[0]))),
+        mInv2Dx(ValueType(0.5 / grid.voxelSize()[0])),
         mInvDx2(ValueType(1.0 / mDx2))
     {
     }
 
     WenoStencil(const GridType& grid, Real dx):
-        BaseType(grid, /*size=*/19),
+        BaseType(grid, SIZE),
         mDx2(ValueType(dx * dx)),
         mInv2Dx(ValueType(0.5 / dx)),
         mInvDx2(ValueType(1.0 / mDx2))
@@ -1267,15 +1304,17 @@ public:
     typedef BaseStencil<GridType, CurvatureStencil<GridType> > BaseType;
     typedef typename GridType::ValueType                       ValueType;
 
+     static const int SIZE = 19;
+
     CurvatureStencil(const GridType& grid):
-        BaseType(grid, /*size=*/19),
-        mInv2Dx(ValueType(0.5 / grid.voxelDimensions()[0])),
+        BaseType(grid, SIZE),
+        mInv2Dx(ValueType(0.5 / grid.voxelSize()[0])),
         mInvDx2(ValueType(4.0 * mInv2Dx * mInv2Dx))
     {
     }
 
     CurvatureStencil(const GridType& grid, Real dx):
-        BaseType(grid, /*size=*/19),
+        BaseType(grid, SIZE),
         mInv2Dx(ValueType(0.5 / dx)),
         mInvDx2(ValueType(4.0 * mInv2Dx * mInv2Dx))
     {
@@ -1399,6 +1438,7 @@ public:
         BaseType(grid, /*size=*/math::Pow3(2 * halfWidth + 1)),
         mHalfWidth(halfWidth)
     {
+        //assert(halfWidth>0);//should this be allowed?
     }
 
 private:
