@@ -100,6 +100,8 @@ void initialize()
     Vec3IMetadata::registerType();
     Vec3SMetadata::registerType();
     Vec3DMetadata::registerType();
+    Mat4SMetadata::registerType();
+    Mat4DMetadata::registerType();
 
     // Register maps
     math::MapRegistry::clear();     
