@@ -37,11 +37,13 @@
 #ifndef OPENVDB_TOOLS_COMPOSITE_HAS_BEEN_INCLUDED
 #define OPENVDB_TOOLS_COMPOSITE_HAS_BEEN_INCLUDED
 
+#include <openvdb/Platform.h>
 #include <openvdb/Exceptions.h>
 #include <openvdb/Types.h>
 #include <openvdb/Grid.h>
 #include <openvdb/math/Math.h> // for isExactlyEqual()
 #include "ValueTransformer.h" // for transformValues()
+#include <boost/utility/enable_if.hpp>
 
 
 namespace openvdb {
@@ -52,48 +54,86 @@ namespace tools {
 /// @brief Given two level set grids, replace the A grid with the union of A and B.
 /// @throw ValueError if the background value of either grid is not greater than zero.
 /// @note This operation always leaves the B grid empty.
-template<typename GridOrTreeT> inline void csgUnion(GridOrTreeT& a, GridOrTreeT& b,
-    bool prune = true);
+template<typename GridOrTreeT> OPENVDB_STATIC_SPECIALIZATION
+inline void csgUnion(GridOrTreeT& a, GridOrTreeT& b, bool prune = true);
 /// @brief Given two level set grids, replace the A grid with the intersection of A and B.
 /// @throw ValueError if the background value of either grid is not greater than zero.
 /// @note This operation always leaves the B grid empty.
-template<typename GridOrTreeT> inline void csgIntersection(GridOrTreeT& a, GridOrTreeT& b,
-    bool prune = true);
+template<typename GridOrTreeT> OPENVDB_STATIC_SPECIALIZATION
+inline void csgIntersection(GridOrTreeT& a, GridOrTreeT& b, bool prune = true);
 /// @brief Given two level set grids, replace the A grid with the difference A / B.
 /// @throw ValueError if the background value of either grid is not greater than zero.
 /// @note This operation always leaves the B grid empty.
-template<typename GridOrTreeT> inline void csgDifference(GridOrTreeT& a, GridOrTreeT& b,
-    bool prune = true);
+template<typename GridOrTreeT> OPENVDB_STATIC_SPECIALIZATION
+inline void csgDifference(GridOrTreeT& a, GridOrTreeT& b, bool prune = true);
 
 /// @brief Given grids A and B, compute max(a, b) per voxel (using sparse traversal).
 /// Store the result in the A grid and leave the B grid empty.
-template<typename GridOrTreeT> inline void compMax(GridOrTreeT& a, GridOrTreeT& b);
+template<typename GridOrTreeT> OPENVDB_STATIC_SPECIALIZATION
+inline void compMax(GridOrTreeT& a, GridOrTreeT& b);
 /// @brief Given grids A and B, compute min(a, b) per voxel (using sparse traversal).
 /// Store the result in the A grid and leave the B grid empty.
-template<typename GridOrTreeT> inline void compMin(GridOrTreeT& a, GridOrTreeT& b);
+template<typename GridOrTreeT> OPENVDB_STATIC_SPECIALIZATION
+inline void compMin(GridOrTreeT& a, GridOrTreeT& b);
 /// @brief Given grids A and B, compute a + b per voxel (using sparse traversal).
 /// Store the result in the A grid and leave the B grid empty.
-template<typename GridOrTreeT> inline void compSum(GridOrTreeT& a, GridOrTreeT& b);
+template<typename GridOrTreeT> OPENVDB_STATIC_SPECIALIZATION
+inline void compSum(GridOrTreeT& a, GridOrTreeT& b);
 /// @brief Given grids A and B, compute a * b per voxel (using sparse traversal).
 /// Store the result in the A grid and leave the B grid empty.
-template<typename GridOrTreeT> inline void compMul(GridOrTreeT& a, GridOrTreeT& b);
+template<typename GridOrTreeT> OPENVDB_STATIC_SPECIALIZATION
+inline void compMul(GridOrTreeT& a, GridOrTreeT& b);
 
 /// Copy the active voxels of B into A.
-template<typename GridOrTreeT> inline void compReplace(GridOrTreeT& a, const GridOrTreeT& b);
+template<typename GridOrTreeT> OPENVDB_STATIC_SPECIALIZATION
+inline void compReplace(GridOrTreeT& a, const GridOrTreeT& b);
 
 
 ////////////////////////////////////////
 
 
+namespace composite {
+
+// composite::min() and composite::max() for non-vector types compare with operator<().
+template<typename T> inline
+const typename boost::disable_if_c<VecTraits<T>::IsVec, T>::type& // = T if T is not a vector type
+min(const T& a, const T& b) { return std::min(a, b); }
+
+template<typename T> inline
+const typename boost::disable_if_c<VecTraits<T>::IsVec, T>::type&
+max(const T& a, const T& b) { return std::max(a, b); }
+
+
+// composite::min() and composite::max() for OpenVDB vector types compare by magnitude.
+template<typename T> inline
+const typename boost::enable_if_c<VecTraits<T>::IsVec, T>::type& // = T if T is a vector type
+min(const T& a, const T& b)
+{
+    const typename T::ValueType aMag = a.lengthSqr(), bMag = b.lengthSqr();
+    return (aMag < bMag ? a : (bMag < aMag ? b : std::min(a, b)));
+}
+
+template<typename T> inline
+const typename boost::enable_if_c<VecTraits<T>::IsVec, T>::type&
+max(const T& a, const T& b)
+{
+    const typename T::ValueType aMag = a.lengthSqr(), bMag = b.lengthSqr();
+    return (aMag < bMag ? b : (bMag < aMag ? a : std::max(a, b)));
+}
+
+} // namespace composite
+
+
 template<typename GridOrTreeT>
-inline void
+OPENVDB_STATIC_SPECIALIZATION inline void
 compMax(GridOrTreeT& aTree, GridOrTreeT& bTree)
 {
-    typedef TreeAdapter<GridOrTreeT> Adapter;
-    typedef typename Adapter::TreeType TreeT;
+    typedef TreeAdapter<GridOrTreeT>    Adapter;
+    typedef typename Adapter::TreeType  TreeT;
+    typedef typename TreeT::ValueType   ValueT;
     struct Local {
-        static inline void op(CombineArgs<typename TreeT::ValueType>& args) {
-            args.setResult(std::max(args.a(), args.b()));
+        static inline void op(CombineArgs<ValueT>& args) {
+            args.setResult(composite::max(args.a(), args.b()));
         }
     };
     Adapter::tree(aTree).combineExtended(Adapter::tree(bTree), Local::op, /*prune=*/false);
@@ -101,14 +141,15 @@ compMax(GridOrTreeT& aTree, GridOrTreeT& bTree)
 
 
 template<typename GridOrTreeT>
-inline void
+OPENVDB_STATIC_SPECIALIZATION inline void
 compMin(GridOrTreeT& aTree, GridOrTreeT& bTree)
 {
-    typedef TreeAdapter<GridOrTreeT> Adapter;
-    typedef typename Adapter::TreeType TreeT;
+    typedef TreeAdapter<GridOrTreeT>    Adapter;
+    typedef typename Adapter::TreeType  TreeT;
+    typedef typename TreeT::ValueType   ValueT;
     struct Local {
-        static inline void op(CombineArgs<typename TreeT::ValueType>& args) {
-            args.setResult(std::min(args.a(), args.b()));
+        static inline void op(CombineArgs<ValueT>& args) {
+            args.setResult(composite::min(args.a(), args.b()));
         }
     };
     Adapter::tree(aTree).combineExtended(Adapter::tree(bTree), Local::op, /*prune=*/false);
@@ -116,7 +157,7 @@ compMin(GridOrTreeT& aTree, GridOrTreeT& bTree)
 
 
 template<typename GridOrTreeT>
-inline void
+OPENVDB_STATIC_SPECIALIZATION inline void
 compSum(GridOrTreeT& aTree, GridOrTreeT& bTree)
 {
     typedef TreeAdapter<GridOrTreeT> Adapter;
@@ -131,7 +172,7 @@ compSum(GridOrTreeT& aTree, GridOrTreeT& bTree)
 
 
 template<typename GridOrTreeT>
-inline void
+OPENVDB_STATIC_SPECIALIZATION inline void
 compMul(GridOrTreeT& aTree, GridOrTreeT& bTree)
 {
     typedef TreeAdapter<GridOrTreeT> Adapter;
@@ -145,28 +186,55 @@ compMul(GridOrTreeT& aTree, GridOrTreeT& bTree)
 }
 
 
+////////////////////////////////////////
+
+
+template<typename TreeT>
+struct CompReplaceOp
+{
+    TreeT* const aTree;
+
+    CompReplaceOp(TreeT& _aTree): aTree(&_aTree) {}
+
+    void operator()(const typename TreeT::ValueOnCIter& iter) const
+    {
+        CoordBBox bbox;
+        iter.getBoundingBox(bbox);
+        aTree->fill(bbox, *iter);
+    }
+
+    void operator()(const typename TreeT::LeafCIter& leafIter) const
+    {
+        tree::ValueAccessor<TreeT> acc(*aTree);
+        for (typename TreeT::LeafCIter::LeafNodeT::ValueOnCIter iter =
+            leafIter->cbeginValueOn(); iter; ++iter)
+        {
+            acc.setValue(iter.getCoord(), *iter);
+        }
+    }
+};
+
+
 template<typename GridOrTreeT>
-inline void
+OPENVDB_STATIC_SPECIALIZATION inline void
 compReplace(GridOrTreeT& aTree, const GridOrTreeT& bTree)
 {
     typedef TreeAdapter<GridOrTreeT> Adapter;
     typedef typename Adapter::TreeType TreeT;
-    typedef typename Adapter::GridType GridT;
-    typedef typename GridT::Accessor AccessorT;
+    typedef typename TreeT::ValueOnCIter ValueOnCIterT;
 
-    struct Local {
-        static void op(const typename TreeT::ValueOnCIter& iter, AccessorT& accessor)
-        {
-            if (iter.isVoxelValue()) { // set a single voxel
-                accessor.setValue(iter.getCoord(), *iter);
-            } else { // fill an entire tile
-                CoordBBox bbox;
-                iter.getBoundingBox(bbox);
-                accessor.getTree()->fill(bbox, *iter);
-            }
-        }
-    };
-    transformValues(bTree.cbeginValueOn(), aTree, Local::op);
+    // Copy active states (but not values) from B to A.
+    Adapter::tree(aTree).topologyUnion(Adapter::tree(bTree));
+
+    CompReplaceOp<TreeT> op(Adapter::tree(aTree));
+
+    // Copy all active tile values from B to A.
+    ValueOnCIterT iter = bTree.cbeginValueOn();
+    iter.setMaxDepth(iter.getLeafDepth() - 1); // don't descend into leaf nodes
+    foreach(iter, op);
+
+    // Copy all active voxel values from B to A.
+    foreach(Adapter::tree(bTree).cbeginLeaf(), op);
 }
 
 
@@ -438,7 +506,7 @@ struct CsgDiffVisitor: public CsgVisitorBase<TreeType>
 
 
 template<typename GridOrTreeT>
-inline void
+OPENVDB_STATIC_SPECIALIZATION inline void
 csgUnion(GridOrTreeT& a, GridOrTreeT& b, bool prune)
 {
     typedef TreeAdapter<GridOrTreeT> Adapter;
@@ -451,7 +519,7 @@ csgUnion(GridOrTreeT& a, GridOrTreeT& b, bool prune)
 }
 
 template<typename GridOrTreeT>
-inline void
+OPENVDB_STATIC_SPECIALIZATION inline void
 csgIntersection(GridOrTreeT& a, GridOrTreeT& b, bool prune)
 {
     typedef TreeAdapter<GridOrTreeT> Adapter;
@@ -464,7 +532,7 @@ csgIntersection(GridOrTreeT& a, GridOrTreeT& b, bool prune)
 }
 
 template<typename GridOrTreeT>
-inline void
+OPENVDB_STATIC_SPECIALIZATION inline void
 csgDifference(GridOrTreeT& a, GridOrTreeT& b, bool prune)
 {
     typedef TreeAdapter<GridOrTreeT> Adapter;
