@@ -44,7 +44,7 @@
 #include <openvdb/Types.h>
 #include <openvdb/math/Math.h> // for isApproxEqual()
 #include <openvdb/util/NullInterrupter.h>
-
+#include "Interpolation.h"
 
 namespace openvdb {
 OPENVDB_USE_VERSION_NAMESPACE
@@ -107,122 +107,6 @@ resampleToMatch(const GridType& inGrid, GridType& outGrid);
 ////////////////////////////////////////
 
 
-// The following samplers operate in voxel space.
-// When the samplers are applied to grids holding vector or other non-scalar data type,
-// the data is assumed to be co-located.  For example, using the BoxSampler on a grid
-// with ValueType Vec3f assumes that all three elements in a vector can be assigned
-// the same physical location.
-
-struct PointSampler
-{
-    static const char* name() { return "point"; }
-    static int radius() { return 0; }
-    static bool prescale() { return false; }
-
-    /// Sample @a inTree at the nearest neighbor to @a inCoord
-    /// and store the result in @a result.
-    /// @return true if the sampled value is active.
-    template<class TreeT>
-    static bool sample(const TreeT& inTree, const Vec3R& inCoord,
-        typename TreeT::ValueType& result);
-};
-
-
-struct BoxSampler
-{
-    static const char* name() { return "box"; }
-    static int radius() { return 1; }
-    static bool prescale() { return true; }
-
-    /// Trilinearly reconstruct @a inTree at @a inCoord
-    /// and store the result in @a result.
-    /// @return true if any one of the sampled value is active.
-    template<class TreeT>
-    static bool sample(const TreeT& inTree, const Vec3R& inCoord,
-        typename TreeT::ValueType& result);
-
-    /// Trilinearly reconstruct @a inTree at @a inCoord
-    /// @return sampled value
-    template<class TreeT>
-    static typename TreeT::ValueType sample(const TreeT& inTree, const Vec3R& inCoord);
-
-private:
-    template<class ValueT, size_t N>
-    static inline ValueT trilinearInterpolation(ValueT (& data)[N][N][N], const Vec3R& uvw);
-};
-
-
-struct QuadraticSampler
-{
-    static const char* name() { return "quadratic"; }
-    static int radius() { return 1; }
-    static bool prescale() { return true; }
-
-    /// Triquadratically reconstruct @a inTree at @a inCoord
-    /// and store the result in @a result.
-    /// @return true if any one of the sampled values is active.
-    template<class TreeT>
-    static bool sample(const TreeT& inTree, const Vec3R& inCoord,
-        typename TreeT::ValueType& result);
-};
-
-
-////////////////////////////////////////
-
-
-// The following samplers operate in voxel space and are designed for Vec3
-// staggered grid data (e.g., fluid simulations using the Marker-and-Cell approach
-// associate elements of the velocity vector with different physical locations:
-// the faces of a cube).
-
-struct StaggeredPointSampler
-{
-    static const char* name() { return "point"; }
-    static int radius() { return 0; }
-    static bool prescale() { return false; }
-
-    /// Sample @a inTree at the nearest neighbor to @a inCoord
-    /// and store the result in @a result.
-    /// @return true if the sampled value is active.
-    template<class TreeT>
-    static bool sample(const TreeT& inTree, const Vec3R& inCoord,
-        typename TreeT::ValueType& result);
-};
-
-
-struct StaggeredBoxSampler
-{
-    static const char* name() { return "box"; }
-    static int radius() { return 1; }
-    static bool prescale() { return true; }
-
-    /// Trilinearly reconstruct @a inTree at @a inCoord
-    /// and store the result in @a result.
-    /// @return true if any one of the sampled value is active.
-    template<class TreeT>
-    static bool sample(const TreeT& inTree, const Vec3R& inCoord,
-        typename TreeT::ValueType& result);
-};
-
-
-struct StaggeredQuadraticSampler
-{
-    static const char* name() { return "quadratic"; }
-    static int radius() { return 1; }
-    static bool prescale() { return true; }
-
-    /// Triquadratically reconstruct @a inTree at @a inCoord
-    /// and store the result in @a result.
-    /// @return true if any one of the sampled values is active.
-    template<class TreeT>
-    static bool sample(const TreeT& inTree, const Vec3R& inCoord,
-        typename TreeT::ValueType& result);
-};
-
-
-////////////////////////////////////////
-
-
 namespace internal {
 
 /// @brief A TileSampler wraps a grid sampler of another type (BoxSampler,
@@ -407,30 +291,6 @@ private:
 
 namespace local_util {
 
-inline Vec3i
-floorVec3(const Vec3R& v)
-{
-    return Vec3i(int(std::floor(v(0))), int(std::floor(v(1))), int(std::floor(v(2))));
-}
-
-
-inline Vec3i
-ceilVec3(const Vec3R& v)
-{
-    return Vec3i(int(std::ceil(v(0))), int(std::ceil(v(1))), int(std::ceil(v(2))));
-}
-
-
-inline Vec3i
-roundVec3(const Vec3R& v)
-{
-    return Vec3i(int(::round(v(0))), int(::round(v(1))), int(::round(v(2))));
-}
-
-
-////////////////////////////////////////
-
-
 /// @brief Decompose an affine transform into scale, rotation and translation components.
 /// @return @c false if the given matrix is not affine or cannot otherwise be decomposed.
 /// @todo This is not safe for matrices with shear.
@@ -441,6 +301,7 @@ decompose(const math::Mat4<T>& m, math::Vec3<T>& scale,
 {
     if (!math::isAffine(m)) return false;
 
+    // this is the translation in world space
     translate = m.getTranslation();
     // Extract translation.
     math::Mat3<T> temp = m.getMat3();
@@ -500,10 +361,13 @@ public:
     ABTransform(const math::Transform& aXform, const math::Transform& bXform):
         mAXform(aXform),
         mBXform(bXform),
-        mAffine(mAXform.isLinear() && mBXform.isLinear())
+        mIsAffine(mAXform.isLinear() && mBXform.isLinear()),
+        mIsIdentity(mIsAffine && mAXform == mBXform)
         {}
 
-    bool isAffine() const { return mAffine; }
+    bool isAffine() const { return mIsAffine; }
+
+    bool isIdentity() const { return mIsIdentity; }
 
     openvdb::Vec3R transform(const openvdb::Vec3R& pos) const
     {
@@ -520,7 +384,8 @@ public:
 
 private:
     const math::Transform &mAXform, &mBXform;
-    const bool mAffine;
+    const bool mIsAffine;
+    const bool mIsIdentity;
 };
 
 
@@ -530,12 +395,16 @@ resampleToMatch(const GridType& inGrid, GridType& outGrid, Interrupter& interrup
 {
     ABTransform xform(inGrid.transform(), outGrid.transform());
 
-    if (xform.isAffine()) {
+    if (Sampler::consistent() && xform.isIdentity()) {
+        // If the transform of the input and output are identical, the
+        // output tree is simply a deep copy of the input tree.
+        outGrid.setTree(inGrid.tree().copy());
+    } else if (xform.isAffine()) {
         // If the input and output transforms are both affine, create an
         // input to output transform (in:index-to-world * out:world-to-index)
         // and use the fast GridTransformer API.
         Mat4R mat = xform.getA().baseMap()->getAffineMap()->getMat4() *
-            xform.getB().baseMap()->getAffineMap()->getMat4().inverse();
+            ( xform.getB().baseMap()->getAffineMap()->getMat4().inverse() );
 
         GridTransformer transformer(mat);
         transformer.setInterrupter(interrupter);
@@ -565,261 +434,6 @@ resampleToMatch(const GridType& inGrid, GridType& outGrid)
 ////////////////////////////////////////
 
 
-template<class TreeT>
-inline bool
-PointSampler::sample(const TreeT& inTree, const Vec3R& inCoord,
-    typename TreeT::ValueType& result)
-{
-    Vec3i inIdx = local_util::roundVec3(inCoord);
-    return inTree.probeValue(Coord(inIdx), result);
-}
-
-
-////////////////////////////////////////
-
-
-template<class ValueT, size_t N>
-inline ValueT
-BoxSampler::trilinearInterpolation(ValueT (& data)[N][N][N], const Vec3R& uvw)
-{
-    // Trilinear interpolation:
-    // The eight surrounding latice values are used to construct the result. \n
-    // result(x,y,z) =
-    //     v000 (1-x)(1-y)(1-z) + v001 (1-x)(1-y)z + v010 (1-x)y(1-z) + v011 (1-x)yz
-    //   + v100 x(1-y)(1-z)     + v101 x(1-y)z     + v110 xy(1-z)     + v111 xyz
-
-    ValueT resultA, resultB;
-
-    resultA = data[0][0][0] + ValueT((data[0][0][1] - data[0][0][0]) * uvw[2]);
-    resultB = data[0][1][0] + ValueT((data[0][1][1] - data[0][1][0]) * uvw[2]);
-    ValueT result1 = resultA + ValueT((resultB-resultA) * uvw[1]);
-
-    resultA = data[1][0][0] + ValueT((data[1][0][1] - data[1][0][0]) * uvw[2]);
-    resultB = data[1][1][0] + ValueT((data[1][1][1] - data[1][1][0]) * uvw[2]);
-    ValueT result2 = resultA + ValueT((resultB - resultA) * uvw[1]);
-
-    return result1 + ValueT(uvw[0] * (result2 - result1));
-}
-
-
-template<class TreeT>
-inline bool
-BoxSampler::sample(const TreeT& inTree, const Vec3R& inCoord,
-    typename TreeT::ValueType& result)
-{
-    typedef typename TreeT::ValueType ValueT;
-
-    Vec3i inIdx = local_util::floorVec3(inCoord);
-    Vec3R uvw = inCoord - inIdx;
-
-    // Retrieve the values of the eight voxels surrounding the
-    // fractional source coordinates.
-    ValueT data[2][2][2];
-
-    bool hasActiveValues = false;
-    Coord ijk(inIdx);
-    hasActiveValues |= inTree.probeValue(ijk, data[0][0][0]);  // i, j, k
-    ijk[2] += 1;
-    hasActiveValues |= inTree.probeValue(ijk, data[0][0][1]);  // i, j, k + 1
-    ijk[1] += 1;
-    hasActiveValues |= inTree.probeValue(ijk, data[0][1][1]); // i, j+1, k + 1
-    ijk[2] = inIdx[2];
-    hasActiveValues |= inTree.probeValue(ijk, data[0][1][0]);  // i, j+1, k
-    ijk[0] += 1;
-    ijk[1] = inIdx[1];
-    hasActiveValues |= inTree.probeValue(ijk, data[1][0][0]); // i+1, j, k
-    ijk[2] += 1;
-    hasActiveValues |= inTree.probeValue(ijk, data[1][0][1]); // i+1, j, k + 1
-    ijk[1] += 1;
-    hasActiveValues |= inTree.probeValue(ijk, data[1][1][1]); // i+1, j+1, k + 1
-    ijk[2] = inIdx[2];
-    hasActiveValues |= inTree.probeValue(ijk, data[1][1][0]); // i+1, j+1, k
-
-    result = trilinearInterpolation(data, uvw);
-    return hasActiveValues;
-}
-
-
-template<class TreeT>
-inline typename TreeT::ValueType
-BoxSampler::sample(const TreeT& inTree, const Vec3R& inCoord)
-{
-    typedef typename TreeT::ValueType ValueT;
-
-    Vec3i inIdx = local_util::floorVec3(inCoord);
-    Vec3R uvw = inCoord - inIdx;
-
-    // Retrieve the values of the eight voxels surrounding the
-    // fractional source coordinates.
-    ValueT data[2][2][2];
-
-    Coord ijk(inIdx);
-    data[0][0][0] = inTree.getValue(ijk);  // i, j, k
-    ijk[2] += 1;
-    data[0][0][1] = inTree.getValue(ijk);  // i, j, k + 1
-    ijk[1] += 1;
-    data[0][1][1] = inTree.getValue(ijk); // i, j+1, k + 1
-    ijk[2] = inIdx[2];
-    data[0][1][0] = inTree.getValue(ijk);  // i, j+1, k
-    ijk[0] += 1;
-    ijk[1] = inIdx[1];
-    data[1][0][0] = inTree.getValue(ijk); // i+1, j, k
-    ijk[2] += 1;
-    data[1][0][1] = inTree.getValue(ijk); // i+1, j, k + 1
-    ijk[1] += 1;
-    data[1][1][1] = inTree.getValue(ijk); // i+1, j+1, k + 1
-    ijk[2] = inIdx[2];
-    data[1][1][0] = inTree.getValue(ijk); // i+1, j+1, k
-
-    return trilinearInterpolation(data, uvw);
-}
-
-
-////////////////////////////////////////
-
-
-template<class TreeT>
-inline bool
-QuadraticSampler::sample(const TreeT& inTree, const Vec3R& inCoord,
-    typename TreeT::ValueType& result)
-{
-    typedef typename TreeT::ValueType ValueT;
-
-    Vec3i
-        inIdx = local_util::floorVec3(inCoord),
-        inLoIdx = inIdx - Vec3i(1, 1, 1);
-    Vec3R frac = inCoord - inIdx;
-
-    // Retrieve the values of the 27 voxels surrounding the
-    // fractional source coordinates.
-    bool active = false;
-    ValueT v[3][3][3];
-    for (int dx = 0, ix = inLoIdx.x(); dx < 3; ++dx, ++ix) {
-        for (int dy = 0, iy = inLoIdx.y(); dy < 3; ++dy, ++iy) {
-            for (int dz = 0, iz = inLoIdx.z(); dz < 3; ++dz, ++iz) {
-                if (inTree.probeValue(Coord(ix, iy, iz), v[dx][dy][dz])) {
-                    active = true;
-                }
-            }
-        }
-    }
-
-    /// @todo For vector types, interpolate over each component independently.
-    ValueT vx[3];
-    for (int dx = 0; dx < 3; ++dx) {
-        ValueT vy[3];
-        for (int dy = 0; dy < 3; ++dy) {
-            // Fit a parabola to three contiguous samples in z
-            // (at z=-1, z=0 and z=1), then evaluate the parabola at z',
-            // where z' is the fractional part of inCoord.z, i.e.,
-            // inCoord.z - inIdx.z.  The coefficients come from solving
-            //
-            // | (-1)^2  -1   1 || a |   | v0 |
-            // |    0     0   1 || b | = | v1 |
-            // |   1^2    1   1 || c |   | v2 |
-            //
-            // for a, b and c.
-            const ValueT* vz = &v[dx][dy][0];
-            const ValueT
-                az = static_cast<ValueT>(0.5 * (vz[0] + vz[2]) - vz[1]),
-                bz = static_cast<ValueT>(0.5 * (vz[2] - vz[0])),
-                cz = static_cast<ValueT>(vz[1]);
-            vy[dy] = static_cast<ValueT>(frac.z() * (frac.z() * az + bz) + cz);
-        }
-        // Fit a parabola to three interpolated samples in y, then
-        // evaluate the parabola at y', where y' is the fractional
-        // part of inCoord.y.
-        const ValueT
-            ay = static_cast<ValueT>(0.5 * (vy[0] + vy[2]) - vy[1]),
-            by = static_cast<ValueT>(0.5 * (vy[2] - vy[0])),
-            cy = static_cast<ValueT>(vy[1]);
-        vx[dx] = static_cast<ValueT>(frac.y() * (frac.y() * ay + by) + cy);
-    }
-    // Fit a parabola to three interpolated samples in x, then
-    // evaluate the parabola at the fractional part of inCoord.x.
-    const ValueT
-        ax = static_cast<ValueT>(0.5 * (vx[0] + vx[2]) - vx[1]),
-        bx = static_cast<ValueT>(0.5 * (vx[2] - vx[0])),
-        cx = static_cast<ValueT>(vx[1]);
-    result = static_cast<ValueT>(frac.x() * (frac.x() * ax + bx) + cx);
-
-    return active;
-}
-
-
-////////////////////////////////////////
-
-
-template<class TreeT>
-inline bool
-StaggeredPointSampler::sample(const TreeT& inTree, const Vec3R& inCoord,
-    typename TreeT::ValueType& result)
-{
-    typedef typename TreeT::ValueType ValueType;
-
-    ValueType tempX, tempY, tempZ;
-    bool active = false;
-
-    active = PointSampler::sample<TreeT>(inTree, inCoord + Vec3R(0.5, 0, 0), tempX) || active;
-    active = PointSampler::sample<TreeT>(inTree, inCoord + Vec3R(0, 0.5, 0), tempY) || active;
-    active = PointSampler::sample<TreeT>(inTree, inCoord + Vec3R(0, 0, 0.5), tempZ) || active;
-
-    result.x() = tempX.x();
-    result.y() = tempY.y();
-    result.z() = tempZ.z();
-
-    return active;
-}
-
-
-template<class TreeT>
-inline bool
-StaggeredBoxSampler::sample(const TreeT& inTree, const Vec3R& inCoord,
-    typename TreeT::ValueType& result)
-{
-    typedef typename TreeT::ValueType ValueType;
-
-    ValueType tempX, tempY, tempZ;
-    tempX = tempY = tempZ = zeroVal<ValueType>();
-    bool active = false;
-
-    active = BoxSampler::sample<TreeT>(inTree, inCoord + Vec3R(0.5, 0, 0), tempX) || active;
-    active = BoxSampler::sample<TreeT>(inTree, inCoord + Vec3R(0, 0.5, 0), tempY) || active;
-    active = BoxSampler::sample<TreeT>(inTree, inCoord + Vec3R(0, 0, 0.5), tempZ) || active;
-
-    result.x() = tempX.x();
-    result.y() = tempY.y();
-    result.z() = tempZ.z();
-
-    return active;
-}
-
-
-template<class TreeT>
-inline bool
-StaggeredQuadraticSampler::sample(const TreeT& inTree, const Vec3R& inCoord,
-    typename TreeT::ValueType& result)
-{
-    typedef typename TreeT::ValueType ValueType;
-
-    ValueType tempX, tempY, tempZ;
-    bool active = false;
-
-    active = QuadraticSampler::sample<TreeT>(inTree, inCoord + Vec3R(0.5, 0, 0), tempX) || active;
-    active = QuadraticSampler::sample<TreeT>(inTree, inCoord + Vec3R(0, 0.5, 0), tempY) || active;
-    active = QuadraticSampler::sample<TreeT>(inTree, inCoord + Vec3R(0, 0, 0.5), tempZ) || active;
-
-    result.x() = tempX.x();
-    result.y() = tempY.y();
-    result.z() = tempZ.z();
-
-    return active;
-}
-
-
-////////////////////////////////////////
-
-
 inline
 GridTransformer::GridTransformer(const Mat4R& xform):
     mPivot(0, 0, 0),
@@ -832,7 +446,7 @@ GridTransformer::GridTransformer(const Mat4R& xform):
     if (local_util::decompose(mTransform, scale, rotate, translate)) {
         // If the transform can be decomposed into affine components,
         // use them to set up a mipmapping-like scheme for downsampling.
-        init(mPivot, scale, rotate, translate, "tsr", "zyx");
+        init(mPivot, scale, rotate, translate, "srt", "zyx");
     }
 }
 
@@ -895,8 +509,8 @@ GridTransformer::init(
 
         case 'r':
             rpos = ix;
-            mTransform.accumTranslation(pivot);
-            remainder->accumTranslation(pivot);
+            mTransform.preTranslate(pivot);
+            remainder->preTranslate(pivot);
 
             int xpos, ypos, zpos;
             xpos = ypos = zpos = 3;
@@ -904,18 +518,18 @@ GridTransformer::init(
                 switch (rotOrder[ir]) {
                 case 'x':
                     xpos = ir;
-                    mTransform.accumRotation(math::X_AXIS, rotate.x());
-                    remainder->accumRotation(math::X_AXIS, rotate.x());
+                    mTransform.preRotate(math::X_AXIS, rotate.x());
+                    remainder->preRotate(math::X_AXIS, rotate.x());
                     break;
                 case 'y':
                     ypos = ir;
-                    mTransform.accumRotation(math::Y_AXIS, rotate.y());
-                    remainder->accumRotation(math::Y_AXIS, rotate.y());
+                    mTransform.preRotate(math::Y_AXIS, rotate.y());
+                    remainder->preRotate(math::Y_AXIS, rotate.y());
                     break;
                 case 'z':
                     zpos = ir;
-                    mTransform.accumRotation(math::Z_AXIS, rotate.z());
-                    remainder->accumRotation(math::Z_AXIS, rotate.z());
+                    mTransform.preRotate(math::Z_AXIS, rotate.z());
+                    remainder->preRotate(math::Z_AXIS, rotate.z());
                     break;
                 }
             }
@@ -925,26 +539,26 @@ GridTransformer::init(
                 OPENVDB_THROW(ValueError, "invalid rotation order (" + rotOrder + ")");
             }
 
-            mTransform.accumTranslation(-pivot);
-            remainder->accumTranslation(-pivot);
+            mTransform.preTranslate(-pivot);
+            remainder->preTranslate(-pivot);
             break;
 
         case 's':
             spos = ix;
-            mTransform.accumTranslation(pivot);
-            mTransform.accumScale(scale);
-            mTransform.accumTranslation(-pivot);
+            mTransform.preTranslate(pivot);
+            mTransform.preScale(scale);
+            mTransform.preTranslate(-pivot);
 
-            remainder->accumTranslation(pivot);
-            remainder->accumScale(scaleRemainder);
-            remainder->accumTranslation(-pivot);
+            remainder->preTranslate(pivot);
+            remainder->preScale(scaleRemainder);
+            remainder->preTranslate(-pivot);
             remainder = &mPreScaleTransform;
             break;
 
         case 't':
             tpos = ix;
-            mTransform.accumTranslation(translate);
-            remainder->accumTranslation(translate);
+            mTransform.preTranslate(translate);
+            remainder->preTranslate(translate);
             break;
         }
     }
@@ -984,10 +598,11 @@ GridTransformer::transformGrid(const GridT& inGrid, GridT& outGrid) const
 {
     outGrid.setBackground(inGrid.background());
 
-    if (!Sampler::prescale() || mMipLevels == Vec3i::zero()) {
+    if (!Sampler::mipmap() || mMipLevels == Vec3i::zero()) {
         // Skip the mipmapping step.
         const MatrixTransform xform(mTransform);
         applyTransform<Sampler>(xform, inGrid, outGrid);
+
     } else {
         bool firstPass = true;
         const typename GridT::ValueType background = inGrid.background();
@@ -1007,9 +622,9 @@ GridTransformer::transformGrid(const GridT& inGrid, GridT& outGrid) const
         while (count != Vec3i::zero()) {
             MatrixTransform xform;
             xform.mat.setTranslation(mPivot);
-            xform.mat.accumScale(Vec3R(
+            xform.mat.preScale(Vec3R(
                 count.x() ? .5 : 1, count.y() ? .5 : 1, count.z() ? .5 : 1));
-            xform.mat.accumTranslation(-mPivot);
+            xform.mat.preTranslate(-mPivot);
             xform.invMat = xform.mat.inverse();
 
             if (firstPass) {
@@ -1163,9 +778,9 @@ GridResampler::applyTransform(const Transformer& xform,
 
     typedef RangeProcessor<Sampler, TreeT, Transformer> RangeProc;
 
-    const bool isLevelSet = (inGrid.getGridClass() == GRID_LEVEL_SET);
+    const GridClass gridClass = inGrid.getGridClass();
 
-    if (!isLevelSet && mTransformTiles) {
+    if (gridClass != GRID_LEVEL_SET && mTransformTiles) {
         // Independently transform the tiles of the input grid.
         // Note: Tiles in level sets can only be background tiles, and they
         // are handled more efficiently with a signed flood fill (see below).
@@ -1185,7 +800,7 @@ GridResampler::applyTransform(const Transformer& xform,
     }
 
     CoordBBox clipBBox;
-    if (isLevelSet) {
+    if (gridClass == GRID_LEVEL_SET) {
         // Inactive voxels in level sets can only be background voxels, and they
         // are handled more efficiently with a signed flood fill (see below).
         clipBBox = inGrid.evalActiveVoxelBoundingBox();
@@ -1205,7 +820,10 @@ GridResampler::applyTransform(const Transformer& xform,
     }
 
     // If the grid is a level set, mark inactive voxels as inside or outside.
-    if (isLevelSet) outTree.signedFloodFill();
+    if (gridClass == GRID_LEVEL_SET) {
+        outTree.pruneInactive();
+        outTree.signedFloodFill();
+    }
 }
 
 
