@@ -178,7 +178,7 @@ public:
     typedef typename FieldT::VectorType        VectorType;
 
     /// Main constructor
-    LevelSetAdvection(GridT& grid, const FieldT& field, InterruptT& interrupt):
+    LevelSetAdvection(GridT& grid, const FieldT& field, InterruptT* interrupt = NULL):
         mTracker(grid, interrupt), mField(field),
         mSpatialScheme(math::HJWENO5_BIAS),
         mTemporalScheme(math::TVD_RK2) {}
@@ -462,7 +462,7 @@ LevelSetAdvect<MapT, SpatialScheme, TemporalScheme>::
 advect(ScalarType time0, ScalarType time1)
 {
     size_t count = 0;
-    while (time1 > time0) {
+    while (time1 > time0 && mParent.mTracker.checkInterrupter()) {
         /// Make sure we have enough temporal auxiliary buffers
         mParent.mTracker.mLeafs->rebuildAuxBuffers(TemporalScheme == math::TVD_RK3 ? 2 : 1);
 
@@ -616,7 +616,7 @@ LevelSetAdvection<GridT, FieldT, InterruptT>::
 LevelSetAdvect<MapT, SpatialScheme, TemporalScheme>::
 cook(ThreadingMode mode, size_t swapBuffer)
 {
-    mParent.mTracker.mInterrupter.start("Advecting level set");
+    mParent.mTracker.startInterrupter("Advecting level set");
 
     if (mParent.mTracker.getGrainSize()==0) {
         (*this)(mParent.mTracker.mLeafs->getRange());
@@ -625,10 +625,12 @@ cook(ThreadingMode mode, size_t swapBuffer)
     } else if (mode == PARALLEL_REDUCE) {
         tbb::parallel_reduce(mParent.mTracker.mLeafs->getRange(mParent.mTracker.getGrainSize()), *this);
     } else {
-        throw std::runtime_error("Unknown threading mode");
+        throw std::runtime_error("Undefined threading mode");
     }
 
     mParent.mTracker.mLeafs->swapLeafBuffer(swapBuffer, mParent.mTracker.getGrainSize()==0);
+
+    mParent.mTracker.endInterrupter();
 }
 
 // Forward Euler advection steps:
