@@ -43,6 +43,7 @@
 #include <tbb/parallel_for.h>
 #include <boost/bind.hpp>
 #include <boost/function.hpp>
+#include <boost/type_traits/is_floating_point.hpp>
 #include <openvdb/openvdb.h>
 #include <openvdb/Types.h>
 #include <openvdb/math/Math.h>
@@ -60,7 +61,7 @@ namespace OPENVDB_VERSION_NAME {
 namespace tools {
 
 /// @brief Performs multi-threaded interface tracking of narrow band level sets
-template<typename GridT, typename InterruptT=util::NullInterrupter>
+template<typename GridT, typename InterruptT = util::NullInterrupter>
 class LevelSetTracker
 {
 public:
@@ -71,9 +72,10 @@ public:
     typedef typename tree::LeafManager<TreeType> LeafManagerType; // leafs + buffers
     typedef typename LeafManagerType::RangeType  RangeType;
     typedef typename LeafManagerType::BufferType BufferType;
+    BOOST_STATIC_ASSERT(boost::is_floating_point<ValueType>::value);
 
     /// Main constructor
-    LevelSetTracker(GridT& grid, InterruptT& interrupt);
+    LevelSetTracker(GridT& grid, InterruptT* interrupt = NULL);
 
     /// Shallow copy constructor called by tbb::parallel_for() threads during filtering
     LevelSetTracker(const LevelSetTracker& other);
@@ -115,8 +117,10 @@ public:
 
     ValueType voxelSize() const { return mDx; }
 
-    void startInterrupter(char* msg);
-    void checkInterrupter();
+    void startInterrupter(const char* msg);
+    void endInterrupter();
+    /// @return false if the process was interrupted
+    bool checkInterrupter();
 
     const GridType& grid() const { return mGrid; }
 
@@ -132,19 +136,19 @@ protected:
 
     template<math::BiasedGradientScheme      SpatialScheme,
              math::TemporalIntegrationScheme TemporalScheme>
-    struct LevelSetNormalizer
+    struct Normalizer
     {
-        LevelSetNormalizer(LevelSetTracker& tracker): mTracker(tracker), mTask(0) {}
+        Normalizer(LevelSetTracker& tracker): mTracker(tracker), mTask(0) {}
         void normalize();
-        void operator()(const RangeType& r) const {mTask(const_cast<LevelSetNormalizer*>(this), r);}
-        typedef typename boost::function<void (LevelSetNormalizer*, const RangeType&)> FuncType;
+        void operator()(const RangeType& r) const {mTask(const_cast<Normalizer*>(this), r);}
+        typedef typename boost::function<void (Normalizer*, const RangeType&)> FuncType;
         LevelSetTracker& mTracker;
         FuncType         mTask;
         void cook(int swapBuffer=0);
         void euler1(const RangeType& range, ValueType dt, Index resultBuffer);
         void euler2(const RangeType& range, ValueType dt, ValueType alpha,
                     Index phiBuffer, Index resultBuffer);
-    }; // end of protected LevelSetNormalizer class
+    }; // end of protected Normalizer class
 
     // required to access mLeafs
     template<typename, typename, typename>
@@ -156,7 +160,7 @@ protected:
     // other hand always have to be allocated locally, since some
     // methods need them and others don't!
     LeafManagerType*  mLeafs;
-    InterruptT&       mInterrupter;
+    InterruptT*       mInterrupter;
     const ValueType   mDx;
 
 private:
@@ -184,14 +188,14 @@ private:
 }; // end of LevelSetTracker class
 
 template<typename GridT, typename InterruptT>
-LevelSetTracker<GridT, InterruptT>::LevelSetTracker(GridT& grid, InterruptT& interrupt):
+LevelSetTracker<GridT, InterruptT>::LevelSetTracker(GridT& grid, InterruptT* interrupt):
     mGrid(grid),
     mLeafs(new LeafManagerType(grid.tree())),
     mInterrupter(interrupt),
-    mDx(grid.voxelDimensions()[0]),
+    mDx(grid.voxelSize()[0]),
     mSpatialScheme(math::HJWENO5_BIAS),
     mTemporalScheme(math::TVD_RK1),
-    mNormCount(3),
+    mNormCount(static_cast<int>(LEVEL_SET_HALF_WIDTH)),
     mGrainSize(1),
     mTask(0),
     mIsMaster(true)// N.B.
@@ -200,10 +204,11 @@ LevelSetTracker<GridT, InterruptT>::LevelSetTracker(GridT& grid, InterruptT& int
          OPENVDB_THROW(RuntimeError,
              "The transform must have uniform scale for the LevelSetTracker to function");
     }
-    if ( grid.getGridClass() != GRID_LEVEL_SET ) {
+    if ( grid.getGridClass() != GRID_LEVEL_SET) {
         OPENVDB_THROW(RuntimeError,
-            "LevelSetTracker only supports level sets!\n"
-            "Hint: Grid::setGridClass(openvdb::GRID_LEVEL_SET)");
+                      "LevelSetTracker only supports level sets!\n"
+                      "However, only level sets are guaranteed to work!\n"
+                      "Hint: Grid::setGridClass(openvdb::GRID_LEVEL_SET)");
     }
 }
 
@@ -240,6 +245,7 @@ LevelSetTracker<GridT, InterruptT>::prune()
 
     // The tree topology has changes so rebuild the list of leafs
     mLeafs->rebuildLeafs();
+    this->endInterrupter();
 }
 
 template<typename GridT, typename InterruptT>
@@ -260,16 +266,27 @@ LevelSetTracker<GridT, InterruptT>::track()
 
 template<typename GridT,  typename InterruptT>
 inline void
-LevelSetTracker<GridT, InterruptT>::startInterrupter(char* msg)
+LevelSetTracker<GridT, InterruptT>::startInterrupter(const char* msg)
 {
-    mInterrupter.start(msg);
+    if (mInterrupter) mInterrupter->start(msg);
 }
 
 template<typename GridT,  typename InterruptT>
 inline void
+LevelSetTracker<GridT, InterruptT>::endInterrupter()
+{
+    if (mInterrupter) mInterrupter->end();
+}    
+
+template<typename GridT,  typename InterruptT>
+inline bool
 LevelSetTracker<GridT, InterruptT>::checkInterrupter()
 {
-    if (mInterrupter.wasInterrupted()) tbb::task::self().cancel_group_execution();
+    if (util::wasInterrupted(mInterrupter)) {
+        tbb::task::self().cancel_group_execution();
+        return false;
+    }
+    return true;
 }
 
 /// Prunes away voxels that have moved outside the narrow band
@@ -335,7 +352,7 @@ template<math::BiasedGradientScheme SpatialScheme,
 inline void
 LevelSetTracker<GridT, InterruptT>::normalize2()
 {
-    LevelSetNormalizer<SpatialScheme, TemporalScheme> tmp(*this);
+    Normalizer<SpatialScheme, TemporalScheme> tmp(*this);
     tmp.normalize();
 }
 
@@ -343,7 +360,7 @@ template<typename GridT, typename InterruptT>
 template<math::BiasedGradientScheme SpatialScheme,
          math::TemporalIntegrationScheme TemporalScheme>
 inline void
-LevelSetTracker<GridT,InterruptT>::LevelSetNormalizer<SpatialScheme, TemporalScheme>::
+LevelSetTracker<GridT,InterruptT>::Normalizer<SpatialScheme, TemporalScheme>::
 normalize()
 {
     /// Make sure we have enough temporal auxiliary buffers
@@ -360,7 +377,7 @@ normalize()
             //std::cerr << "1";
             // Perform one explicit Euler step: t1 = t0 + dt
             // Phi_t1(0) = Phi_t0(0) - dt * VdotG_t0(1)
-            mTask = boost::bind(&LevelSetNormalizer::euler1, _1, _2, dt, /*result=*/1);
+            mTask = boost::bind(&Normalizer::euler1, _1, _2, dt, /*result=*/1);
             // Cook and swap buffer 0 and 1 such that Phi_t1(0) and Phi_t0(1)
             this->cook(1);
             break;
@@ -368,13 +385,13 @@ normalize()
             //std::cerr << "2";
             // Perform one explicit Euler step: t1 = t0 + dt
             // Phi_t1(1) = Phi_t0(0) - dt * VdotG_t0(1)
-            mTask = boost::bind(&LevelSetNormalizer::euler1, _1, _2, dt, /*result=*/1);
+            mTask = boost::bind(&Normalizer::euler1, _1, _2, dt, /*result=*/1);
             // Cook and swap buffer 0 and 1 such that Phi_t1(0) and Phi_t0(1)
             this->cook(1);
 
             // Convex combine explict Euler step: t2 = t0 + dt
             // Phi_t2(1) = 1/2 * Phi_t0(1) + 1/2 * (Phi_t1(0) - dt * V.Grad_t1(0))
-            mTask = boost::bind(&LevelSetNormalizer::euler2,
+            mTask = boost::bind(&Normalizer::euler2,
                 _1, _2, dt, ValueType(0.5), /*phi=*/1, /*result=*/1);
             // Cook and swap buffer 0 and 1 such that Phi_t2(0) and Phi_t1(1)
             this->cook(1);
@@ -383,20 +400,20 @@ normalize()
             //std::cerr << "3";
             // Perform one explicit Euler step: t1 = t0 + dt
             // Phi_t1(1) = Phi_t0(0) - dt * VdotG_t0(1)
-            mTask = boost::bind(&LevelSetNormalizer::euler1, _1, _2, dt, /*result=*/1);
+            mTask = boost::bind(&Normalizer::euler1, _1, _2, dt, /*result=*/1);
             // Cook and swap buffer 0 and 1 such that Phi_t1(0) and Phi_t0(1)
             this->cook(1);
 
             // Convex combine explict Euler step: t2 = t0 + dt/2
             // Phi_t2(2) = 3/4 * Phi_t0(1) + 1/4 * (Phi_t1(0) - dt * V.Grad_t1(0))
-            mTask = boost::bind(&LevelSetNormalizer::euler2,
+            mTask = boost::bind(&Normalizer::euler2,
                 _1, _2, dt, ValueType(0.75), /*phi=*/1, /*result=*/2);
             // Cook and swap buffer 0 and 2 such that Phi_t2(0) and Phi_t1(2)
             this->cook(2);
 
             // Convex combine explict Euler step: t3 = t0 + dt
             // Phi_t3(2) = 1/3 * Phi_t0(1) + 2/3 * (Phi_t2(0) - dt * V.Grad_t2(0)
-            mTask = boost::bind(&LevelSetNormalizer::euler2,
+            mTask = boost::bind(&Normalizer::euler2,
                 _1, _2, dt, ValueType(1.0/3.0), /*phi=*/1, /*result=*/2);
             // Cook and swap buffer 0 and 2 such that Phi_t3(0) and Phi_t2(2)
             this->cook(2);
@@ -414,7 +431,7 @@ template<typename GridT, typename InterruptT>
 template<math::BiasedGradientScheme      SpatialScheme,
          math::TemporalIntegrationScheme TemporalScheme>
 inline void
-LevelSetTracker<GridT,InterruptT>::LevelSetNormalizer<SpatialScheme, TemporalScheme>::
+LevelSetTracker<GridT,InterruptT>::Normalizer<SpatialScheme, TemporalScheme>::
 cook(int swapBuffer)
 {
     mTracker.startInterrupter("Normalizing Level Set");
@@ -426,6 +443,8 @@ cook(int swapBuffer)
     }
 
     mTracker.mLeafs->swapLeafBuffer(swapBuffer, mTracker.getGrainSize()==0);
+
+    mTracker.endInterrupter();
 }
 
 
@@ -436,7 +455,7 @@ template<typename GridT, typename InterruptT>
 template<math::BiasedGradientScheme      SpatialScheme,
          math::TemporalIntegrationScheme TemporalScheme>
 inline void
-LevelSetTracker<GridT,InterruptT>::LevelSetNormalizer<SpatialScheme, TemporalScheme>::
+LevelSetTracker<GridT,InterruptT>::Normalizer<SpatialScheme, TemporalScheme>::
 euler1(const RangeType &range, ValueType dt, Index resultBuffer)
 {
     typedef math::BIAS_SCHEME<SpatialScheme>                             Scheme;
@@ -464,7 +483,7 @@ template<typename GridT, typename InterruptT>
 template<math::BiasedGradientScheme      SpatialScheme,
          math::TemporalIntegrationScheme TemporalScheme>
 inline void
-LevelSetTracker<GridT,InterruptT>::LevelSetNormalizer<SpatialScheme, TemporalScheme>::
+LevelSetTracker<GridT,InterruptT>::Normalizer<SpatialScheme, TemporalScheme>::
 euler2(const RangeType& range, ValueType dt, ValueType alpha, Index phiBuffer, Index resultBuffer)
 {
     typedef math::BIAS_SCHEME<SpatialScheme>                             Scheme;
