@@ -100,7 +100,8 @@ namespace tools {
 /// the util::NullInterrupter is used which implies that all
 /// interrupter calls are no-ops (i.e. incurs no computational overhead).
 ///
-
+    
+namespace local {
 /// Trait class needed to merge and split a distance and a particle id required
 /// during attribute transfer. The default implementation of merge simply
 /// ignores the particle id. A specialized implementation is given
@@ -111,6 +112,7 @@ struct DualTrait
     static T merge(T dist, Index32) { return dist; }
     static T split(T dist) { return dist; }
 };
+}// namespace local    
 
 template<typename GridT,
          typename ParticleListT,
@@ -133,13 +135,13 @@ public:
     /// compile out interruption checks during optimization, thus
     /// incurring no run-time overhead.
     ///
-    ParticlesToLevelSet(GridT& grid, InterruptT& interrupt) :
+    ParticlesToLevelSet(GridT& grid, InterruptT* interrupt = NULL) :
         mGrid(&grid),
         mPa(NULL),
-        mDx(grid.transform().voxelDimensions()[0]),
-        mHalfWidth(DualTrait<ValueT>::split(grid.background()) / mDx),
+        mDx(grid.transform().voxelSize()[0]),
+        mHalfWidth(local::DualTrait<ValueT>::split(grid.background()) / mDx),
         mRmin(1.5),// corresponds to the Nyquist grid sampling frequency
-        mRmax(100.0f),// corresponds to a huge particle (probably too huge!)
+        mRmax(100.0f),// corresponds to a huge particle (probably too large!)
         mGrainSize(1),
         mInterrupter(interrupt),
         mMinCount(0),
@@ -150,7 +152,7 @@ public:
             OPENVDB_THROW(RuntimeError,
                 "The transform must have uniform scale for ParticlesToLevelSet to function!");
         }
-        if ( mGrid->getGridClass() != GRID_LEVEL_SET ) {
+        if (mGrid->getGridClass() != GRID_LEVEL_SET) {
             OPENVDB_THROW(RuntimeError,
                 "ParticlesToLevelSet only supports level sets!"
                 "\nUse Grid::setGridClass(openvdb::GRID_LEVEL_SET)");
@@ -212,9 +214,10 @@ public:
     void rasterizeSpheres(const ParticleListT& pa)
     {
         mPa = &pa;
-        mInterrupter.start("Rasterizing particles to level set using spheres");
+        if (mInterrupter) mInterrupter->start("Rasterizing particles to level set using spheres");
         mTask = boost::bind(&ParticlesToLevelSet::rasterSpheres, _1, _2);
         this->cook();
+        if (mInterrupter) mInterrupter->end();
     }
 
     /// @brief Rasterize a trail per particle derived from their
@@ -235,9 +238,10 @@ public:
     void rasterizeTrails(const ParticleListT& pa, Real delta=1.0)
     {
         mPa = &pa;
-        mInterrupter.start("Rasterizing particles to level set using trails");
+        if (mInterrupter) mInterrupter->start("Rasterizing particles to level set using trails");
         mTask = boost::bind(&ParticlesToLevelSet::rasterTrails, _1, _2, RealT(delta));
         this->cook();
+        if (mInterrupter) mInterrupter->end();
     }
 
     //  ========> DO NOT CALL ANY OF THE PUBLIC METHODS BELOW! <=============
@@ -276,13 +280,12 @@ private:
     RealT                 mRmax;//ignore particles larger than this radius in voxels
     int                   mGrainSize;
     FuncType              mTask;
-    InterruptT&           mInterrupter;
+    InterruptT*           mInterrupter;
     size_t                mMinCount, mMaxCount;//counters for ignored particles!
     const bool            mIsSlave;
 
     void cook()
     {
-        mInterrupter.start("Rasterizing particles into a level set");
         if (mGrainSize>0) {
             tbb::parallel_reduce(tbb::blocked_range<size_t>(0,mPa->size(),mGrainSize), *this);
         } else {
@@ -319,10 +322,6 @@ private:
     /// outside the surface an negative distances inside the surface.
     inline bool rasterSphere(const Vec3R &P, RealT R, Index32 id, Accessor& accessor)
     {
-        if (mInterrupter.wasInterrupted()) {
-            tbb::task::self().cancel_group_execution();
-            return false;
-        }
         const ValueT inside = -mGrid->background();
         const RealT dx = mDx;
         const RealT max = R + mHalfWidth;// maximum distance in voxel units
@@ -331,18 +330,26 @@ private:
         const RealT max2 = math::Pow2(max);//square of maximum distance in voxel units
         const RealT min2 = math::Pow2(math::Max(RealT(0), R - mHalfWidth));//square of minimum distance
         ValueT v;
+        size_t count = 0;
         for ( Coord c = a; c.x() <= b.x(); ++c.x() ) {
+            //only check interrupter every 32'th scan in x
+            if (!(count++ & (1<<5)-1) && util::wasInterrupted(mInterrupter)) {
+                tbb::task::self().cancel_group_execution();
+                return false;
+            }
             RealT x2 = math::Pow2( c.x() - P[0] );
             for ( c.y() = a.y(); c.y() <= b.y(); ++c.y() ) {
                 RealT x2y2 = x2 + math::Pow2( c.y() - P[1] );
                 for ( c.z() = a.z(); c.z() <= b.z(); ++c.z() ) {
                     RealT x2y2z2 = x2y2 + math::Pow2( c.z() - P[2] );//square distance from c to P
-                    if ( x2y2z2 >= max2 || (!accessor.probeValue(c,v) && v<=inside) ) continue;//outside pa or inside ls
-                    if ( x2y2z2 <= min2 ) {//inside particle
+                    if ( x2y2z2 >= max2 || (!accessor.probeValue(c,v) && v<ValueT(0)) )
+                        continue;//outside narrow band of the particle or inside existing ls
+                    if ( x2y2z2 <= min2 ) {//inside narrowband of the particle.
                         accessor.setValueOff(c, inside);
                         continue;
                     }
-                    const ValueT d = DualTrait<ValueT>::merge(dx*(math::Sqrt(x2y2z2) - R), id);// distance in world units
+                    // distance in world units
+                    const ValueT d = local::DualTrait<ValueT>::merge(dx*(math::Sqrt(x2y2z2) - R), id);
                     if (d < v) accessor.setValue(c, d);//CSG union
                 }//end loop over z
             }//end loop over y
@@ -402,10 +409,10 @@ private:
 };//end of ParticlesToLevelSet class
 
 ///////////////////// YOU CAN SAFELY IGNORE THIS SECTION /////////////////////
-
+namespace local {
 // This is a simple type that combines a distance value and a particle
-// id. It's required for during attribute transfer which is defined
-// in the ParticlesToLevelSetAndId class below.
+// id. It's required for attribute transfer which is defined in the
+// ParticlesToLevelSetAndId class below.
 template <typename RealT>
 class Dual
 {
@@ -450,7 +457,7 @@ struct DualTrait<Dual<T> >
     static Dual<T> merge(T dist, Index32 id) { return Dual<T>(dist, id); }
     static T split(Dual<T> dual) { return dual.dist(); }
 };
-
+}// local namespace
 //////////////////////////////////////////////////////////////////////////////
 
 /// @brief Use this wrapper class to convert particles into a level set and a
@@ -461,19 +468,19 @@ struct DualTrait<Dual<T> >
 /// only exception being the raster methods that return the index grid!
 template<typename LevelSetGridT,
          typename ParticleListT,
-         typename InterruptT=util::NullInterrupter>
+         typename InterruptT = util::NullInterrupter>
 class ParticlesToLevelSetAndId
 {
 public:
     typedef typename LevelSetGridT::ValueType  RealT;
-    typedef Dual<RealT>                        DualT;
+    typedef typename local::Dual<RealT>        DualT;
     typedef typename LevelSetGridT::TreeType   RealTreeT;
-    typedef typename RealTreeT::template ValueConverter<DualT>::Type   DualTreeT;
-    typedef Int32Tree                     IndxTreeT;
-    typedef Grid<DualTreeT>               DualGridT;
-    typedef Int32Grid                     IndxGridT;
+    typedef typename RealTreeT::template ValueConverter<DualT>::Type DualTreeT;
+    typedef Int32Tree                          IndxTreeT;
+    typedef Grid<DualTreeT>                    DualGridT;
+    typedef Int32Grid                          IndxGridT;
 
-    ParticlesToLevelSetAndId(LevelSetGridT& ls, InterruptT& interrupter) :
+    ParticlesToLevelSetAndId(LevelSetGridT& ls, InterruptT* interrupter = NULL) :
         mRealGrid(ls),
         mDualGrid(DualT(ls.background()))
     {
@@ -545,7 +552,7 @@ public:
 
 private:
 
-    /// disallow copy construction;
+    /// disallow copy construction
     ParticlesToLevelSetAndId(const ParticlesToLevelSetAndId& other) {}
     /// disallow copy assignment
     ParticlesToLevelSetAndId& operator=(const ParticlesToLevelSetAndId& rhs)
