@@ -37,6 +37,7 @@
 
 #include <boost/shared_array.hpp>
 #include <boost/static_assert.hpp>
+#include <openvdb/Platform.h>
 #include <openvdb/util/NodeMasks.h>
 #include <openvdb/io/Compression.h> // for io::readData(), etc.
 #include <openvdb/math/Math.h> // for Abs(), isExactlyEqual()
@@ -45,7 +46,6 @@
 #include "Iterator.h"
 #include "NodeUnion.h"
 #include "Util.h"
-#include <openvdb/Platform.h>
 
 
 namespace openvdb {
@@ -70,7 +70,7 @@ public:
         NUM_VALUES   = 1 << (3 * Log2Dim),
         LEVEL        = 1 + ChildNodeType::LEVEL; // level 0 = leaf
     static const Index64
-    NUM_VOXELS   = uint64_t(1) << (3 * TOTAL); // total number of voxels represented by this node
+        NUM_VOXELS   = uint64_t(1) << (3 * TOTAL); // total # of voxels represented by this node
 
     /// @brief ValueConverter<T>::Type is the type of an InternalNode having the same
     /// child hierarchy and dimensions as this node but a different value type, T.
@@ -231,14 +231,14 @@ public:
 
     /// Return the total amount of memory in bytes occupied by this node and its children.
     Index64 memUsage() const;
-    
-    /// Expand the specified bbox so it includes the active tiles of
-    /// this internal node as well as all the active values in its child nodes.
+
+    /// @brief Expand the specified bounding box so that it includes the active tiles
+    /// of this internal node as well as all the active values in its child nodes.
     void evalActiveVoxelBoundingBox(CoordBBox& bbox) const;
 
-    /// @return the bounding box of this node, i.e. the full index space
+    /// @brief Return the bounding box of this node, i.e., the full index space
     /// spanned by the node regardless of its content.
-    CoordBBox getNodeBoundingBox() const { return CoordBBox(mOrigin, DIM); }
+    CoordBBox getNodeBoundingBox() const { return CoordBBox::createCube(mOrigin, DIM); }
 
     bool isEmpty() const { return mChildMask.isOff(); }
 
@@ -354,13 +354,13 @@ public:
     /// in @a on and @a level, respectively.
     /// @note Used internally by ValueAccessor.
     template<typename AccessorT>
-    const ValueType& getValue(const Coord& xyz, bool& on, int& level, AccessorT&) const;
+    OPENVDB_DEPRECATED const ValueType& getValue(const Coord& xyz, bool& on, int& level, AccessorT&) const;
 
     template<typename ProbeType, typename AccessorT>
-    const ValueType& probe(const Coord& xyz, ProbeType& p, AccessorT&) const;
+    OPENVDB_DEPRECATED const ValueType& probe(const Coord& xyz, ProbeType& p, AccessorT&) const;
 
     template<bool State, bool Level, typename AccessorT>
-    const ValueType& probe(const Coord& xyz, bool& state, int& level, AccessorT&) const;
+    OPENVDB_DEPRECATED const ValueType& probe(const Coord& xyz, bool& state, int& level, AccessorT&) const;
 
     /// @brief Return the level of the tree (0 = leaf) at which the value
     /// at the given coordinates resides.
@@ -378,7 +378,7 @@ public:
     /// from the root node to the node containing voxel (x, y, z).
     /// @note Used internally by ValueAccessor.
     template<typename AccessorT>
-    void updateCache(const Coord& xyz, AccessorT&) const;
+    OPENVDB_DEPRECATED void updateCache(const Coord& xyz, AccessorT&) const;
 
     //
     // I/O
@@ -395,7 +395,7 @@ public:
 
     void negate();
 
-    /// Turns active tiles into dense voxels, i.e. active leaf nodes
+    /// Replace active tiles with dense voxels, i.e., with active leaf nodes.
     void voxelizeActiveTiles();
 
     /// @brief Simple merge: Nodes and values of this node are always unchanged!
@@ -448,7 +448,7 @@ public:
     template<typename IterT, typename VisitorOp>
     void visit2(IterT& otherIter, VisitorOp&, bool otherIsLHS = false) const;
 
-    /// Call the @c PruneOp functor for each child node and, if the functor
+    /// @brief Call the @c PruneOp functor for each child node and, if the functor
     /// returns @c true, prune the node and replace it with a tile.
     ///
     /// This method is used to implement all of the various pruning algorithms
@@ -469,17 +469,20 @@ public:
     /// background tiles any nodes whose values are all inactive.
     void pruneInactive();
 
-    /// @brief @return the leaf node that contains voxel (x, y, z) and
-    /// if it doesn't exist, create it, but preserve the values and
+    /// @brief Return the leaf node that contains voxel (x, y, z).
+    /// If no such node exists, create one, but preserve the values and
     /// active states of all voxels.
     ///
-    /// Use this method to preallocate a static tree topology over which to
-    /// safely perform multithreaded processing.
+    /// @details Use this method to preallocate a static tree topology
+    /// over which to safely perform multithreaded processing.
     LeafNodeType* touchLeaf(const Coord& xyz);
 
-     /// @brief @return the leaf node that contains voxel (x, y, z) and
-    /// if it doesn't exist, return NULL
+    /// @brief Return a pointer to the leaf node that contains voxel (x, y, z).
+    /// If no such node exists, return NULL.
     LeafNodeType* probeLeaf(const Coord& xyz);
+    /// @brief Return a const pointer to the leaf node that contains voxel (x, y, z).
+    /// If no such node exists, return NULL.
+    const LeafNodeType* probeConstLeaf(const Coord& xyz) const;
 
     /// @brief Same as touchLeaf except, if necessary, it update the accessor with pointers
     /// to the nodes along the path from the root node to the node containing the coordinate.
@@ -490,6 +493,10 @@ public:
     /// to the nodes along the path from the root node to the node containing the coordinate.
     template<typename AccessorT>
     LeafNodeType* probeLeafAndCache(const Coord& xyz, AccessorT&);
+    /// @brief Same as probeConstLeaf except, if necessary, it update the accessor with pointers
+    /// to the nodes along the path from the root node to the node containing the coordinate.
+    template<typename AccessorT>
+    const LeafNodeType* probeConstLeafAndCache(const Coord& xyz, AccessorT&) const;
 
     void resetBackground(const ValueType& oldBackground, const ValueType& newBackground);
 
@@ -755,6 +762,7 @@ InternalNode<ChildT, Log2Dim>::evalActiveVoxelBoundingBox(CoordBBox& bbox) const
 
 ////////////////////////////////////////
 
+
 template<typename ChildT, Index Log2Dim>
 template<typename PruneOp>
 inline void
@@ -834,6 +842,14 @@ InternalNode<ChildT, Log2Dim>::probeLeaf(const Coord& xyz)
     return mChildMask.isOn(n) ? mNodes[n].getChild()->probeLeaf(xyz) : NULL;
 }
 
+template<typename ChildT, Index Log2Dim>
+inline const typename ChildT::LeafNodeType*
+InternalNode<ChildT, Log2Dim>::probeConstLeaf(const Coord& xyz) const
+{
+    const Index n = this->coord2offset(xyz);
+    return mChildMask.isOn(n) ? mNodes[n].getChild()->probeConstLeaf(xyz) : NULL;
+}
+
 template<typename ChildT, Index Log2Dim>
 template<typename AccessorT>
 inline typename ChildT::LeafNodeType*
@@ -845,6 +861,18 @@ InternalNode<ChildT, Log2Dim>::probeLeafAndCache(const Coord& xyz, AccessorT& ac
     return mNodes[n].getChild()->probeLeafAndCache(xyz,acc);
 }
 
+template<typename ChildT, Index Log2Dim>
+template<typename AccessorT>
+inline const typename ChildT::LeafNodeType*
+InternalNode<ChildT, Log2Dim>::probeConstLeafAndCache(const Coord& xyz, AccessorT& acc) const
+{
+    const Index n = this->coord2offset(xyz);
+    if (mChildMask.isOff(n)) return NULL;
+    acc.insert(xyz, mNodes[n].getChild());
+    return mNodes[n].getChild()->probeConstLeafAndCache(xyz,acc);
+}
+
+
 ////////////////////////////////////////
 
 
@@ -1019,6 +1047,7 @@ InternalNode<ChildT, Log2Dim>::getValue(const Coord& xyz, bool& state, int& leve
     return mNodes[n].getValue();
 }
 
+
 template<typename ChildT, Index Log2Dim>
 template<typename ProbeT, typename AccessorT>
 inline const typename ChildT::ValueType&
@@ -1059,8 +1088,8 @@ InternalNode<ChildT, Log2Dim>::setValueOff(const Coord& xyz)
     if (!hasChild && this->isValueMaskOn(n)) {
         // If the voxel belongs to a constant tile that is active,
         // a child subtree must be constructed.
-        mChildMask.setOn(n);//  we're adding a child node so set the mask on
-        mValueMask.setOff(n);// value mask is always off if child mask is on
+        mChildMask.setOn(n); // we're adding a child node so set the mask on
+        mValueMask.setOff(n); // value mask is always off if child mask is on
         hasChild = true;
         mNodes[n].setChild(new ChildNodeType(xyz, mNodes[n].getValue(), /*active=*/true));
     }
@@ -1077,8 +1106,8 @@ InternalNode<ChildT, Log2Dim>::setValueOn(const Coord& xyz)
     if (!hasChild && !this->isValueMaskOn(n)) {
         // If the voxel belongs to a constant tile that is inactive,
         // a child subtree must be constructed.
-        mChildMask.setOn(n);//  we're adding a child node so set the mask on
-        mValueMask.setOff(n);// value mask is always off if child mask is on
+        mChildMask.setOn(n); // we're adding a child node so set the mask on
+        mValueMask.setOff(n); // value mask is always off if child mask is on
         hasChild = true;
         mNodes[n].setChild(new ChildNodeType(xyz, mNodes[n].getValue(), /*active=*/false));
     }
@@ -1098,8 +1127,8 @@ InternalNode<ChildT, Log2Dim>::setValueOff(const Coord& xyz, const ValueType& va
             // If the voxel belongs to a tile that is either active or that
             // has a constant value that is different from the one provided,
             // a child subtree must be constructed.
-            mChildMask.setOn(n);//  we're adding a child node so set the mask on
-            mValueMask.setOff(n);// value mask is always off if child mask is on
+            mChildMask.setOn(n); // we're adding a child node so set the mask on
+            mValueMask.setOff(n); // value mask is always off if child mask is on
             hasChild = true;
             mNodes[n].setChild(new ChildNodeType(xyz, mNodes[n].getValue(), active));
         }
@@ -1121,8 +1150,8 @@ InternalNode<ChildT, Log2Dim>::setValueOffAndCache(const Coord& xyz,
             // If the voxel belongs to a tile that is either active or that
             // has a constant value that is different from the one provided,
             // a child subtree must be constructed.
-            mChildMask.setOn(n);//  we're adding a child node so set the mask on
-            mValueMask.setOff(n);// value mask is always off if child mask is on
+            mChildMask.setOn(n); // we're adding a child node so set the mask on
+            mValueMask.setOff(n); // value mask is always off if child mask is on
             hasChild = true;
             mNodes[n].setChild(new ChildNodeType(xyz, mNodes[n].getValue(), active));
         }
@@ -1147,8 +1176,8 @@ InternalNode<ChildT, Log2Dim>::setValueOn(const Coord& xyz, const ValueType& val
             // If the voxel belongs to a tile that is either inactive or that
             // has a constant value that is different from the one provided,
             // a child subtree must be constructed.
-            mChildMask.setOn(n);//  we're adding a child node so set the mask on
-            mValueMask.setOff(n);// value mask is always off if child mask is on
+            mChildMask.setOn(n); // we're adding a child node so set the mask on
+            mValueMask.setOff(n); // value mask is always off if child mask is on
             hasChild = true;
             mNodes[n].setChild(new ChildNodeType(xyz, mNodes[n].getValue(), active));
         }
@@ -1156,24 +1185,6 @@ InternalNode<ChildT, Log2Dim>::setValueOn(const Coord& xyz, const ValueType& val
     if (hasChild) mNodes[n].getChild()->setValueOn(xyz, value);
 }
 
-template<typename ChildT, Index Log2Dim>
-inline void
-InternalNode<ChildT, Log2Dim>::setValueOnly(const Coord& xyz, const ValueType& value)
-{
-    const Index n = this->coord2offset(xyz);
-    bool hasChild = this->isChildMaskOn(n);
-    if (!hasChild && !math::isExactlyEqual(mNodes[n].getValue(), value)) {
-        // If the voxel has a tile value that is different from the one provided,
-        // a child subtree must be constructed.
-        const bool active = this->isValueMaskOn(n);
-        mChildMask.setOn(n);//  we're adding a child node so set the mask on
-        mValueMask.setOff(n);// value mask is always off if child mask is on
-        hasChild = true;
-        mNodes[n].setChild(new ChildNodeType(xyz, mNodes[n].getValue(), active));
-    }
-    if (hasChild) mNodes[n].getChild()->setValueOnly(xyz, value);
-}
-
 template<typename ChildT, Index Log2Dim>
 template<typename AccessorT>
 inline void
@@ -1188,8 +1199,8 @@ InternalNode<ChildT, Log2Dim>::setValueAndCache(const Coord& xyz,
             // If the voxel belongs to a tile that is either inactive or that
             // has a constant value that is different from the one provided,
             // a child subtree must be constructed.
-            mChildMask.setOn(n);//  we're adding a child node so set the mask on
-            mValueMask.setOff(n);// value mask is always off if child mask is on
+            mChildMask.setOn(n); // we're adding a child node so set the mask on
+            mValueMask.setOff(n); // value mask is always off if child mask is on
             hasChild = true;
             mNodes[n].setChild(new ChildNodeType(xyz, mNodes[n].getValue(), active));
         }
@@ -1200,6 +1211,25 @@ InternalNode<ChildT, Log2Dim>::setValueAndCache(const Coord& xyz,
     }
 }
 
+
+template<typename ChildT, Index Log2Dim>
+inline void
+InternalNode<ChildT, Log2Dim>::setValueOnly(const Coord& xyz, const ValueType& value)
+{
+    const Index n = this->coord2offset(xyz);
+    bool hasChild = this->isChildMaskOn(n);
+    if (!hasChild && !math::isExactlyEqual(mNodes[n].getValue(), value)) {
+        // If the voxel has a tile value that is different from the one provided,
+        // a child subtree must be constructed.
+        const bool active = this->isValueMaskOn(n);
+        mChildMask.setOn(n); // we're adding a child node so set the mask on
+        mValueMask.setOff(n); // value mask is always off if child mask is on
+        hasChild = true;
+        mNodes[n].setChild(new ChildNodeType(xyz, mNodes[n].getValue(), active));
+    }
+    if (hasChild) mNodes[n].getChild()->setValueOnly(xyz, value);
+}
+
 template<typename ChildT, Index Log2Dim>
 template<typename AccessorT>
 inline void
@@ -1212,8 +1242,8 @@ InternalNode<ChildT, Log2Dim>::setValueOnlyAndCache(const Coord& xyz,
         // If the voxel has a tile value that is different from the one provided,
         // a child subtree must be constructed.
         const bool active = this->isValueMaskOn(n);
-        mChildMask.setOn(n);//  we're adding a child node so set the mask on
-        mValueMask.setOff(n);// value mask is always off if child mask is on
+        mChildMask.setOn(n); // we're adding a child node so set the mask on
+        mValueMask.setOff(n); // value mask is always off if child mask is on
         hasChild = true;
         mNodes[n].setChild(new ChildNodeType(xyz, mNodes[n].getValue(), active));
     }
@@ -1223,6 +1253,7 @@ InternalNode<ChildT, Log2Dim>::setValueOnlyAndCache(const Coord& xyz,
     }
 }
 
+
 template<typename ChildT, Index Log2Dim>
 inline void
 InternalNode<ChildT, Log2Dim>::setActiveState(const Coord& xyz, bool on)
@@ -1233,8 +1264,8 @@ InternalNode<ChildT, Log2Dim>::setActiveState(const Coord& xyz, bool on)
         if (on != this->isValueMaskOn(n)) {
             // If the voxel belongs to a tile with the wrong active state,
             // then a child subtree must be constructed.
-            mChildMask.setOn(n);//  we're adding a child node so set the mask on
-            mValueMask.setOff(n);// value mask is always off if child mask is on
+            mChildMask.setOn(n); // we're adding a child node so set the mask on
+            mValueMask.setOff(n); // value mask is always off if child mask is on
             mNodes[n].setChild(new ChildNodeType(xyz, mNodes[n].getValue(), !on));
                 // 'on' is the voxel's new state, therefore '!on' is the tile's current state
             hasChild = true;
@@ -1254,8 +1285,8 @@ InternalNode<ChildT, Log2Dim>::setActiveStateAndCache(const Coord& xyz, bool on,
         if (on != this->isValueMaskOn(n)) {
             // If the voxel belongs to a tile with the wrong active state,
             // then a child subtree must be constructed.
-            mChildMask.setOn(n);//  we're adding a child node so set the mask on
-            mValueMask.setOff(n);// value mask is always off if child mask is on
+            mChildMask.setOn(n); // we're adding a child node so set the mask on
+            mValueMask.setOff(n); // value mask is always off if child mask is on
             mNodes[n].setChild(new ChildNodeType(xyz, mNodes[n].getValue(), !on));
                 // 'on' is the voxel's new state, therefore '!on' is the tile's current state
             hasChild = true;
@@ -1292,8 +1323,8 @@ InternalNode<ChildT, Log2Dim>::setValueOnMin(const Coord& xyz, const ValueType&
             // If the voxel belongs to a tile that is either inactive or that
             // has a constant value that is greater than the one provided,
             // a child subtree must be constructed.
-            mChildMask.setOn(n);//  we're adding a child node so set the mask on
-            mValueMask.setOff(n);// value mask is always off if child mask is on
+            mChildMask.setOn(n); // we're adding a child node so set the mask on
+            mValueMask.setOff(n); // value mask is always off if child mask is on
             hasChild = true;
             mNodes[n].setChild(new ChildNodeType(xyz, mNodes[n].getValue(), active));
         }
@@ -1314,8 +1345,8 @@ InternalNode<ChildT, Log2Dim>::setValueOnMax(const Coord& xyz, const ValueType&
             // If the voxel belongs to a tile that is either inactive or that
             // has a constant value that is less than the one provided,
             // a child subtree must be constructed.
-            mChildMask.setOn(n);//  we're adding a child node so set the mask on
-            mValueMask.setOff(n);// value mask is always off if child mask is on
+            mChildMask.setOn(n); // we're adding a child node so set the mask on
+            mValueMask.setOff(n); // value mask is always off if child mask is on
             hasChild = true;
             mNodes[n].setChild(new ChildNodeType(xyz, mNodes[n].getValue(), active));
         }
@@ -1357,8 +1388,8 @@ InternalNode<ChildT, Log2Dim>::setValueOnSumAndCache(const Coord& xyz,
         if (!active || !math::isExactlyEqual(addend, zeroVal<ValueType>())) {
             // If the voxel belongs to a tile that is inactive or if
             // the addend is nonzero, a child subtree must be constructed.
-            mChildMask.setOn(n);//  we're adding a child node so set the mask on
-            mValueMask.setOff(n);// value mask is always off if child mask is on
+            mChildMask.setOn(n); // we're adding a child node so set the mask on
+            mValueMask.setOff(n); // value mask is always off if child mask is on
             hasChild = true;
             mNodes[n].setChild(new ChildNodeType(xyz, mNodes[n].getValue(), active));
         }
@@ -1448,23 +1479,15 @@ InternalNode<ChildT, Log2Dim>::writeTopology(std::ostream& os, bool toHalf) cons
     mChildMask.save(os);
     mValueMask.save(os);
 
-    const Index numValues = mChildMask.countOff();
     {
-        // Copy all of this node's values into a contiguous array.
-        boost::shared_array<ValueType> values(new ValueType[numValues]);
-        Index n = 0;
-        for (ValueAllCIter iter = this->cbeginValueAll(); iter; ++iter) {
-            values[n++] = *iter;
+        // Copy all of this node's values into an array.
+        boost::shared_array<ValueType> values(new ValueType[NUM_VALUES]);
+        const ValueType zero = zeroVal<ValueType>();
+        for (Index i = 0; i < NUM_VALUES; ++i) {
+            values[i] = (mChildMask.isOff(i) ? mNodes[i].getValue() : zero);
         }
-        assert(n == numValues);
         // Compress (optionally) and write out the contents of the array.
-        const bool compress = io::getDataCompression(os);
-        if (toHalf) {
-            io::HalfWriter<io::RealToHalf<ValueType>::isReal, ValueType>::write(
-                os, values.get(), numValues, compress);
-        } else {
-            io::writeData<ValueType>(os, values.get(), numValues, compress);
-        }
+        io::writeCompressedValues(os, values.get(), NUM_VALUES, mValueMask, mChildMask, toHalf);
     }
     // Write out the child nodes in order.
     for (ChildOnCIter iter = this->cbeginChildOn(); iter; ++iter) {
@@ -1494,24 +1517,27 @@ InternalNode<ChildT, Log2Dim>::readTopology(std::istream& is, bool fromHalf)
             }
         }
     } else {
-        const Index numValues = mChildMask.countOff();
+        const bool oldVersion =
+            (io::getFormatVersion(is) < OPENVDB_FILE_VERSION_NODE_MASK_COMPRESSION);
+        const Index numValues = (oldVersion ? mChildMask.countOff() : NUM_VALUES);
         {
             // Read in (and uncompress, if necessary) all of this node's values
             // into a contiguous array.
-            const bool isCompressed = io::getDataCompression(is);
             boost::shared_array<ValueType> values(new ValueType[numValues]);
-            if (fromHalf) {
-                io::HalfReader<io::RealToHalf<ValueType>::isReal, ValueType>::read(
-                    is, values.get(), numValues, isCompressed);
-            } else {
-                io::readData<ValueType>(is, values.get(), numValues, isCompressed);
-            }
+            io::readCompressedValues(is, values.get(), numValues, mValueMask, fromHalf);
+
             // Copy values from the array into this node's table.
-            Index n = 0;
-            for (ValueAllIter iter = this->beginValueAll(); iter; ++iter) {
-                mNodes[iter.pos()].setValue(values[n++]);
+            if (oldVersion) {
+                Index n = 0;
+                for (ValueAllIter iter = this->beginValueAll(); iter; ++iter) {
+                    mNodes[iter.pos()].setValue(values[n++]);
+                }
+                assert(n == numValues);
+            } else {
+                for (ValueAllIter iter = this->beginValueAll(); iter; ++iter) {
+                    mNodes[iter.pos()].setValue(values[iter.pos()]);
+                }
             }
-            assert(n == numValues);
         }
         // Read in all child nodes and insert them into the table at their proper locations.
         for (ChildOnIter iter = this->beginChildOn(); iter; ++iter) {
@@ -1522,6 +1548,7 @@ InternalNode<ChildT, Log2Dim>::readTopology(std::istream& is, bool fromHalf)
     }
 }
 
+
 ////////////////////////////////////////
 
 
@@ -1637,7 +1664,7 @@ InternalNode<ChildT, Log2Dim>::merge(InternalNode& other,
         }
     }
 
-    // Copy avctive tile values.
+    // Copy active tile values.
     for (ValueOnCIter iter = other.cbeginValueOn(); iter; ++iter) {
         const Index n = iter.pos();
         if (mChildMask.isOff(n) && mValueMask.isOff(n)) {
@@ -1841,8 +1868,7 @@ InternalNode<ChildT, Log2Dim>::combine2(const ValueType& value, const InternalNo
             if (this->isChildMaskOff(i)) {
                 // Add a new child with the same coordinates, etc.
                 // as the other node's child.
-                /// @todo Could the other node's ChildNodeType be different
-                /// from this node's?
+                /// @todo Could the other node's ChildNodeType be different from this node's?
                 mChildMask.setOn(i);
                 mValueMask.setOff(i);
                 mNodes[i].setChild(new ChildNodeType(*otherChild));
@@ -1893,6 +1919,7 @@ InternalNode<ChildT, Log2Dim>::combine2(const InternalNode& other, const ValueTy
 
 ////////////////////////////////////////
 
+
 template<typename ChildT, Index Log2Dim>
 template<typename BBoxOp>
 inline void
@@ -1900,9 +1927,9 @@ InternalNode<ChildT, Log2Dim>::visitActiveBBox(BBoxOp& op) const
 {
     for (ValueOnCIter i = this->cbeginValueOn(); i; ++i) {
 #ifdef _MSC_VER
-        op.operator()<LEVEL>(CoordBBox(i.getCoord(), ChildNodeType::DIM));
+        op.operator()<LEVEL>(CoordBBox::createCube(i.getCoord(), ChildNodeType::DIM));
 #else
-        op.template operator()<LEVEL>(CoordBBox(i.getCoord(), ChildNodeType::DIM));
+        op.template operator()<LEVEL>(CoordBBox::createCube(i.getCoord(), ChildNodeType::DIM));
 #endif
     }
     if (op.template descent<LEVEL>()) {
@@ -1985,7 +2012,7 @@ template<
 inline void
 InternalNode<ChildT, Log2Dim>::doVisit2Node(NodeT& self, OtherNodeT& other, VisitorOp& op)
 {
-    /// Allow the two nodes to have different ValueTypes, but not different dimensions.
+    // Allow the two nodes to have different ValueTypes, but not different dimensions.
     BOOST_STATIC_ASSERT(OtherNodeT::NUM_VALUES == NodeT::NUM_VALUES);
     BOOST_STATIC_ASSERT(OtherNodeT::LEVEL == NodeT::LEVEL);
 
