@@ -44,7 +44,6 @@
 #include <openvdb/io/Compression.h> // for io::readData(), etc.
 #include "Iterator.h"
 #include "Util.h"
-#include "LeafArray.h"
 
 
 class TestLeaf;
@@ -75,8 +74,8 @@ public:
         SIZE        = NUM_VALUES,
         LEVEL       = 0;            // level 0 = leaf
 
-    /// ValueConverter<T>::Type is the type of a LeafNode having the same child
-    /// hierarchy and dimensions as this node but a different value type, T.
+    /// @brief ValueConverter<T>::Type is the type of a LeafNode having the same
+    /// child hierarchy and dimensions as this node but a different value type, T.
     template<typename OtherValueType>
     struct ValueConverter {
         typedef LeafNode<OtherValueType, Log2Dim> Type;
@@ -96,7 +95,7 @@ public:
             Index size = SIZE;
             while (size--) *target++ = val;
         }
-        /// Fast access methods
+        // Fast access methods
         const ValueType& getValue(Index i) const { assert(i < SIZE); return mData[i]; }
         void setValue(Index i, const ValueType& val) { assert(i < SIZE); mData[i] = val; }
         const ValueType& operator[](Index i) const { return this->getValue(i); }
@@ -130,9 +129,8 @@ public:
         static Index size() { return SIZE; }
 
     private:
-
         /// This direct access method is private since it makes
-        /// assumtions about the implementations of the memory layout.
+        /// assumptions about the implementations of the memory layout.
         ValueType& operator[](Index i) { assert(i < SIZE); return mData[i]; }
 
         friend class ::TestLeaf;
@@ -146,7 +144,7 @@ public:
     /// Default constructor
     LeafNode();
 
-    /// Constructor
+    /// @brief Constructor
     /// @param coords  the grid index coordinates of a voxel
     /// @param value   a value with which to fill the buffer
     /// @param active  the active state to which to initialize all voxels
@@ -199,14 +197,13 @@ public:
 
     /// Return the memory in bytes occupied by this node.
     Index64 memUsage() const;
-    
-    /// Expand the specified bbox so it includes the active voxels of
-    /// this leaf node.
-    void evalActiveVoxelBoundingBox(CoordBBox& bbox) const;
 
-    /// @return the bounding box of this node, i.e. the full index space
+    /// Expand the given bounding box so that it includes this leaf node's active voxels.
+    void evalActiveVoxelBoundingBox(CoordBBox&) const;
+
+    /// @brief Return the bounding box of this node, i.e., the full index space
     /// spanned by this leaf node.
-    CoordBBox getNodeBoundingBox() const { return CoordBBox(mOrigin, DIM); }
+    CoordBBox getNodeBoundingBox() const { return CoordBBox::createCube(mOrigin, DIM); }
 
     /// Get the grid index coordinates of this node's local origin.
     const Coord& getOrigin() const { return mOrigin; }
@@ -609,7 +606,7 @@ public:
         return mBuffer[offset];
     }
     template<typename ProbeType, typename AccessorT>
-    const ValueType& probe(const Coord& xyz, ProbeType& p, AccessorT&) const
+    OPENVDB_DEPRECATED const ValueType& probe(const Coord& xyz, ProbeType& p, AccessorT&) const
     {
         const Index offset = this->coord2offset(xyz);
         p.setState(mValueMask.isOn(offset));
@@ -617,7 +614,7 @@ public:
         return mBuffer[offset];
     }
     template<bool State, bool Level, typename AccessorT>
-    const ValueType& probe(const Coord& xyz, bool& state, int& level, AccessorT&) const
+    OPENVDB_DEPRECATED const ValueType& probe(const Coord& xyz, bool& state, int& level, AccessorT&) const
     {
         const Index offset = this->coord2offset(xyz);
         if (State) state =  mValueMask.isOn(offset);
@@ -632,7 +629,7 @@ public:
 
     /// @note Used internally by ValueAccessor.
     template<typename AccessorT>
-    void updateCache(const Coord&, AccessorT&) const {}
+    OPENVDB_DEPRECATED void updateCache(const Coord&, AccessorT&) const {}
 
     /// @brief Return a const reference to the first value in the buffer.
     /// @note Though it is potentially risky you can convert this
@@ -711,13 +708,21 @@ public:
     void prune(const ValueType& /*tolerance*/ = zeroVal<ValueType>()) {}
     void pruneInactive(const ValueType&) {}
     //@}
-    /// @brief return reference to itself
+    //@{
+    /// @brief return a pointer to itself
     LeafNode* touchLeaf(const Coord&) { return this; }
     template<typename AccessorT>
     LeafNode* touchLeafAndCache(const Coord&, AccessorT&) { return this; }
     LeafNode* probeLeaf(const Coord&) { return this; }
     template<typename AccessorT>
     LeafNode* probeLeafAndCache(const Coord&, AccessorT&) { return this; }
+    //@}
+    //@{
+    /// @brief return a const pointer to itself
+    const LeafNode* probeConstLeaf(const Coord&) const { return this; }
+    template<typename AccessorT>
+    const LeafNode* probeConstLeafAndCache(const Coord&, AccessorT&) const { return this; }
+    //@}
 
     /// Return @c true if all of this node's values have the same active state
     /// and are equal to within the given tolerance, and return the value in @a constValue
@@ -731,8 +736,8 @@ protected:
     friend class ::TestLeaf;
     template<typename> friend class ::TestLeafIO;
 
-    /// During topology-only construction, access is needed
-    /// to protected/private members of other template instances.
+    // During topology-only construction, access is needed
+    // to protected/private members of other template instances.
     template<typename, Index> friend class LeafNode;
 
     friend struct ValueIter<MaskOnIterator, LeafNode, ValueType, ValueOn>;
@@ -742,7 +747,7 @@ protected:
     friend struct ValueIter<MaskOffIterator, const LeafNode, ValueType, ValueOff>;
     friend struct ValueIter<MaskDenseIterator, const LeafNode, ValueType, ValueAll>;
 
-    /// Allow iterators to call mask accessor methods (see below).
+    // Allow iterators to call mask accessor methods (see below).
     /// @todo Make mask accessors public?
     friend class IteratorBase<MaskOnIterator, LeafNode>;
     friend class IteratorBase<MaskOffIterator, LeafNode>;
@@ -771,9 +776,6 @@ protected:
     void setValueMaskOn(Index n) { setValueMask(n, true); }
     void setValueMaskOff(Index n) { mValueMask.setOff(n); }
 
-    //const ValueType& bufferValueAt(Index idx) const { return mBuffer[idx]; }
-    //ValueType& bufferValueAt(Index idx) { return mBuffer[idx]; }
-
     /// Compute the origin of the leaf node that contains the voxel with the given coordinates.
     static void evalNodeOrigin(Coord& xyz) { xyz &= ~(DIM - 1); }
 
@@ -1039,6 +1041,9 @@ LeafNode<T, Log2Dim>::writeTopology(std::ostream& os, bool /*toHalf*/) const
 }
 
 
+////////////////////////////////////////
+
+
 template<typename T, Index Log2Dim>
 inline void
 LeafNode<T,Log2Dim>::readBuffers(std::istream& is, bool fromHalf)
@@ -1046,32 +1051,27 @@ LeafNode<T,Log2Dim>::readBuffers(std::istream& is, bool fromHalf)
     // Read in the value mask.
     mValueMask.load(is);
 
-    // Read in the origin.
-    is.read(reinterpret_cast<char*>(&mOrigin), sizeof(Coord::ValueType) * 3);//not needed!
+    int8_t numBuffers = 1;
+    if (io::getFormatVersion(is) < OPENVDB_FILE_VERSION_NODE_MASK_COMPRESSION) {
+        // Read in the origin.
+        is.read(reinterpret_cast<char*>(&mOrigin), sizeof(Coord::ValueType) * 3);
 
-    // Read in the number of buffers, which should now always be one.
-    int8_t numBuffers = 0;
-    is.read(reinterpret_cast<char*>(&numBuffers), sizeof(int8_t));//not needed!
-
-    const bool isCompressed = io::getDataCompression(is);
-
-    // Read in the buffer.
-    if (fromHalf) {
-        io::HalfReader<io::RealToHalf<T>::isReal, T>::read(is, mBuffer.mData, SIZE, isCompressed);
-    } else {
-        io::readData<T>(is, mBuffer.mData, SIZE, isCompressed);
+        // Read in the number of buffers, which should now always be one.
+        is.read(reinterpret_cast<char*>(&numBuffers), sizeof(int8_t));
     }
 
+    io::readCompressedValues(is, mBuffer.mData, SIZE, mValueMask, fromHalf);
+
     if (numBuffers > 1) {
-        // Read in and discard auxiliary buffers that were created with
-        // earlier versions of the library.
+        // Read in and discard auxiliary buffers that were created with earlier
+        // versions of the library.  (Auxiliary buffers are not mask compressed.)
+        const bool zipped = io::getDataCompression(is) & io::COMPRESS_ZIP;
         Buffer temp;
         for (int i = 1; i < numBuffers; ++i) {
             if (fromHalf) {
-                io::HalfReader<io::RealToHalf<T>::isReal, T>::read(
-                    is, temp.mData, SIZE, isCompressed);
+                io::HalfReader<io::RealToHalf<T>::isReal, T>::read(is, temp.mData, SIZE, zipped);
             } else {
-                io::readData<T>(is, temp.mData, SIZE, isCompressed);
+                io::readData<T>(is, temp.mData, SIZE, zipped);
             }
         }
     }
@@ -1085,22 +1085,14 @@ LeafNode<T, Log2Dim>::writeBuffers(std::ostream& os, bool toHalf) const
     // Write out the value mask.
     mValueMask.save(os);
 
-    // Write out the origin.
-    os.write(reinterpret_cast<const char*>(&mOrigin), sizeof(Coord::ValueType) * 3);//not needed!
-
-    // Write out the number of buffers.
-    const int8_t numBuffers = 1;
-    os.write(reinterpret_cast<const char*>(&numBuffers), sizeof(int8_t));//not needed!
-
-    // Write out the buffer.
-    const bool compress = io::getDataCompression(os);
-    if (toHalf) {
-        io::HalfWriter<io::RealToHalf<T>::isReal, T>::write(os, mBuffer.mData, SIZE, compress);
-    } else {
-        io::writeData(os, mBuffer.mData, SIZE, compress);
-    }
+    io::writeCompressedValues(os, mBuffer.mData, SIZE,
+        mValueMask, /*childMask=*/NodeMaskType(), toHalf);
 }
 
+
+////////////////////////////////////////
+
+
 template<typename T, Index Log2Dim>
 inline bool
 LeafNode<T, Log2Dim>::operator==(const LeafNode& other) const
@@ -1365,9 +1357,9 @@ LeafNode<T, Log2Dim>::visitActiveBBox(BBoxOp& op) const
     if (op.template descent<LEVEL>()) {
         for (ValueOnCIter i=this->cbeginValueOn(); i; ++i) {
 #ifdef _MSC_VER
-            op.operator()<LEVEL>(CoordBBox(i.getCoord(), 1));
+            op.operator()<LEVEL>(CoordBBox::createCube(i.getCoord(), 1));
 #else
-            op.template operator()<LEVEL>(CoordBBox(i.getCoord(), 1));
+            op.template operator()<LEVEL>(CoordBBox::createCube(i.getCoord(), 1));
 #endif
         }
     } else {
@@ -1442,7 +1434,7 @@ template<
 inline void
 LeafNode<T, Log2Dim>::doVisit2Node(NodeT& self, OtherNodeT& other, VisitorOp& op)
 {
-    /// Allow the two nodes to have different ValueTypes, but not different dimensions.
+    // Allow the two nodes to have different ValueTypes, but not different dimensions.
     BOOST_STATIC_ASSERT(OtherNodeT::SIZE == NodeT::SIZE);
     BOOST_STATIC_ASSERT(OtherNodeT::LEVEL == NodeT::LEVEL);
 
@@ -1520,6 +1512,7 @@ operator<<(std::ostream& os, const typename LeafNode<T, Log2Dim>::Buffer& buf)
 
 ////////////////////////////////////////
 
+
 // Specialization for LeafNodes of type bool
 #include "LeafNodeBool.h"
 
