@@ -168,14 +168,14 @@ public:
 
     /// Return the memory in bytes occupied by this node.
     Index64 memUsage() const;
-    
+
     /// Expand the specified bbox so it includes the active voxels of
     /// this leaf node.
     void evalActiveVoxelBoundingBox(CoordBBox& bbox) const;
 
-    /// @return the bounding box of this node, i.e. the full index space
+    /// @brief Return the bounding box of this node, i.e., the full index space
     /// spanned by this leaf node.
-    CoordBBox getNodeBoundingBox() const { return CoordBBox(mOrigin, DIM); }
+    CoordBBox getNodeBoundingBox() const { return CoordBBox::createCube(mOrigin, DIM); }
 
     /// Get the grid index coordinates of this node's local origin.
     const Coord& getOrigin() const { return mOrigin; }
@@ -369,7 +369,7 @@ public:
     /// its active state and level (i.e., 0) in @a state and @a level.
     /// @note Used internally by ValueAccessor (note last argument is a dummy).
     template<typename AccessorT>
-    const ValueType& getValue(const Coord& xyz, bool& state, int& level, AccessorT&) const
+    OPENVDB_DEPRECATED const ValueType& getValue(const Coord& xyz, bool& state, int& level, AccessorT&) const
     {
         const Index offset = this->coord2offset(xyz);
         state = mValueMask.isOn(offset);
@@ -378,7 +378,7 @@ public:
         if (mBuffer.mData.isOn(offset)) return sOn; else return sOff;
     }
     template<typename ProbeType, typename AccessorT>
-    const ValueType& probe(const Coord& xyz, ProbeType& p, AccessorT&) const
+    OPENVDB_DEPRECATED const ValueType& probe(const Coord& xyz, ProbeType& p, AccessorT&) const
     {
         const Index offset = this->coord2offset(xyz);
         p.setState(mValueMask.isOn(offset));
@@ -387,7 +387,7 @@ public:
         if (mBuffer.mData.isOn(offset)) return sOn; else return sOff;
     }
     template<bool State, bool Level, typename AccessorT>
-    const ValueType& probe(const Coord& xyz, bool& state, int& level, AccessorT&) const
+    OPENVDB_DEPRECATED const ValueType& probe(const Coord& xyz, bool& state, int& level, AccessorT&) const
     {
         const Index offset = this->coord2offset(xyz);
         if (State) state =  mValueMask.isOn(offset);
@@ -402,7 +402,7 @@ public:
 
     /// @note Used internally by ValueAccessor.
     template<typename AccessorT>
-    void updateCache(const Coord&, AccessorT&) const {}
+    OPENVDB_DEPRECATED void updateCache(const Coord&, AccessorT&) const {}
 
     /// @brief Return a const reference to the first entry in the buffer.
     /// @note Since it's actually a reference to a static data member
@@ -483,13 +483,22 @@ public:
     void prune(const ValueType& /*tolerance*/ = zeroVal<ValueType>()) {}
     void pruneInactive(const ValueType&) {}
     //@}
-    /// @brief return reference to itself
+
+    //@{
+    /// @brief return a pointer to itself
     LeafNode* touchLeaf(const Coord&) { return this; }
     template<typename AccessorT>
     LeafNode* touchLeafAndCache(const Coord&, AccessorT&) { return this; }
     LeafNode* probeLeaf(const Coord&) { return this; }
     template<typename AccessorT>
     LeafNode* probeLeafAndCache(const Coord&, AccessorT&) { return this; }
+    //@}
+    //@{
+    /// @brief return a const pointer to itself
+    const LeafNode* probeConstLeaf(const Coord&) const { return this; }
+    template<typename AccessorT>
+    const LeafNode* probeConstLeafAndCache(const Coord&, AccessorT&) const { return this; }
+    //@}
 
     void merge(const LeafNode& other, bool, bool) { this->merge(other); }
 
@@ -1326,7 +1335,7 @@ template<
 inline void
 LeafNode<bool, Log2Dim>::doVisit2Node(NodeT& self, OtherNodeT& other, VisitorOp& op)
 {
-    /// Allow the two nodes to have different ValueTypes, but not different dimensions.
+    // Allow the two nodes to have different ValueTypes, but not different dimensions.
     BOOST_STATIC_ASSERT(OtherNodeT::SIZE == NodeT::SIZE);
     BOOST_STATIC_ASSERT(OtherNodeT::LEVEL == NodeT::LEVEL);
 
