@@ -81,12 +81,12 @@ public:
 
     /// @brief Topology copy constructor that guarantees the
     /// configuration of the constructed tree is topologically
-    /// identical to the other tree. All values that are active
-    /// in the other tree are set to the foreground value and all
-    /// other values to the background value.
+    /// identical to the other tree
     ///
-    /// Reproduce the topology and active states of the other node
+    /// @details Reproduce the topology and active states of the other tree
     /// (which may have a different ValueType), but don't copy values.
+    /// All values that are active in the other tree are set to the foreground value
+    /// and all other values to the background value.
     /// @param other       the root node of a tree having (possibly) a different ValueType
     /// @param background  the value to which inactive tiles and voxels are initialized
     /// @param foreground  the value to which active tiles and voxels are initialized
@@ -97,16 +97,17 @@ public:
 
     /// @brief Topology copy constructor that guarantees the
     /// configuration of the constructed tree is topologically
-    /// identical to the other tree. All values in the constructed
-    /// tree are set to the background value regardless of its state.
+    /// identical to the other tree
     ///
     /// @note this copy constructor is generally faster then the one
     /// above that takes both a forground and a background value. Its
     /// main application is multithreading where the topology of
     /// the output tree exactly matches the input tree.
     ///
-    /// Reproduce the topology and active states of the other node
+    /// @details Reproduce the topology and active states of the other node
     /// (which may have a different ValueType), but don't copy values.
+    /// All values in the constructed tree are set to the background value
+    /// regardless of their active states.
     /// @param other       the root node of a tree having (possibly) a different ValueType
     /// @param background  the value to which inactive tiles and voxels are initialized
     template<typename OtherChildType>
@@ -224,8 +225,8 @@ private:
         bool next() { this->increment(); return this->test(); }
         void increment(Index n) { for (int i = 0; i < n && this->next(); ++i) {} }
 
-        /// Return this iterator's position as an offset from the beginning
-        /// of the parent node's map.
+        /// @brief Return this iterator's position as an offset from
+        /// the beginning of the parent node's map.
         Index pos() const
         {
             return !mParentNode ? 0U : Index(std::distance(mParentNode->mTable.begin(), mIter));
@@ -249,7 +250,7 @@ private:
 
         RootNodeT* mParentNode;
         MapIterT mIter;
-    };// BaseIter
+    }; // BaseIter
 
     template<typename RootNodeT, typename MapIterT, typename FilterPredT, typename ChildNodeT>
     class ChildIter: public BaseIter<RootNodeT, MapIterT, FilterPredT>
@@ -272,7 +273,7 @@ private:
         ChildNodeT& getValue() const { return getChild(mIter); }
         ChildNodeT& operator*() const { return this->getValue(); }
         ChildNodeT* operator->() const { return &this->getValue(); }
-    };// ChildIter
+    }; // ChildIter
 
     template<typename RootNodeT, typename MapIterT, typename FilterPredT, typename ValueT>
     class ValueIter: public BaseIter<RootNodeT, MapIterT, FilterPredT>
@@ -295,7 +296,7 @@ private:
         ValueT* operator->() const { return &(this->getValue()); }
 
         void setValue(const ValueT& v) const { assert(isTile(mIter)); getTile(mIter).value = v; }
-    };// ValueIter
+    }; // ValueIter
 
     template<typename RootNodeT, typename MapIterT, typename ChildNodeT, typename ValueT>
     class DenseIter: public BaseIter<RootNodeT, MapIterT, NullPred>
@@ -340,7 +341,7 @@ private:
             /// setTile() here, because that would also delete the child.
             else stealChild(mIter, Tile(v, /*active=*/true));
         }
-    };// DenseIter
+    }; // DenseIter
 
 public:
     typedef ChildIter<RootNode, MapIter, ChildOnPred, ChildType>                  ChildOnIter;
@@ -380,13 +381,12 @@ public:
 
     /// Return the total amount of memory in bytes occupied by this node and its children.
     Index64 memUsage() const;
-    
-    /// Expand the specified bbox so it includes the active tiles of
+
+    /// @brief Expand the specified bbox so it includes the active tiles of
     /// this root node as well as all the active values in its child nodes.
     void evalActiveVoxelBoundingBox(CoordBBox& bbox) const;
 
-    /// @return the bounding box of this RootNode, i.e. an infinite
-    /// bounding box :) The actual index range if defined from Coord's range.
+    /// Return the bounding box of this RootNode, i.e., an infinite bounding box.
     static CoordBBox getNodeBoundingBox() { return CoordBBox::inf(); }
 
     void setBackground(const ValueType& value);
@@ -457,9 +457,8 @@ public:
     bool probeValue(const Coord& xyz, ValueType& value) const;
 
     /// @brief Return the tree depth (0 = root) at which the value of voxel (x, y, z) resides.
-    ///
-    /// If (x, y, z) isn't explicitly represented in the tree (i.e., it is implicitly
-    /// a background voxel), return -1.
+    /// @details If (x, y, z) isn't explicitly represented in the tree (i.e.,
+    /// it is implicitly a background voxel), return -1.
     int getValueDepth(const Coord& xyz) const;
 
     /// Set the active state of the voxel at the given coordinates, but don't change its value.
@@ -495,13 +494,13 @@ public:
     void readBuffers(std::istream&, bool fromHalf = false);
 
     template<typename AccessorT>
-    const ValueType& getValue(const Coord& xyz, bool& state, int& level, AccessorT&) const;
+    OPENVDB_DEPRECATED const ValueType& getValue(const Coord& xyz, bool& state, int& level, AccessorT&) const;
 
     template<typename ProbeType, typename AccessorT>
-    const ValueType& probe(const Coord& xyz, ProbeType& p, AccessorT&) const;
+    OPENVDB_DEPRECATED const ValueType& probe(const Coord& xyz, ProbeType& p, AccessorT&) const;
 
     template<bool State, bool Level, typename AccessorT>
-    const ValueType& probe(const Coord& xyz, bool& state, int& level, AccessorT&) const;
+    OPENVDB_DEPRECATED const ValueType& probe(const Coord& xyz, bool& state, int& level, AccessorT&) const;
 
     /// Return the value of the voxel at the given coordinates and, if necessary, update
     /// the accessor with pointers to the nodes along the path from the root node to
@@ -571,7 +570,7 @@ public:
     /// from the root node to the node containing voxel (x, y, z).
     /// @note Used internally by ValueAccessor.
     template<typename AccessorT>
-    void updateCache(const Coord& xyz, AccessorT&) const;
+    OPENVDB_DEPRECATED void updateCache(const Coord& xyz, AccessorT&) const;
 
     /// Call the @c PruneOp functor for each child node and, if the functor
     /// returns @c true, prune the node and replace it with a tile.
@@ -594,17 +593,20 @@ public:
     /// background tiles any nodes whose values are all inactive.
     void pruneInactive();
 
-    /// @brief @return the leaf node that contains voxel (x, y, z) and
-    /// if it doesn't exist, create it, but preserve the values and
+    /// @brief Return the leaf node that contains voxel (x, y, z).
+    /// If no such node exists, create one, but preserve the values and
     /// active states of all voxels.
     ///
-    /// Use this method to preallocate a static tree topology over which to
-    /// safely perform multithreaded processing.
+    /// @details Use this method to preallocate a static tree topology
+    /// over which to safely perform multithreaded processing.
     LeafNodeType* touchLeaf(const Coord& xyz);
 
-    /// @brief @return the leaf node that contains voxel (x, y, z) and
-    /// if it doesn't exist, return NULL.
+    /// @brief Return a pointer to the leaf node that contains voxel (x, y, z).
+    /// If no such node exists, return NULL.
     LeafNodeType* probeLeaf(const Coord& xyz);
+    /// @brief Return a const pointer to the leaf node that contains voxel (x, y, z).
+    /// If no such node exists, return NULL.
+    const LeafNodeType* probeConstLeaf(const Coord& xyz) const;
 
     /// @brief Same as touchLeaf except, if necessary, it update the accessor with pointers
     /// to the nodes along the path from the root node to the node containing the coordinate.
@@ -615,6 +617,10 @@ public:
     /// to the nodes along the path from the root node to the node containing the coordinate.
     template<typename AccessorT>
     LeafNodeType* probeLeafAndCache(const Coord& xyz, AccessorT&);
+    /// @brief Same as probeConstLeaf except, if necessary, it update the accessor with pointers
+    /// to the nodes along the path from the root node to the node containing the coordinate.
+    template<typename AccessorT>
+    const LeafNodeType* probeConstLeafAndCache(const Coord& xyz, AccessorT&) const;
 
     /// @brief Propagate the sign from a narrow-band level set into inactive voxels and tiles.
     /// @note This method should only be used on closed narrow-band level sets!!!
@@ -627,7 +633,7 @@ public:
     /// @note This operation always empties the other tree, whether or not any nodes were moved.
     void merge(RootNode& other);
 
-    /// Turns active tiles into dense voxels, i.e. active leaf nodes
+    /// Turn active tiles into dense voxels, i.e., into leaf nodes that are entirely active.
     void voxelizeActiveTiles();
 
     /// @brief Union this tree's set of active values with the other
@@ -703,8 +709,7 @@ private:
     MapCIter findCoord(const Coord& xyz) const { return mTable.find(coordToKey(xyz)); }
     //@}
     /// @brief Convert the given coordinates to a key and look the key up in this node's mTable.
-    ///
-    /// If the key is not found, insert a background tile with that key.
+    /// @details If the key is not found, insert a background tile with that key.
     /// @return an iterator pointing to the matching mTable entry.
     MapIter findOrAddCoord(const Coord& xyz);
 
@@ -766,6 +771,7 @@ RootNode<ChildT>::RootNode(const RootNode<OtherChildType>& other,
     }
 }
 
+
 template<typename ChildT>
 template<typename OtherChildType>
 inline
@@ -1139,9 +1145,10 @@ inline Index32
 RootNode<ChildT>::nonLeafCount() const
 {
     Index32 sum = 1;
-    if (ChildT::LEVEL == 0) return sum;
-    for (MapCIter i = mTable.begin(), e = mTable.end(); i != e; ++i) {
-        if (isChild(i)) sum += getChild(i).nonLeafCount();
+    if (ChildT::LEVEL != 0) {
+        for (MapCIter i = mTable.begin(), e = mTable.end(); i != e; ++i) {
+            if (isChild(i)) sum += getChild(i).nonLeafCount();
+        }
     }
     return sum;
 }
@@ -1410,24 +1417,6 @@ RootNode<ChildT>::setValueOn(const Coord& xyz, const ValueType& value)
     if (child) child->setValueOn(xyz, value);
 }
 
-template<typename ChildT>
-inline void
-RootNode<ChildT>::setValueOnly(const Coord& xyz, const ValueType& value)
-{
-    ChildT* child = NULL;
-    MapIter iter = this->findCoord(xyz);
-    if (iter == mTable.end()) {
-        child = new ChildT(xyz, mBackground);
-        mTable[this->coordToKey(xyz)] = NodeStruct(*child);
-    } else if (isChild(iter)) {
-        child = &getChild(iter);
-    } else if (!math::isExactlyEqual(getTile(iter).value, value)) {
-        child = new ChildT(xyz, getTile(iter).value, isTileOn(iter));
-        setChild(iter, *child);
-    }
-    if (child) child->setValueOnly(xyz, value);
-}
-
 template<typename ChildT>
 template<typename AccessorT>
 inline void
@@ -1450,6 +1439,25 @@ RootNode<ChildT>::setValueAndCache(const Coord& xyz, const ValueType& value, Acc
     }
 }
 
+
+template<typename ChildT>
+inline void
+RootNode<ChildT>::setValueOnly(const Coord& xyz, const ValueType& value)
+{
+    ChildT* child = NULL;
+    MapIter iter = this->findCoord(xyz);
+    if (iter == mTable.end()) {
+        child = new ChildT(xyz, mBackground);
+        mTable[this->coordToKey(xyz)] = NodeStruct(*child);
+    } else if (isChild(iter)) {
+        child = &getChild(iter);
+    } else if (!math::isExactlyEqual(getTile(iter).value, value)) {
+        child = new ChildT(xyz, getTile(iter).value, isTileOn(iter));
+        setChild(iter, *child);
+    }
+    if (child) child->setValueOnly(xyz, value);
+}
+
 template<typename ChildT>
 template<typename AccessorT>
 inline void
@@ -1585,6 +1593,7 @@ RootNode<ChildT>::probeValueAndCache(const Coord& xyz, ValueType& value, Accesso
     return isTileOn(iter);
 }
 
+
 template<typename ChildT>
 template<typename AccessorT>
 inline const typename ChildT::ValueType&
@@ -1604,6 +1613,7 @@ RootNode<ChildT>::getValue(const Coord& xyz, bool& state, int& level, AccessorT&
     return getTile(iter).value;
 }
 
+
 template<typename ChildT>
 template<typename ProbeT, typename AccessorT>
 inline const typename ChildT::ValueType&
@@ -1642,6 +1652,7 @@ RootNode<ChildT>::probe(const Coord& xyz, bool& state, int& level, AccessorT& ac
     return getTile(iter).value;
 }
 
+
 template<typename ChildT>
 template<typename AccessorT>
 inline void
@@ -1662,6 +1673,8 @@ template<typename ChildT>
 inline void
 RootNode<ChildT>::fill(const CoordBBox& bbox, const ValueType& value, bool active)
 {
+    if (bbox.empty()) return;
+
     Coord xyz, tileMax;
     for (int x = bbox.min().x(); x <= bbox.max().x(); x = tileMax.x() + 1) {
         xyz.setX(x);
@@ -1720,6 +1733,7 @@ inline bool
 RootNode<ChildT>::writeTopology(std::ostream& os, bool toHalf) const
 {
     os.write(reinterpret_cast<const char*>(&mBackground), sizeof(ValueType));
+    io::setGridBackgroundValuePtr(os, &mBackground);
 
     const Index numTiles = this->getTileCount(), numChildren = this->getChildCount();
     os.write(reinterpret_cast<const char*>(&numTiles), sizeof(Index));
@@ -1761,6 +1775,8 @@ RootNode<ChildT>::readTopology(std::istream& is, bool fromHalf)
         ValueType inside;
         is.read(reinterpret_cast<char*>(&inside), sizeof(ValueType));
 
+        io::setGridBackgroundValuePtr(is, &mBackground);
+
         // Read the index range.
         Coord rangeMin, rangeMax;
         is.read(reinterpret_cast<char*>(rangeMin.asPointer()), 3 * sizeof(Int32));
@@ -1816,6 +1832,7 @@ RootNode<ChildT>::readTopology(std::istream& is, bool fromHalf)
     // Read a RootNode that was stored in the current format.
 
     is.read(reinterpret_cast<char*>(&mBackground), sizeof(ValueType));
+    io::setGridBackgroundValuePtr(is, &mBackground);
 
     Index numTiles = 0, numChildren = 0;
     is.read(reinterpret_cast<char*>(&numTiles), sizeof(Index));
@@ -1847,6 +1864,7 @@ RootNode<ChildT>::readTopology(std::istream& is, bool fromHalf)
     return true; // not empty
 }
 
+
 template<typename ChildT>
 inline void
 RootNode<ChildT>::writeBuffers(std::ostream& os, bool toHalf) const
@@ -1947,6 +1965,7 @@ RootNode<ChildT>::touchLeafAndCache(const Coord& xyz, AccessorT& acc)
     return child->touchLeafAndCache(xyz, acc);
 }
 
+
 template<typename ChildT>
 inline typename ChildT::LeafNodeType*
 RootNode<ChildT>::probeLeaf(const Coord& xyz)
@@ -1956,6 +1975,15 @@ RootNode<ChildT>::probeLeaf(const Coord& xyz)
     return getChild(iter).probeLeaf(xyz);
 }
 
+template<typename ChildT>
+inline const typename ChildT::LeafNodeType*
+RootNode<ChildT>::probeConstLeaf(const Coord& xyz) const
+{
+    MapCIter iter = this->findCoord(xyz);
+    if (iter == mTable.end() || isTile(iter)) return NULL;
+    return getChild(iter).probeConstLeaf(xyz);
+}
+
 template<typename ChildT>
 template<typename AccessorT>
 inline typename ChildT::LeafNodeType*
@@ -1964,9 +1992,23 @@ RootNode<ChildT>::probeLeafAndCache(const Coord& xyz, AccessorT& acc)
     MapIter iter = this->findCoord(xyz);
     if (iter == mTable.end() || isTile(iter)) return NULL;
     ChildT* child = &getChild(iter);
+    acc.insert(xyz, child);
     return child->probeLeafAndCache(xyz, acc);
 }
 
+template<typename ChildT>
+template<typename AccessorT>
+inline const typename ChildT::LeafNodeType*
+RootNode<ChildT>::probeConstLeafAndCache(const Coord& xyz, AccessorT& acc) const
+{
+    MapCIter iter = this->findCoord(xyz);
+    if (iter == mTable.end() || isTile(iter)) return NULL;
+    const ChildT* child = &getChild(iter);
+    acc.insert(xyz, child);
+    return child->probeConstLeafAndCache(xyz, acc);
+}
+
+
 ////////////////////////////////////////
 
 
@@ -1997,7 +2039,7 @@ RootNode<ChildT>::signedFloodFill()
         if (d[0]!=0 || d[1]!=0 || d[2]==Int32(ChildT::DIM)) continue;//not z-scanline or neighbors
         MapIter i = mTable.find(*a), j = mTable.find(*b);
         const ValueType fill[] = { getChild(i).getLastValue(), getChild(j).getFirstValue() };
-        if (fill[0] > zero || fill[1] > zero) continue; // scanline isn't inside
+        if (!(fill[0] < zero) || !(fill[1] < zero)) continue; // scanline isn't inside
         for (Coord c = *a + Coord(0u,0u,ChildT::DIM); c[2] != (*b)[2]; c[2] += ChildT::DIM) {
             mTable[c] = inside;
         }
@@ -2009,6 +2051,25 @@ RootNode<ChildT>::signedFloodFill()
 ////////////////////////////////////////
 
 
+template<typename ChildT>
+inline void
+RootNode<ChildT>::voxelizeActiveTiles()
+{
+    for (MapIter i = mTable.begin(), e = mTable.end(); i != e; ++i) {
+        if (this->isTileOff(i)) continue;
+        ChildT* child = i->second.child;
+        if (child==NULL) {
+            child = new ChildT(i->first, this->getTile(i).value, true);
+            i->second.child = child;
+        }
+        child->voxelizeActiveTiles();
+    }
+}
+
+
+////////////////////////////////////////
+
+
 template<typename ChildT>
 inline void
 RootNode<ChildT>::merge(RootNode& other)
@@ -2033,21 +2094,6 @@ RootNode<ChildT>::merge(RootNode& other)
     other.clear();
 }
 
-template<typename ChildT>
-inline void
-RootNode<ChildT>::voxelizeActiveTiles()
-{
-    for (MapIter i = mTable.begin(), e = mTable.end(); i != e; ++i) {
-        if (this->isTileOff(i)) continue;
-        ChildT* child = i->second.child;
-        if (child==NULL) {
-            child = new ChildT(i->first, this->getTile(i).value, true);
-            i->second.child = child;
-        }
-        child->voxelizeActiveTiles();
-    }
-}
-
 template<typename ChildT>
 template<typename OtherChildType>
 inline void
@@ -2221,9 +2267,9 @@ RootNode<ChildT>::visitActiveBBox(BBoxOp& op) const
             this->getChild(i).visitActiveBBox(op);
         } else {
 #ifdef _MSC_VER
-            op.operator()<LEVEL>(CoordBBox(i->first, ChildT::DIM));
+            op.operator()<LEVEL>(CoordBBox::createCube(i->first, ChildT::DIM));
 #else
-            op.template operator()<LEVEL>(CoordBBox(i->first, ChildT::DIM));
+            op.template operator()<LEVEL>(CoordBBox::createCube(i->first, ChildT::DIM));
 #endif
         }
     }
