@@ -177,6 +177,8 @@ public:
     typedef typename RootNodeType::ValueType     ValueType;
     typedef typename RootNodeType::LeafNodeType  LeafNodeType;
 
+    static const Index DEPTH = RootNodeType::LEVEL + 1;
+
     /// @brief ValueConverter<T>::Type is the type of a tree having the same
     /// hierarchy as this tree but a different value type, T.
     ///
@@ -302,7 +304,7 @@ public:
     /// @brief Return the depth of this tree.
     ///
     /// A tree with only a root node and leaf nodes has depth 2, for example.
-    virtual Index treeDepth() const { return 1 + mRoot.getLevel(); }
+    virtual Index treeDepth() const { return DEPTH; }
     /// Return the number of leaf nodes.
     virtual Index32 leafCount() const { return mRoot.leafCount(); }
     /// Return the number of non-leaf nodes.
@@ -426,9 +428,12 @@ public:
     /// safely perform multithreaded processing.
     LeafNodeType* touchLeaf(const Coord& xyz);
     
-    /// @brief @return the leaf node that contains voxel (x, y, z) and
-    /// if it doesn't exist, return NULL.
+    /// @brief @return a pointer to the leaf node that contains
+    /// voxel (x, y, z) and if it doesn't exist, return NULL.
     LeafNodeType* probeLeaf(const Coord& xyz);
+    /// @brief @return a const pointer to the leaf node that contains
+    /// voxel (x, y, z) and if it doesn't exist, return NULL.
+    const LeafNodeType* probeConstLeaf(const Coord& xyz) const;
 
     /// Expand this tree's index range to include the given coordinates.
     OPENVDB_DEPRECATED bool expand(const Coord& xyz) { return mRoot.expand(xyz); }
@@ -1430,6 +1435,13 @@ Tree<RootNodeType>::probeLeaf(const Coord& xyz)
     return mRoot.probeLeaf(xyz);
 }
 
+template<typename RootNodeType>
+inline const typename RootNodeType::LeafNodeType*
+Tree<RootNodeType>::probeConstLeaf(const Coord& xyz) const
+{
+    return mRoot.probeConstLeaf(xyz);
+}
+
 ////////////////////////////////////////
 
 
@@ -1884,10 +1896,12 @@ Tree<RootNodeType>::print(std::ostream& os, int verboseLevel) const
 
         const double activeRatio = (100.0 * numActiveVoxels) / totalVoxels;
         os << "  Percentage of active voxels:   " << std::setprecision(3) << activeRatio << "%\n";
-
-        const double fillRatio =
-            (100.0 * numActiveLeafVoxels) / (leafCount * LeafNodeType::NUM_VOXELS);
-        os << "  Average leaf node fill ratio:  " << fillRatio << "%\n";
+        
+        if (leafCount>0) {
+            const double fillRatio =
+                (100.0 * numActiveLeafVoxels) / (leafCount * LeafNodeType::NUM_VOXELS);
+            os << "  Average leaf node fill ratio:  " << fillRatio << "%\n";
+        }
     } else {
         os << "  Tree is empty!\n";
     }
