@@ -1194,7 +1194,7 @@ public:
     typedef typename iter::InvertedTree<NCRootNodeT, ROOT_LEVEL>::Type InvTreeT;
     typedef typename boost::mpl::front<InvTreeT>::type NCLeafNodeT;
     typedef typename CopyConstness<RootNodeT, NCLeafNodeT>::Type LeafNodeT;
-    static const Index LEAF_LEVEL = 0;
+    static const Index LEAF_LEVEL = 0, LEAF_PARENT_LEVEL = LEAF_LEVEL + 1;
 
     typedef IterTraits<NCRootNodeT, RootIterT> RootIterTraits;
 
@@ -1202,8 +1202,13 @@ public:
 
     LeafIteratorBase(TreeT& tree): mIterList(NULL), mTree(&tree)
     {
+        // Initialize the iterator list with a root node iterator.
         mIterList.setIter(RootIterTraits::begin(tree.getRootNode()));
-        for (Index lvl = ROOT_LEVEL; lvl > 0 && mIterList.down(lvl); --lvl) {}
+        // Descend along the first branch, initializing the node iterator at each level.
+        Index lvl = ROOT_LEVEL;
+        for ( ; lvl > 0 && mIterList.down(lvl); --lvl) {}
+        // If the first branch terminated above the leaf level, backtrack to the next branch.
+        if (lvl > 0) this->next();
     }
 
     LeafIteratorBase(const LeafIteratorBase& other): mIterList(other.mIterList), mTree(other.mTree)
@@ -1227,7 +1232,7 @@ public:
     LeafNodeT* operator->() const { return this->getLeaf(); }
     //@}
 
-    bool test() const { return mIterList.test(LEAF_LEVEL + 1); }
+    bool test() const { return mIterList.test(LEAF_PARENT_LEVEL); }
     operator bool() const { return this->test(); }
 
     //@{
@@ -1256,28 +1261,30 @@ template<typename TreeT, typename RootChildOnIterT>
 inline bool
 LeafIteratorBase<TreeT, RootChildOnIterT>::next()
 {
-    if (mIterList.next(LEAF_LEVEL + 1)) {
-        mIterList.down(LEAF_LEVEL + 1); // initialize the leaf iterator
+    // If the iterator is valid for the current node one level above the leaf level,
+    // advance the iterator to the node's next child.
+    if (mIterList.test(LEAF_PARENT_LEVEL) && mIterList.next(LEAF_PARENT_LEVEL)) {
+        mIterList.down(LEAF_PARENT_LEVEL); // initialize the leaf iterator
         return true;
     }
 
-    Index lvl = LEAF_LEVEL + 1;
-    while (!mIterList.test(LEAF_LEVEL + 1)) {
-        if (!mIterList.test(lvl)) {
+    Index lvl = LEAF_PARENT_LEVEL;
+    while (!mIterList.test(LEAF_PARENT_LEVEL)) {
+        if (mIterList.test(lvl)) {
+            mIterList.next(lvl);
+        } else {
             do {
                 // Ascend to the nearest level at which
                 // one of the iterators is not yet exhausted.
                 if (lvl == ROOT_LEVEL) return false;
                 ++lvl;
-                mIterList.next(lvl);
+                if (mIterList.test(lvl)) mIterList.next(lvl);
             } while (!mIterList.test(lvl));
-        } else {
-            mIterList.next(lvl);
         }
         // Descend to the lowest child, but not as far as the leaf iterator.
-        while (lvl > LEAF_LEVEL + 1 && mIterList.down(lvl)) --lvl;
+        while (lvl > LEAF_PARENT_LEVEL && mIterList.down(lvl)) --lvl;
     }
-    mIterList.down(LEAF_LEVEL + 1); // initialize the leaf iterator
+    mIterList.down(LEAF_PARENT_LEVEL); // initialize the leaf iterator
     return true;
 }
 
