@@ -31,7 +31,7 @@
 #include <cppunit/extensions/HelperMacros.h>
 #include <openvdb/Types.h>
 #include <openvdb/openvdb.h>
-#include <openvdb/tools/Curl.h>
+#include <openvdb/tools/GridOperators.h>
 
 #define ASSERT_DOUBLES_EXACTLY_EQUAL(expected, actual) \
     CPPUNIT_ASSERT_DOUBLES_EQUAL((expected), (actual), /*tolerance=*/0.0);
@@ -89,8 +89,7 @@ TestCurl::testCurlTool()
     CPPUNIT_ASSERT(!inTree.empty());
     CPPUNIT_ASSERT_EQUAL(math::Pow3(2*dim), int(inTree.activeVoxelCount()));
 
-    tools::Curl<VectorGrid> curl(*inGrid);
-    VectorGrid::Ptr curl_grid = curl.process();
+    VectorGrid::Ptr curl_grid = tools::curl(*inGrid);
     CPPUNIT_ASSERT_EQUAL(math::Pow3(2*dim), int(curl_grid->activeVoxelCount()));
 
     VectorGrid::ConstAccessor curlAccessor = curl_grid->getConstAccessor();
@@ -138,8 +137,7 @@ TestCurl::testISCurl()
     CPPUNIT_ASSERT(!inTree.empty());
     CPPUNIT_ASSERT_EQUAL(math::Pow3(2*dim), int(inTree.activeVoxelCount()));
 
-    tools::Curl<VectorGrid> curl(*inGrid);
-    VectorGrid::Ptr curl_grid = curl.process();
+    VectorGrid::Ptr curl_grid = tools::curl(*inGrid);
     CPPUNIT_ASSERT_EQUAL(math::Pow3(2*dim), int(curl_grid->activeVoxelCount()));
 
     --dim;//ignore boundary curl vectors
@@ -256,8 +254,7 @@ TestCurl::testISCurlStencil()
     CPPUNIT_ASSERT(!inTree.empty());
     CPPUNIT_ASSERT_EQUAL(math::Pow3(2*dim), int(inTree.activeVoxelCount()));
 
-    tools::Curl<VectorGrid> curl(*inGrid);
-    VectorGrid::Ptr curl_grid = curl.process();
+    VectorGrid::Ptr curl_grid = tools::curl(*inGrid);
     CPPUNIT_ASSERT_EQUAL(math::Pow3(2*dim), int(curl_grid->activeVoxelCount()));
 
     math::SevenPointStencil<VectorGrid> sevenpt(*inGrid);
@@ -385,8 +382,7 @@ TestCurl::testWSCurl()
     CPPUNIT_ASSERT(!inTree.empty());
     CPPUNIT_ASSERT_EQUAL(math::Pow3(2*dim), int(inTree.activeVoxelCount()));
 
-    tools::Curl<VectorGrid> curl(*inGrid);
-    VectorGrid::Ptr curl_grid = curl.process();
+    VectorGrid::Ptr curl_grid = tools::curl(*inGrid);
     CPPUNIT_ASSERT_EQUAL(math::Pow3(2*dim), int(curl_grid->activeVoxelCount()));
 
     // test with a map
@@ -466,8 +462,7 @@ TestCurl::testWSCurlStencil()
     CPPUNIT_ASSERT(!inTree.empty());
     CPPUNIT_ASSERT_EQUAL(math::Pow3(2*dim), int(inTree.activeVoxelCount()));
 
-    tools::Curl<VectorGrid> curl(*inGrid);
-    VectorGrid::Ptr curl_grid = curl.process();
+    VectorGrid::Ptr curl_grid = tools::curl(*inGrid);
     CPPUNIT_ASSERT_EQUAL(math::Pow3(2*dim), int(curl_grid->activeVoxelCount()));
 
     // test with a map
