@@ -31,7 +31,7 @@
 #include <cppunit/extensions/HelperMacros.h>
 #include <openvdb/Types.h>
 #include <openvdb/openvdb.h>
-#include <openvdb/tools/Divergence.h>
+#include <openvdb/tools/GridOperators.h>
 
 #define ASSERT_DOUBLES_EXACTLY_EQUAL(expected, actual) \
     CPPUNIT_ASSERT_DOUBLES_EQUAL((expected), (actual), /*tolerance=*/0.0);
@@ -90,8 +90,7 @@ TestDivergence::testDivergenceTool()
     CPPUNIT_ASSERT(!inTree.empty());
     CPPUNIT_ASSERT_EQUAL(math::Pow3(2*dim), int(inTree.activeVoxelCount()));
 
-    tools::Divergence<VectorGrid> div(*inGrid);
-    FloatGrid::Ptr divGrid = div.process();
+    FloatGrid::Ptr divGrid = tools::divergence(*inGrid);
     CPPUNIT_ASSERT_EQUAL(math::Pow3(2*dim), int(divGrid->activeVoxelCount()));
 
     FloatGrid::ConstAccessor accessor = divGrid->getConstAccessor();
@@ -118,9 +117,9 @@ TestDivergence::testStaggeredDivergence()
 {
     // This test is slightly different than the one above for sanity
     // checking purposes.
-    
+
     using namespace openvdb;
-    
+
     typedef VectorGrid::ConstAccessor Accessor;
 
     VectorGrid::Ptr inGrid = VectorGrid::create();
@@ -140,8 +139,7 @@ TestDivergence::testStaggeredDivergence()
     CPPUNIT_ASSERT(!inTree.empty());
     CPPUNIT_ASSERT_EQUAL(math::Pow3(2*dim), int(inTree.activeVoxelCount()));
 
-    tools::Divergence<VectorGrid> div(*inGrid);
-    FloatGrid::Ptr divGrid = div.process();
+    FloatGrid::Ptr divGrid = tools::divergence(*inGrid);
     CPPUNIT_ASSERT_EQUAL(math::Pow3(2*dim), int(divGrid->activeVoxelCount()));
 
     FloatGrid::ConstAccessor accessor = divGrid->getConstAccessor();
