@@ -35,8 +35,10 @@
 #include <openvdb/io/Stream.h>
 #include <openvdb/Metadata.h>
 #include <openvdb/math/Transform.h>
+#include <openvdb/tools/LevelSetUtil.h> // for tools::sdfToFogVolume()
 #include <openvdb/version.h>
 #include <openvdb/openvdb.h>
+#include "util.h" // for unittest_util::makeSphere()
 #include <sys/types.h> // for stat()
 #include <sys/stat.h>
 #ifndef _WIN32
@@ -72,7 +74,7 @@ public:
     CPPUNIT_TEST(testHasGrid);
     CPPUNIT_TEST(testNameIterator);
     CPPUNIT_TEST(testReadOldFileFormat);
-    CPPUNIT_TEST(testZipCompression);
+    CPPUNIT_TEST(testCompression);
     CPPUNIT_TEST_SUITE_END();
 
     void testHeader();
@@ -96,7 +98,7 @@ public:
     void testHasGrid();
     void testNameIterator();
     void testReadOldFileFormat();
-    void testZipCompression();
+    void testCompression();
 };
 
 CPPUNIT_TEST_SUITE_REGISTRATION(TestFile);
@@ -219,6 +221,7 @@ TestFile::testWriteGrid()
 
     // Position the stream to beginning of the grid storage and read the grid.
     gd2.seekToGrid(istr);
+    Archive::readGridCompression(istr);
     gd2_grid->readMeta(istr);
     gd2_grid->readTransform(istr);
     gd2_grid->readTopology(istr);
@@ -335,6 +338,7 @@ TestFile::testWriteMultipleGrids()
 
     // Position the stream to beginning of the grid storage and read the grid.
     gd_in.seekToGrid(istr);
+    Archive::readGridCompression(istr);
     gd_in_grid->readMeta(istr);
     gd_in_grid->readTransform(istr);
     gd_in_grid->readTopology(istr);
@@ -379,6 +383,7 @@ TestFile::testWriteMultipleGrids()
 
     // Position the stream to beginning of the grid storage and read the grid.
     gd2_in.seekToGrid(istr);
+    Archive::readGridCompression(istr);
     gd2_in_grid->readMeta(istr);
     gd2_in_grid->readTransform(istr);
     gd2_in_grid->readTopology(istr);
@@ -933,6 +938,7 @@ TestFile::testEmptyGridIO()
     GridDescriptor file2gd = it->second;
     file2gd.seekToGrid(istr);
     GridBase::Ptr gd_grid = GridBase::createGrid(file2gd.gridType());
+    Archive::readGridCompression(istr);
     gd_grid->readMeta(istr);
     gd_grid->readTransform(istr);
     gd_grid->readTopology(istr);
@@ -950,6 +956,7 @@ TestFile::testEmptyGridIO()
     file2gd = it->second;
     file2gd.seekToGrid(istr);
     gd_grid = GridBase::createGrid(file2gd.gridType());
+    Archive::readGridCompression(istr);
     gd_grid->readMeta(istr);
     gd_grid->readTransform(istr);
     gd_grid->readTopology(istr);
@@ -1403,6 +1410,7 @@ TestFile::testReadGridMetadata()
                 statsMetadata = grid->getStatsMetadata(),
                 otherMetadata = grid->copyMeta(); // shallow copy
             CPPUNIT_ASSERT(!statsMetadata->empty());
+            statsMetadata->insertMeta(GridBase::META_FILE_COMPRESSION, StringMetadata(""));
             for (MetaMap::ConstMetaIterator it = grid->beginMeta(), end = grid->endMeta();
                 it != end; ++it)
             {
@@ -1816,7 +1824,7 @@ TestFile::testReadOldFileFormat()
 
 
 void
-TestFile::testZipCompression()
+TestFile::testCompression()
 {
     using namespace openvdb;
     using namespace openvdb::io;
@@ -1826,23 +1834,34 @@ TestFile::testZipCompression()
     // Register types.
     openvdb::initialize();
 
-    // Create a grid.
-    IntGrid::Ptr grid = IntGrid::create(/*background=*/0);
-    grid->fill(CoordBBox(Coord(0), Coord(100)), /*value=*/999, /*active=*/true);
+    // Create reference grids.
+    IntGrid::Ptr intGrid = IntGrid::create(/*background=*/0);
+    intGrid->fill(CoordBBox(Coord(0), Coord(49)), /*value=*/999, /*active=*/true);
+    intGrid->fill(CoordBBox(Coord(6), Coord(43)), /*value=*/0, /*active=*/false);
+
+    FloatGrid::Ptr lsGrid = createLevelSet<FloatGrid>();
+    unittest_util::makeSphere(/*dim=*/Coord(100), /*ctr=*/Vec3f(50, 50, 50), /*r=*/20.0,
+        *lsGrid, unittest_util::SPHERE_SPARSE_NARROW_BAND);
+    CPPUNIT_ASSERT_EQUAL(int(GRID_LEVEL_SET), int(lsGrid->getGridClass()));
+
+    FloatGrid::Ptr fogGrid = lsGrid->deepCopy();
+    tools::sdfToFogVolume(*fogGrid);
+    CPPUNIT_ASSERT_EQUAL(int(GRID_FOG_VOLUME), int(fogGrid->getGridClass()));
+
 
     GridPtrVec grids;
-    grids.push_back(grid);
+    grids.push_back(intGrid);
+    grids.push_back(lsGrid);
+    grids.push_back(fogGrid);
 
-    const char* filename = "/tmp/testZipCompression.vdb2";
+    const char* filename = "/tmp/testCompression.vdb2";
     boost::shared_ptr<const char> scopedFile(filename, ::remove);
 
     size_t uncompressedSize = 0;
     {
-        // Write the grid out to a file with ZIP compression disabled.
+        // Write the grids out to a file with compression disabled.
         io::File vdbfile(filename);
-        CPPUNIT_ASSERT(vdbfile.isCompressionEnabled());
-        vdbfile.setCompressionEnabled(false);
-        CPPUNIT_ASSERT(!vdbfile.isCompressionEnabled());
+        vdbfile.setCompressionFlags(io::COMPRESS_NONE);
         vdbfile.write(grids);
         vdbfile.close();
 
@@ -1852,45 +1871,71 @@ TestFile::testZipCompression()
         CPPUNIT_ASSERT_EQUAL(0, ::stat(filename, &buf));
         uncompressedSize = buf.st_size;
     }
-    {
-        // Verify that the grid can be read back successfully.
 
-        io::File vdbfile(filename);
-        vdbfile.open();
-        // Verify that the file is uncompressed.
-        CPPUNIT_ASSERT(!vdbfile.isCompressionEnabled());
+    // Write the grids out with various combinations of compression options
+    // and verify that they can be read back successfully.
+    // Currently, only bits 0 and 1 have meaning as compression flags
+    // (see io/Compression.h), so the valid combinations range from 0x0 to 0x3.
+    for (uint32_t flags = 0x0; flags <= 0x3; ++flags) {
 
-        // Verify that the original and input grids are equal.
-        GridPtrVecPtr inGrids = vdbfile.getGrids();
-        CPPUNIT_ASSERT_EQUAL(1, int(inGrids->size()));
-        const IntGrid::Ptr inGrid = gridPtrCast<IntGrid>((*inGrids)[0]);
-        CPPUNIT_ASSERT(inGrid.get() != NULL);
-        CPPUNIT_ASSERT(inGrid->tree().hasSameTopology(grid->tree()));
-        CPPUNIT_ASSERT_EQUAL(
-            grid->tree().getValue(Coord(0)),
-            inGrid->tree().getValue(Coord(0)));
+        if (flags != io::COMPRESS_NONE) {
+            io::File vdbfile(filename);
+            vdbfile.setCompressionFlags(flags);
+            vdbfile.write(grids);
+            vdbfile.close();
+        }
+        if (flags != io::COMPRESS_NONE) {
+            // Verify that the compressed file is significantly smaller than
+            // the uncompressed file.
+            size_t compressedSize = 0;
+            struct stat buf;
+            buf.st_size = 0;
+            CPPUNIT_ASSERT_EQUAL(0, ::stat(filename, &buf));
+            compressedSize = buf.st_size;
+            CPPUNIT_ASSERT(compressedSize < size_t(0.75 * uncompressedSize));
+        }
+        {
+            // Verify that the grids can be read back successfully.
 
-        // Verify that the only active value in the input grid is 999.
-        Int32 minVal = -1, maxVal = -1;
-        inGrid->evalMinMax(minVal, maxVal);
-        CPPUNIT_ASSERT_EQUAL(999, minVal);
-        CPPUNIT_ASSERT_EQUAL(999, maxVal);
-    }
-    {
-        // Write the grid out again, but with ZIP compression enabled.
-        io::File vdbfile(filename);
-        CPPUNIT_ASSERT(vdbfile.isCompressionEnabled());
-        vdbfile.write(grids);
-        vdbfile.close();
+            io::File vdbfile(filename);
+            vdbfile.open();
 
-        // Verify that the compressed file is significantly smaller than
-        // the uncompressed file.
-        size_t compressedSize = 0;
-        struct stat buf;
-        buf.st_size = 0;
-        CPPUNIT_ASSERT_EQUAL(0, ::stat(filename, &buf));
-        compressedSize = buf.st_size;
-        CPPUNIT_ASSERT(compressedSize < size_t(0.75 * uncompressedSize));
+            GridPtrVecPtr inGrids = vdbfile.getGrids();
+            CPPUNIT_ASSERT_EQUAL(3, int(inGrids->size()));
+
+            // Verify that the original and input grids are equal.
+            {
+                const IntGrid::Ptr grid = gridPtrCast<IntGrid>((*inGrids)[0]);
+                CPPUNIT_ASSERT(grid.get() != NULL);
+                CPPUNIT_ASSERT_EQUAL(int(intGrid->getGridClass()), int(grid->getGridClass()));
+
+                CPPUNIT_ASSERT(grid->tree().hasSameTopology(intGrid->tree()));
+
+                CPPUNIT_ASSERT_EQUAL(
+                    intGrid->tree().getValue(Coord(0)),
+                    grid->tree().getValue(Coord(0)));
+
+                // Verify that the only active value in this grid is 999.
+                Int32 minVal = -1, maxVal = -1;
+                grid->evalMinMax(minVal, maxVal);
+                CPPUNIT_ASSERT_EQUAL(999, minVal);
+                CPPUNIT_ASSERT_EQUAL(999, maxVal);
+            }
+            for (int idx = 1; idx <= 2; ++idx) {
+                const FloatGrid::Ptr
+                    grid = gridPtrCast<FloatGrid>((*inGrids)[idx]),
+                    refGrid = gridPtrCast<FloatGrid>(grids[idx]);
+                CPPUNIT_ASSERT(grid.get() != NULL);
+                CPPUNIT_ASSERT_EQUAL(int(refGrid->getGridClass()), int(grid->getGridClass()));
+
+                CPPUNIT_ASSERT(grid->tree().hasSameTopology(refGrid->tree()));
+
+                FloatGrid::ConstAccessor refAcc = refGrid->getConstAccessor();
+                for (FloatGrid::ValueAllCIter it = grid->cbeginValueAll(); it; ++it) {
+                    CPPUNIT_ASSERT_EQUAL(refAcc.getValue(it.getCoord()), *it);
+                }
+            }
+        }
     }
 }
 
