@@ -32,7 +32,7 @@
 #include <cppunit/extensions/HelperMacros.h>
 #include <openvdb/Types.h>
 #include <openvdb/openvdb.h>
-#include <openvdb/tools/Gradient.h>
+#include <openvdb/tools/GridOperators.h>
 #include "util.h" // for unittest_util::makeSphere()
 
 #define ASSERT_DOUBLES_EXACTLY_EQUAL(expected, actual) \
@@ -52,6 +52,7 @@ public:
     CPPUNIT_TEST(testWSGradientNormSqr);             // Gradient Norm Sqr (world space only)
     CPPUNIT_TEST(testWSGradientNormSqrStencil);      // Gradient Norm Sqr (world space only)
     CPPUNIT_TEST(testGradientTool);                  // Gradient tool
+    CPPUNIT_TEST(testIntersectsIsoValue);          // zero-crossing
     CPPUNIT_TEST(testOldStyleStencils);              // old stencil impl - deprecate
 
     CPPUNIT_TEST_SUITE_END();
@@ -63,6 +64,7 @@ public:
     void testWSGradientNormSqr();
     void testWSGradientNormSqrStencil();
     void testGradientTool();
+    void testIntersectsIsoValue();
     void testOldStyleStencils();
 };
 
@@ -224,7 +226,7 @@ TestGradient::testWSGradient()
         result = math::Gradient<math::UniformScaleMap, math::CD_2ND>::result(
             map, inAccessor, xyz);
         CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, result.length(), /*tolerance=*/0.01);
-        rotated_map = map.rotate(1.5, math::X_AXIS);
+        rotated_map = map.preRotate(1.5, math::X_AXIS);
         // verify the new map is an affine map
         CPPUNIT_ASSERT(rotated_map->type() == math::AffineMap::mapType());
         math::AffineMap::Ptr affine_map =
@@ -354,7 +356,7 @@ TestGradient::testWSGradientStencil()
         result = math::Gradient<math::UniformScaleMap, math::CD_2ND>::result(
             map, stencil);
         CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, result.length(), /*tolerance=*/0.01);
-        rotated_map = map.rotate(1.5, math::X_AXIS);
+        rotated_map = map.preRotate(1.5, math::X_AXIS);
         // verify the new map is an affine map
         CPPUNIT_ASSERT(rotated_map->type() == math::AffineMap::mapType());
         math::AffineMap::Ptr affine_map =
@@ -567,13 +569,105 @@ TestGradient::testGradientTool()
     const Coord xyz(10, 20, 30);
 
 
-    tools::Gradient<FloatGrid> gradient(*grid);
-    tools::Gradient<FloatGrid>::OutGridType::Ptr grad = gradient.process();
+    Vec3SGrid::Ptr grad = tools::gradient(*grid);
     CPPUNIT_ASSERT_EQUAL(int(tree.activeVoxelCount()), int(grad->activeVoxelCount()));
     CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, grad->getConstAccessor().getValue(xyz).length(),
         /*tolerance=*/0.01);
 }
 
+void
+TestGradient::testIntersectsIsoValue()
+{
+    using namespace openvdb;
+    {// test zero crossing in -x
+        FloatGrid  grid(/*backgroundValue=*/5.0);
+        FloatTree& tree = grid.tree();
+        Coord xyz(2,-5,60);
+        tree.setValue(xyz, 1.3f);
+        tree.setValue(xyz.offsetBy(-1,0,0), -2.0f);
+        math::SevenPointStencil<FloatGrid>  stencil(grid);
+        stencil.moveTo(xyz);
+        CPPUNIT_ASSERT( stencil.intersects(     ));
+        CPPUNIT_ASSERT( stencil.intersects( 0.0f));
+        CPPUNIT_ASSERT( stencil.intersects( 2.0f));
+        CPPUNIT_ASSERT(!stencil.intersects( 5.5f));
+        CPPUNIT_ASSERT(!stencil.intersects(-2.5f));
+    }
+    {// test zero crossing in +x
+        FloatGrid  grid(/*backgroundValue=*/5.0);
+        FloatTree& tree = grid.tree();
+        Coord xyz(2,-5,60);
+        tree.setValue(xyz, 1.3f);
+        tree.setValue(xyz.offsetBy(1,0,0), -2.0f);
+        math::SevenPointStencil<FloatGrid>  stencil(grid);
+        stencil.moveTo(xyz);
+        CPPUNIT_ASSERT(stencil.intersects());
+    }
+    {// test zero crossing in -y
+        FloatGrid  grid(/*backgroundValue=*/5.0);
+        FloatTree& tree = grid.tree();
+        Coord xyz(2,-5,60);
+        tree.setValue(xyz, 1.3f);
+        tree.setValue(xyz.offsetBy(0,-1,0), -2.0f);
+        math::SevenPointStencil<FloatGrid>  stencil(grid);
+        stencil.moveTo(xyz);
+        CPPUNIT_ASSERT(stencil.intersects());
+    }
+    {// test zero crossing in y
+        FloatGrid  grid(/*backgroundValue=*/5.0);
+        FloatTree& tree = grid.tree();
+        Coord xyz(2,-5,60);
+        tree.setValue(xyz, 1.3f);
+        tree.setValue(xyz.offsetBy(0,1,0), -2.0f);
+        math::SevenPointStencil<FloatGrid>  stencil(grid);
+        stencil.moveTo(xyz);
+        CPPUNIT_ASSERT(stencil.intersects());
+    }
+    {// test zero crossing in -z
+        FloatGrid  grid(/*backgroundValue=*/5.0);
+        FloatTree& tree = grid.tree();
+        Coord xyz(2,-5,60);
+        tree.setValue(xyz, 1.3f);
+        tree.setValue(xyz.offsetBy(0,0,-1), -2.0f);
+        math::SevenPointStencil<FloatGrid>  stencil(grid);
+        stencil.moveTo(xyz);
+        CPPUNIT_ASSERT(stencil.intersects());
+    }
+    {// test zero crossing in z
+        FloatGrid  grid(/*backgroundValue=*/5.0);
+        FloatTree& tree = grid.tree();
+        Coord xyz(2,-5,60);
+        tree.setValue(xyz, 1.3f);
+        tree.setValue(xyz.offsetBy(0,0,1), -2.0f);
+        math::SevenPointStencil<FloatGrid>  stencil(grid);
+        stencil.moveTo(xyz);
+        CPPUNIT_ASSERT(stencil.intersects());
+    }
+    {// test zero crossing in -x & z
+        FloatGrid  grid(/*backgroundValue=*/5.0);
+        FloatTree& tree = grid.tree();
+        Coord xyz(2,-5,60);
+        tree.setValue(xyz, 1.3f);
+        tree.setValue(xyz.offsetBy(-1,0,1), -2.0f);
+        math::SevenPointStencil<FloatGrid>  stencil(grid);
+        stencil.moveTo(xyz);
+        CPPUNIT_ASSERT(!stencil.intersects());
+    }
+    {// test zero multiple crossings
+        FloatGrid  grid(/*backgroundValue=*/5.0);
+        FloatTree& tree = grid.tree();
+        Coord xyz(2,-5,60);
+        tree.setValue(xyz, 1.3f);
+        tree.setValue(xyz.offsetBy(-1, 0, 1), -1.0f);
+        tree.setValue(xyz.offsetBy( 0, 0, 1), -2.0f);
+        tree.setValue(xyz.offsetBy( 0, 1, 0), -3.0f);
+        tree.setValue(xyz.offsetBy( 0, 0,-1), -2.0f);
+        math::SevenPointStencil<FloatGrid>  stencil(grid);
+        stencil.moveTo(xyz);
+        CPPUNIT_ASSERT(stencil.intersects());
+    }
+}
+
 void
 TestGradient::testOldStyleStencils()
 {
@@ -608,7 +702,6 @@ TestGradient::testOldStyleStencils()
 
 }
 
-
 // Copyright (c) 2012 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
