@@ -81,6 +81,8 @@ public:
     typedef boost::shared_ptr<ProxyTree> Ptr;
     typedef boost::shared_ptr<const ProxyTree> ConstPtr;
 
+    static const openvdb::Index DEPTH = 0;
+
     ProxyTree() {}
     ProxyTree(const ValueType&) {}
     virtual ~ProxyTree() {}
@@ -208,12 +210,12 @@ TestGrid::testTransform()
 
     CPPUNIT_ASSERT(grid.transformPtr());
 
-    // Verify that calling Transform-related Grid methods (Grid::voxelDimensions(), etc.)
+    // Verify that calling Transform-related Grid methods (Grid::voxelSize(), etc.)
     // is the same as calling those methods on the Transform.
 
-    CPPUNIT_ASSERT(grid.transform().voxelDimensions().eq(grid.voxelDimensions()));
-    CPPUNIT_ASSERT(grid.transform().voxelDimensions(openvdb::Vec3d(0.1, 0.2, 0.3)).eq(
-        grid.voxelDimensions(openvdb::Vec3d(0.1, 0.2, 0.3))));
+    CPPUNIT_ASSERT(grid.transform().voxelSize().eq(grid.voxelSize()));
+    CPPUNIT_ASSERT(grid.transform().voxelSize(openvdb::Vec3d(0.1, 0.2, 0.3)).eq(
+        grid.voxelSize(openvdb::Vec3d(0.1, 0.2, 0.3))));
 
     CPPUNIT_ASSERT(grid.transform().indexToWorld(openvdb::Vec3d(0.1, 0.2, 0.3)).eq(
         grid.indexToWorld(openvdb::Vec3d(0.1, 0.2, 0.3))));
