@@ -123,7 +123,7 @@ TestMaps::testRotation()
     CPPUNIT_ASSERT(*rotation_two != *rotation);
 
     CPPUNIT_ASSERT_DOUBLES_EQUAL(rotation->determinant(), 1, TOL);
-    
+
     CPPUNIT_ASSERT(rotation_two->hasUniformScale());
 
     /// apply the map forward
@@ -287,7 +287,7 @@ TestMaps::testFrustum()
 {
     using namespace openvdb::math;
 
-    openvdb::CoordBBox bbox(Coord(0), Coord(100));
+    openvdb::BBoxd bbox(Vec3d(0), Vec3d(100));
     NonlinearFrustumMap frustum(bbox, 1./6., 5);
     /// frustum will have depth, far plane - near plane = 5
     /// the frustum has width 1 in the front and 6 in the back
@@ -295,12 +295,19 @@ TestMaps::testFrustum()
     Vec3d trans(2,2,2);
     NonlinearFrustumMap::Ptr map =
         boost::static_pointer_cast<NonlinearFrustumMap, MapBase>(
-            frustum.scale(Vec3d(10,10,10))->translate(trans));
+            frustum.preScale(Vec3d(10,10,10))->translate(trans));
 
     CPPUNIT_ASSERT(!map->hasUniformScale());
+
+    Vec3d result;
+    result = map->voxelSize();
+    
+    CPPUNIT_ASSERT( isApproxEqual(result.x(), 0.1));
+    CPPUNIT_ASSERT( isApproxEqual(result.y(), 0.1));
+    CPPUNIT_ASSERT( isApproxEqual(result.z(), 0.5, 0.0001));
     //--------- Front face
     Vec3d corner(0,0,0);
-    Vec3d result  = map->applyMap(corner);
+    result  = map->applyMap(corner);
     CPPUNIT_ASSERT(isApproxEqual(result, Vec3d(-5, -5, 0) + trans));
 
     corner = Vec3d(100,0,0);
@@ -392,13 +399,14 @@ TestMaps::testFrustum()
     // check that the linear map inside the frustum is a simple affine map (i.e. has no shear)
     CPPUNIT_ASSERT(frustumMap_from_camera.hasSimpleAffine());
 
-    // construct a frustum from the frustum map
-    Frustum<double> frustum_from_map(frustumMap_from_camera);
-    // construct the same frustum directly
-    Vec3d left = up.cross(direction);
-    left.normalize();
-    left *= aspect * up.length();
-    Frustum<double> frustum_from_camera(position, direction, up, left, z_near, z_near+depth);
+    //-- The Frustum Class has been deprecated
+    // // construct a frustum from the frustum map
+//     Frustum<double> frustum_from_map(frustumMap_from_camera);
+//     // construct the same frustum directly
+//     Vec3d left = up.cross(direction);
+//     left.normalize();
+//     left *= aspect * up.length();
+//     Frustum<double> frustum_from_camera(position, direction, up, left, z_near, z_near+depth);
 }
 
 
@@ -412,7 +420,7 @@ TestMaps::testCalcBoundingBox()
     openvdb::BBoxd expected;
     {
         AffineMap affine;
-        affine.accumScale(Vec3d(2,2,2));
+        affine.accumPreScale(Vec3d(2,2,2));
 
         openvdb::util::calculateBounds<AffineMap>(affine, world_bbox, voxel_bbox);
 
@@ -420,7 +428,7 @@ TestMaps::testCalcBoundingBox()
         CPPUNIT_ASSERT(isApproxEqual(voxel_bbox.min(), expected.min()));
         CPPUNIT_ASSERT(isApproxEqual(voxel_bbox.max(), expected.max()));
 
-        affine.accumTranslation(Vec3d(1,1,1));
+        affine.accumPostTranslation(Vec3d(1,1,1));
         openvdb::util::calculateBounds<AffineMap>(affine, world_bbox, voxel_bbox);
         expected = openvdb::BBoxd(Vec3d(-0.5,-0.5,-0.5), Vec3d(0, 0, 0));
         CPPUNIT_ASSERT(isApproxEqual(voxel_bbox.min(), expected.min()));
@@ -428,8 +436,8 @@ TestMaps::testCalcBoundingBox()
     }
     {
         AffineMap affine;
-        affine.accumScale(Vec3d(2,2,2));
-        affine.accumTranslation(Vec3d(1,1,1));
+        affine.accumPreScale(Vec3d(2,2,2));
+        affine.accumPostTranslation(Vec3d(1,1,1));
         // test a sphere:
         Vec3d center(0,0,0);
         double radius = 10;
@@ -441,9 +449,9 @@ TestMaps::testCalcBoundingBox()
     }
     {
         AffineMap affine;
-        affine.accumScale(Vec3d(2,2,2));
+        affine.accumPreScale(Vec3d(2,2,2));
         double pi = 4.*atan(1.);
-        affine.accumRotation(X_AXIS, pi/4.);
+        affine.accumPreRotation(X_AXIS, pi/4.);
         Vec3d center(0,0,0);
         double radius = 10;
 
@@ -454,9 +462,9 @@ TestMaps::testCalcBoundingBox()
     }
     {
         AffineMap affine;
-        affine.accumScale(Vec3d(2,1,1));
+        affine.accumPreScale(Vec3d(2,1,1));
         double pi = 4.*atan(1.);
-        affine.accumRotation(X_AXIS, pi/4.);
+        affine.accumPreRotation(X_AXIS, pi/4.);
         Vec3d center(0,0,0);
         double radius = 10;
 
@@ -467,10 +475,10 @@ TestMaps::testCalcBoundingBox()
      }
      {
         AffineMap affine;
-        affine.accumScale(Vec3d(2,1,1));
+        affine.accumPreScale(Vec3d(2,1,1));
         double pi = 4.*atan(1.);
-        affine.accumRotation(X_AXIS, pi/4.);
-        affine.accumTranslation(Vec3d(1,1,1));
+        affine.accumPreRotation(X_AXIS, pi/4.);
+        affine.accumPostTranslation(Vec3d(1,1,1));
         Vec3d center(1,1,1);
         double radius = 10;
 
@@ -480,11 +488,11 @@ TestMaps::testCalcBoundingBox()
         CPPUNIT_ASSERT(isApproxEqual(voxel_bbox.max(), expected.max()));
      }
      {
-         openvdb::CoordBBox bbox(Coord(0), Coord(100));
+         openvdb::BBoxd bbox(Vec3d(0), Vec3d(100));
          NonlinearFrustumMap frustum(bbox, 2, 5);
          NonlinearFrustumMap::Ptr map =
              boost::static_pointer_cast<NonlinearFrustumMap, MapBase>(
-                 frustum.scale(Vec3d(2,2,2)));
+                 frustum.preScale(Vec3d(2,2,2)));
          Vec3d center(20,20,10);
          double radius(1);
 
