@@ -34,7 +34,7 @@
 #include <openvdb/openvdb.h>
 #include <openvdb/math/Stencils.h>
 #include <openvdb/math/Operators.h>
-#include <openvdb/tools/MeanCurvature.h>
+#include <openvdb/tools/GridOperators.h>
 #include "util.h" // for unittest_util::makeSphere()
 
 #define ASSERT_DOUBLES_EXACTLY_EQUAL(expected, actual) \
@@ -526,8 +526,7 @@ TestMeanCurvature::testMeanCurvatureTool()
     unittest_util::makeSphere<FloatGrid>(dim, center, radius, *grid, unittest_util::SPHERE_DENSE);
 
     CPPUNIT_ASSERT(!tree.empty());
-    tools::MeanCurvature<FloatGrid> curvature(*grid);
-    FloatGrid::Ptr curv = curvature.process();
+    FloatGrid::Ptr curv = tools::meanCurvature(*grid);
     FloatGrid::ConstAccessor accessor = curv->getConstAccessor();
 
     Coord xyz(35,30,30);
