@@ -102,7 +102,7 @@ public:
         openvdb::Coord &min= bbox.min(), &max = bbox.max();
         for (int n=0, e=this->size(); n<e; ++n) {
             const openvdb::Vec3d xyz = grid.worldToIndex(this->pos(n));
-            const openvdb::Real   r  = this->radius(n) / grid. voxelDimensions()[0];
+            const openvdb::Real   r  = this->radius(n) / grid.voxelSize()[0];
             for (int i=0; i<3; ++i) {
                 min[i] = openvdb::math::Min(min[i], openvdb::math::Floor(xyz[i] - r));
                 max[i] = openvdb::math::Max(max[i], openvdb::math::Ceil( xyz[i] + r));
@@ -144,11 +144,10 @@ TestParticlesToLevelSet::testMyParticleList()
     ASSERT_DOUBLES_EXACTLY_EQUAL(0 , pa.vel(1)[1]);
     ASSERT_DOUBLES_EXACTLY_EQUAL(0 , pa.vel(1)[2]);
     ASSERT_DOUBLES_EXACTLY_EQUAL(3 , pa.radius(1));
-
-    const float voxelSize = 0.5f;
-    openvdb::FloatGrid grid(/*background=*/2.0f);
-    grid.setTransform(openvdb::math::Transform::createLinearTransform(voxelSize));
-    openvdb::CoordBBox bbox = pa.getBBox(grid);
+    
+    const float voxelSize = 0.5f, halfWidth = 4.0f;
+    openvdb::FloatGrid::Ptr ls = openvdb::createLevelSet<openvdb::FloatGrid>(voxelSize, halfWidth);
+    openvdb::CoordBBox bbox = pa.getBBox(*ls);
     ASSERT_DOUBLES_EXACTLY_EQUAL((10-2)/voxelSize, bbox.min()[0]);
     ASSERT_DOUBLES_EXACTLY_EQUAL((10-2)/voxelSize, bbox.min()[1]);
     ASSERT_DOUBLES_EXACTLY_EQUAL((10-2)/voxelSize, bbox.min()[2]);
@@ -161,12 +160,8 @@ TestParticlesToLevelSet::testMyParticleList()
 void
 TestParticlesToLevelSet::testRasterizeSpheres()
 {
-    const int halfWidth = 2;
-    const float voxelSize = 1.0f, gamma = halfWidth*voxelSize;
-    openvdb::FloatGrid grid(gamma);
-    //openvdb::FloatTree& tree = grid.treeRW();oops
-    grid.setTransform(openvdb::math::Transform::createLinearTransform(voxelSize));
-    grid.setGridClass(openvdb::GRID_LEVEL_SET);
+    const float voxelSize = 1.0f, halfWidth = 2.0f;
+    openvdb::FloatGrid::Ptr ls = openvdb::createLevelSet<openvdb::FloatGrid>(voxelSize, halfWidth);
 
     MyParticleList pa;
     pa.add(openvdb::Vec3R(10,10,10), 2);
@@ -184,60 +179,60 @@ TestParticlesToLevelSet::testRasterizeSpheres()
     pa.add(openvdb::Vec3R(35.5,31,31), 5);
     pa.add(openvdb::Vec3R(36.0,31,31), 5);
     CPPUNIT_ASSERT_EQUAL(13, pa.size());
-
-    openvdb::util::NullInterrupter boss;
-    openvdb::tools::ParticlesToLevelSet<openvdb::FloatGrid, MyParticleList> raster(grid, boss);
+    
+    openvdb::tools::ParticlesToLevelSet<openvdb::FloatGrid, MyParticleList> raster(*ls);
     raster.setGrainSize(1);//a value of zero disables threading
     raster.rasterizeSpheres(pa);
 
-    //grid.tree().print(std::cout,4);
-    //this->writeGrid(grid, "testRasterizeSpheres");
+    //ls->tree().print(std::cout,4);
+    //this->writeGrid(*ls, "testRasterizeSpheres");
 
     ASSERT_DOUBLES_EXACTLY_EQUAL(halfWidth * voxelSize,
-        grid.tree().getValue(openvdb::Coord( 0, 0, 0)));
+        ls->tree().getValue(openvdb::Coord( 0, 0, 0)));
 
-    ASSERT_DOUBLES_EXACTLY_EQUAL( 2, grid.tree().getValue(openvdb::Coord( 6,10,10)));
-    ASSERT_DOUBLES_EXACTLY_EQUAL( 1, grid.tree().getValue(openvdb::Coord( 7,10,10)));
-    ASSERT_DOUBLES_EXACTLY_EQUAL( 0, grid.tree().getValue(openvdb::Coord( 8,10,10)));
-    ASSERT_DOUBLES_EXACTLY_EQUAL(-1, grid.tree().getValue(openvdb::Coord( 9,10,10)));
-    ASSERT_DOUBLES_EXACTLY_EQUAL(-2, grid.tree().getValue(openvdb::Coord(10,10,10)));
-    ASSERT_DOUBLES_EXACTLY_EQUAL(-1, grid.tree().getValue(openvdb::Coord(11,10,10)));
-    ASSERT_DOUBLES_EXACTLY_EQUAL( 0, grid.tree().getValue(openvdb::Coord(12,10,10)));
-    ASSERT_DOUBLES_EXACTLY_EQUAL( 1, grid.tree().getValue(openvdb::Coord(13,10,10)));
-    ASSERT_DOUBLES_EXACTLY_EQUAL( 2, grid.tree().getValue(openvdb::Coord(14,10,10)));
-
-    ASSERT_DOUBLES_EXACTLY_EQUAL( 2, grid.tree().getValue(openvdb::Coord(20,16,20)));
-    ASSERT_DOUBLES_EXACTLY_EQUAL( 1, grid.tree().getValue(openvdb::Coord(20,17,20)));
-    ASSERT_DOUBLES_EXACTLY_EQUAL( 0, grid.tree().getValue(openvdb::Coord(20,18,20)));
-    ASSERT_DOUBLES_EXACTLY_EQUAL(-1, grid.tree().getValue(openvdb::Coord(20,19,20)));
-    ASSERT_DOUBLES_EXACTLY_EQUAL(-2, grid.tree().getValue(openvdb::Coord(20,20,20)));
-    ASSERT_DOUBLES_EXACTLY_EQUAL(-1, grid.tree().getValue(openvdb::Coord(20,21,20)));
-    ASSERT_DOUBLES_EXACTLY_EQUAL( 0, grid.tree().getValue(openvdb::Coord(20,22,20)));
-    ASSERT_DOUBLES_EXACTLY_EQUAL( 1, grid.tree().getValue(openvdb::Coord(20,23,20)));
-    ASSERT_DOUBLES_EXACTLY_EQUAL( 2, grid.tree().getValue(openvdb::Coord(20,24,20)));
+    ASSERT_DOUBLES_EXACTLY_EQUAL( 2, ls->tree().getValue(openvdb::Coord( 6,10,10)));
+    ASSERT_DOUBLES_EXACTLY_EQUAL( 1, ls->tree().getValue(openvdb::Coord( 7,10,10)));
+    ASSERT_DOUBLES_EXACTLY_EQUAL( 0, ls->tree().getValue(openvdb::Coord( 8,10,10)));
+    ASSERT_DOUBLES_EXACTLY_EQUAL(-1, ls->tree().getValue(openvdb::Coord( 9,10,10)));
+    ASSERT_DOUBLES_EXACTLY_EQUAL(-2, ls->tree().getValue(openvdb::Coord(10,10,10)));
+    ASSERT_DOUBLES_EXACTLY_EQUAL(-1, ls->tree().getValue(openvdb::Coord(11,10,10)));
+    ASSERT_DOUBLES_EXACTLY_EQUAL( 0, ls->tree().getValue(openvdb::Coord(12,10,10)));
+    ASSERT_DOUBLES_EXACTLY_EQUAL( 1, ls->tree().getValue(openvdb::Coord(13,10,10)));
+    ASSERT_DOUBLES_EXACTLY_EQUAL( 2, ls->tree().getValue(openvdb::Coord(14,10,10)));
+    
+    ASSERT_DOUBLES_EXACTLY_EQUAL( 2, ls->tree().getValue(openvdb::Coord(20,16,20)));
+    ASSERT_DOUBLES_EXACTLY_EQUAL( 1, ls->tree().getValue(openvdb::Coord(20,17,20)));
+    ASSERT_DOUBLES_EXACTLY_EQUAL( 0, ls->tree().getValue(openvdb::Coord(20,18,20)));
+    ASSERT_DOUBLES_EXACTLY_EQUAL(-1, ls->tree().getValue(openvdb::Coord(20,19,20)));
+    ASSERT_DOUBLES_EXACTLY_EQUAL(-2, ls->tree().getValue(openvdb::Coord(20,20,20)));
+    ASSERT_DOUBLES_EXACTLY_EQUAL(-1, ls->tree().getValue(openvdb::Coord(20,21,20)));
+    ASSERT_DOUBLES_EXACTLY_EQUAL( 0, ls->tree().getValue(openvdb::Coord(20,22,20)));
+    ASSERT_DOUBLES_EXACTLY_EQUAL( 1, ls->tree().getValue(openvdb::Coord(20,23,20)));
+    ASSERT_DOUBLES_EXACTLY_EQUAL( 2, ls->tree().getValue(openvdb::Coord(20,24,20)));
     {// full but slow test of all voxels
-        openvdb::CoordBBox bbox = pa.getBBox(grid);
-        bbox.expand(halfWidth+1);
-        size_t count=0;
+        openvdb::CoordBBox bbox = pa.getBBox(*ls);
+        bbox.expand(static_cast<int>(halfWidth)+1);
+        openvdb::Index64 count=0;
+        const float outside = ls->background(), inside = -outside;
         const openvdb::Coord &min=bbox.min(), &max=bbox.max();
         for (openvdb::Coord ijk=min; ijk[0]<max[0]; ++ijk[0]) {
             for (ijk[1]=min[1]; ijk[1]<max[1]; ++ijk[1]) {
                 for (ijk[2]=min[2]; ijk[2]<max[2]; ++ijk[2]) {
-                    const openvdb::Vec3d xyz = grid.indexToWorld(ijk.asVec3d());
+                    const openvdb::Vec3d xyz = ls->indexToWorld(ijk.asVec3d());
                     double dist = (xyz-pa.pos(0)).length()-pa.radius(0);
                     for (int i=1, s=pa.size(); i<s; ++i) {
                         dist=openvdb::math::Min(dist,(xyz-pa.pos(i)).length()-pa.radius(i));
                     }
-                    const float val = grid.tree().getValue(ijk);
-                    if (dist >= gamma) {
-                        CPPUNIT_ASSERT_DOUBLES_EQUAL( gamma, val, 0.0001);
-                        CPPUNIT_ASSERT(grid.tree().isValueOff(ijk));
-                    } else if (dist <= -gamma) {
-                        CPPUNIT_ASSERT_DOUBLES_EQUAL(-gamma, val, 0.0001);
-                        CPPUNIT_ASSERT(grid.tree().isValueOff(ijk));
+                    const float val = ls->tree().getValue(ijk);
+                    if (dist >= outside) {
+                        CPPUNIT_ASSERT_DOUBLES_EQUAL(outside, val, 0.0001);
+                        CPPUNIT_ASSERT(ls->tree().isValueOff(ijk));
+                    } else if( dist <= inside ) {
+                        CPPUNIT_ASSERT_DOUBLES_EQUAL(inside, val, 0.0001);
+                        CPPUNIT_ASSERT(ls->tree().isValueOff(ijk));
                     } else {
                         CPPUNIT_ASSERT_DOUBLES_EQUAL(  dist, val, 0.0001);
-                        CPPUNIT_ASSERT(grid.tree().isValueOn(ijk));
+                        CPPUNIT_ASSERT(ls->tree().isValueOn(ijk));
                         ++count;
                     }
                 }
@@ -245,8 +240,8 @@ TestParticlesToLevelSet::testRasterizeSpheres()
         }
         //std::cerr << "\nExpected active voxel count = " << count
         //    << ", actual active voxle count = "
-        //    << grid.activeVoxelCount() << std::endl;
-        CPPUNIT_ASSERT_EQUAL(count, grid.activeVoxelCount());
+        //    << ls->activeVoxelCount() << std::endl;
+        CPPUNIT_ASSERT_EQUAL(count, ls->activeVoxelCount());
     }
 }
 
@@ -254,14 +249,8 @@ TestParticlesToLevelSet::testRasterizeSpheres()
 void
 TestParticlesToLevelSet::testRasterizeSpheresAndId()
 {
-    const int halfWidth = 2;
-    const float voxelSize = 1.0f, gamma = halfWidth*voxelSize;
-    typedef openvdb::FloatGrid GridT;
-    GridT grid(gamma);
-    //openvdb::FloatTree& tree = grid.treeRW();//oops
-    //const openvdb::FloatTree& tree = grid.tree();
-    grid.setTransform(openvdb::math::Transform::createLinearTransform(voxelSize));
-    grid.setGridClass(openvdb::GRID_LEVEL_SET);
+    const float voxelSize = 1.0f, halfWidth = 2.0f;
+    openvdb::FloatGrid::Ptr ls = openvdb::createLevelSet<openvdb::FloatGrid>(voxelSize, halfWidth);
 
     MyParticleList pa(0.5f);
     pa.add(openvdb::Vec3R(10,10,10), 4);
@@ -279,10 +268,9 @@ TestParticlesToLevelSet::testRasterizeSpheresAndId()
     pa.add(openvdb::Vec3R(35.5,31,31),10);
     pa.add(openvdb::Vec3R(36.0,31,31),10);
     CPPUNIT_ASSERT_EQUAL(13, pa.size());
-
-    openvdb::util::NullInterrupter boss;
-    typedef openvdb::tools::ParticlesToLevelSetAndId<GridT, MyParticleList> RasterT;
-    RasterT raster(grid, boss);
+    
+    typedef openvdb::tools::ParticlesToLevelSetAndId<openvdb::FloatGrid, MyParticleList> RasterT;
+    RasterT raster(*ls);
     raster.setGrainSize(1);//a value of zero disables threading
     raster.rasterizeSpheres(pa);
     const RasterT::IndxGridT::Ptr id = raster.rasterizeSpheres(pa);
@@ -297,40 +285,41 @@ TestParticlesToLevelSet::testRasterizeSpheresAndId()
 
     //grid.tree().print(std::cout,4);
     //id->print(std::cout,4);
-    //this->writeGrid(grid, "testRasterizeSpheres");
+    //this->writeGrid(*ls, "testRasterizeSpheres");
 
     ASSERT_DOUBLES_EXACTLY_EQUAL(halfWidth * voxelSize,
-        grid.tree().getValue(openvdb::Coord( 0, 0, 0)));
+                                 ls->tree().getValue(openvdb::Coord( 0, 0, 0)));
 
-    ASSERT_DOUBLES_EXACTLY_EQUAL( 2, grid.tree().getValue(openvdb::Coord( 6,10,10)));
-    ASSERT_DOUBLES_EXACTLY_EQUAL( 1, grid.tree().getValue(openvdb::Coord( 7,10,10)));
-    ASSERT_DOUBLES_EXACTLY_EQUAL( 0, grid.tree().getValue(openvdb::Coord( 8,10,10)));
-    ASSERT_DOUBLES_EXACTLY_EQUAL(-1, grid.tree().getValue(openvdb::Coord( 9,10,10)));
-    ASSERT_DOUBLES_EXACTLY_EQUAL(-2, grid.tree().getValue(openvdb::Coord(10,10,10)));
-    ASSERT_DOUBLES_EXACTLY_EQUAL(-1, grid.tree().getValue(openvdb::Coord(11,10,10)));
-    ASSERT_DOUBLES_EXACTLY_EQUAL( 0, grid.tree().getValue(openvdb::Coord(12,10,10)));
-    ASSERT_DOUBLES_EXACTLY_EQUAL( 1, grid.tree().getValue(openvdb::Coord(13,10,10)));
-    ASSERT_DOUBLES_EXACTLY_EQUAL( 2, grid.tree().getValue(openvdb::Coord(14,10,10)));
+    ASSERT_DOUBLES_EXACTLY_EQUAL( 2, ls->tree().getValue(openvdb::Coord( 6,10,10)));
+    ASSERT_DOUBLES_EXACTLY_EQUAL( 1, ls->tree().getValue(openvdb::Coord( 7,10,10)));
+    ASSERT_DOUBLES_EXACTLY_EQUAL( 0, ls->tree().getValue(openvdb::Coord( 8,10,10)));
+    ASSERT_DOUBLES_EXACTLY_EQUAL(-1, ls->tree().getValue(openvdb::Coord( 9,10,10)));
+    ASSERT_DOUBLES_EXACTLY_EQUAL(-2, ls->tree().getValue(openvdb::Coord(10,10,10)));
+    ASSERT_DOUBLES_EXACTLY_EQUAL(-1, ls->tree().getValue(openvdb::Coord(11,10,10)));
+    ASSERT_DOUBLES_EXACTLY_EQUAL( 0, ls->tree().getValue(openvdb::Coord(12,10,10)));
+    ASSERT_DOUBLES_EXACTLY_EQUAL( 1, ls->tree().getValue(openvdb::Coord(13,10,10)));
+    ASSERT_DOUBLES_EXACTLY_EQUAL( 2, ls->tree().getValue(openvdb::Coord(14,10,10)));
 
-    ASSERT_DOUBLES_EXACTLY_EQUAL( 2, grid.tree().getValue(openvdb::Coord(20,16,20)));
-    ASSERT_DOUBLES_EXACTLY_EQUAL( 1, grid.tree().getValue(openvdb::Coord(20,17,20)));
-    ASSERT_DOUBLES_EXACTLY_EQUAL( 0, grid.tree().getValue(openvdb::Coord(20,18,20)));
-    ASSERT_DOUBLES_EXACTLY_EQUAL(-1, grid.tree().getValue(openvdb::Coord(20,19,20)));
-    ASSERT_DOUBLES_EXACTLY_EQUAL(-2, grid.tree().getValue(openvdb::Coord(20,20,20)));
-    ASSERT_DOUBLES_EXACTLY_EQUAL(-1, grid.tree().getValue(openvdb::Coord(20,21,20)));
-    ASSERT_DOUBLES_EXACTLY_EQUAL( 0, grid.tree().getValue(openvdb::Coord(20,22,20)));
-    ASSERT_DOUBLES_EXACTLY_EQUAL( 1, grid.tree().getValue(openvdb::Coord(20,23,20)));
-    ASSERT_DOUBLES_EXACTLY_EQUAL( 2, grid.tree().getValue(openvdb::Coord(20,24,20)));
+    ASSERT_DOUBLES_EXACTLY_EQUAL( 2, ls->tree().getValue(openvdb::Coord(20,16,20)));
+    ASSERT_DOUBLES_EXACTLY_EQUAL( 1, ls->tree().getValue(openvdb::Coord(20,17,20)));
+    ASSERT_DOUBLES_EXACTLY_EQUAL( 0, ls->tree().getValue(openvdb::Coord(20,18,20)));
+    ASSERT_DOUBLES_EXACTLY_EQUAL(-1, ls->tree().getValue(openvdb::Coord(20,19,20)));
+    ASSERT_DOUBLES_EXACTLY_EQUAL(-2, ls->tree().getValue(openvdb::Coord(20,20,20)));
+    ASSERT_DOUBLES_EXACTLY_EQUAL(-1, ls->tree().getValue(openvdb::Coord(20,21,20)));
+    ASSERT_DOUBLES_EXACTLY_EQUAL( 0, ls->tree().getValue(openvdb::Coord(20,22,20)));
+    ASSERT_DOUBLES_EXACTLY_EQUAL( 1, ls->tree().getValue(openvdb::Coord(20,23,20)));
+    ASSERT_DOUBLES_EXACTLY_EQUAL( 2, ls->tree().getValue(openvdb::Coord(20,24,20)));
 
     {// full but slow test of all voxels
-        openvdb::CoordBBox bbox = pa.getBBox(grid);
-        bbox.expand(halfWidth+1);
-        size_t count=0;
+        openvdb::CoordBBox bbox = pa.getBBox(*ls);
+        bbox.expand(static_cast<int>(halfWidth)+1);
+        openvdb::Index64 count = 0;
+        const float outside = ls->background(), inside = -outside;
         const openvdb::Coord &min=bbox.min(), &max=bbox.max();
         for (openvdb::Coord ijk=min; ijk[0]<max[0]; ++ijk[0]) {
             for (ijk[1]=min[1]; ijk[1]<max[1]; ++ijk[1]) {
                 for (ijk[2]=min[2]; ijk[2]<max[2]; ++ijk[2]) {
-                    const openvdb::Vec3d xyz = grid.indexToWorld(ijk.asVec3d());
+                    const openvdb::Vec3d xyz = ls->indexToWorld(ijk.asVec3d());
                     double dist = (xyz-pa.pos(0)).length()-pa.radius(0);
                     openvdb::Index32 k =0;
                     for (int i=1, s=pa.size(); i<s; ++i) {
@@ -340,21 +329,21 @@ TestParticlesToLevelSet::testRasterizeSpheresAndId()
                             dist = d;
                         }
                     }
-                    const float val = grid.tree().getValue(ijk);
+                    const float val = ls->tree().getValue(ijk);
                     openvdb::Index32 m = id->tree().getValue(ijk);
-                    if (dist >= gamma) {
-                        CPPUNIT_ASSERT_DOUBLES_EQUAL( gamma, val, 0.0001);
-                        CPPUNIT_ASSERT(grid.tree().isValueOff(ijk));
+                    if (dist >= outside) {
+                        CPPUNIT_ASSERT_DOUBLES_EQUAL(outside, val, 0.0001);
+                        CPPUNIT_ASSERT(ls->tree().isValueOff(ijk));
                         CPPUNIT_ASSERT_EQUAL(openvdb::util::INVALID_IDX, m);
                         CPPUNIT_ASSERT(id->tree().isValueOff(ijk));
-                    } else if (dist <= -gamma) {
-                        CPPUNIT_ASSERT_DOUBLES_EQUAL(-gamma, val, 0.0001);
-                        CPPUNIT_ASSERT(grid.tree().isValueOff(ijk));
+                    } else if( dist <= inside ) {
+                        CPPUNIT_ASSERT_DOUBLES_EQUAL(inside, val, 0.0001);
+                        CPPUNIT_ASSERT(ls->tree().isValueOff(ijk));
                         CPPUNIT_ASSERT_EQUAL(openvdb::util::INVALID_IDX, m);
                         CPPUNIT_ASSERT(id->tree().isValueOff(ijk));
                     } else {
                         CPPUNIT_ASSERT_DOUBLES_EQUAL(  dist, val, 0.0001);
-                        CPPUNIT_ASSERT(grid.tree().isValueOn(ijk));
+                        CPPUNIT_ASSERT(ls->tree().isValueOn(ijk));
                         CPPUNIT_ASSERT_EQUAL(k, m);
                         CPPUNIT_ASSERT(id->tree().isValueOn(ijk));
                         ++count;
@@ -364,8 +353,8 @@ TestParticlesToLevelSet::testRasterizeSpheresAndId()
         }
         //std::cerr << "\nExpected active voxel count = " << count
         //    << ", actual active voxle count = "
-        //    << grid.activeVoxelCount() << std::endl;
-        CPPUNIT_ASSERT_EQUAL(count, grid.activeVoxelCount());
+        //    << ls->activeVoxelCount() << std::endl;
+        CPPUNIT_ASSERT_EQUAL(count, ls->activeVoxelCount());
     }
 }
 
@@ -375,11 +364,8 @@ TestParticlesToLevelSet::testRasterizeSpheresAndId()
 void
 TestParticlesToLevelSet::testRasterizeTrails()
 {
-    const float voxelSize = 1.0f;
-    const int halfWidth = 2;
-    openvdb::FloatGrid grid(halfWidth*voxelSize);
-    grid.setTransform(openvdb::math::Transform::createLinearTransform(voxelSize));
-    grid.setGridClass(openvdb::GRID_LEVEL_SET);
+    const float voxelSize = 1.0f, halfWidth = 2.0f;
+    openvdb::FloatGrid::Ptr ls = openvdb::createLevelSet<openvdb::FloatGrid>(voxelSize, halfWidth);
 
     MyParticleList pa(1,5);
 
@@ -388,25 +374,20 @@ TestParticlesToLevelSet::testRasterizeTrails()
     pa.add(openvdb::Vec3R(-10,-10,-10), 2, openvdb::Vec3R( 2, 0, 0));
     pa.add(openvdb::Vec3R( 10, 10, 10), 3, openvdb::Vec3R( 0, 1, 0));
     pa.add(openvdb::Vec3R(  0,  0,  0), 6, openvdb::Vec3R( 0, 0,-5));
-
-    openvdb::util::NullInterrupter boss;
-    openvdb::tools::ParticlesToLevelSet<openvdb::FloatGrid, MyParticleList> raster(grid, boss);
+    
+    openvdb::tools::ParticlesToLevelSet<openvdb::FloatGrid, MyParticleList> raster(*ls);
     raster.rasterizeTrails(pa, 0.75);//scale offset between two instances
 
-    //grid.tree().print(std::cout, 4);
-    //this->writeGrid(grid, "testRasterizeTrails");
+    //ls->tree().print(std::cout, 4);
+    //this->writeGrid(*ls, "testRasterizeTrails");
 }
 
 
 void
 TestParticlesToLevelSet::testRasterizeTrailsAndId()
 {
-    const float voxelSize = 1.0f;
-    const int halfWidth = 2;
-    typedef openvdb::FloatGrid GridT;
-    GridT grid(halfWidth*voxelSize);
-    grid.setTransform(openvdb::math::Transform::createLinearTransform(voxelSize));
-    grid.setGridClass(openvdb::GRID_LEVEL_SET);
+    const float voxelSize = 1.0f, halfWidth = 2.0f;
+    openvdb::FloatGrid::Ptr ls = openvdb::createLevelSet<openvdb::FloatGrid>(voxelSize, halfWidth);
 
     MyParticleList pa(1,5);
 
@@ -415,10 +396,9 @@ TestParticlesToLevelSet::testRasterizeTrailsAndId()
     pa.add(openvdb::Vec3R(-10,-10,-10), 2, openvdb::Vec3R( 2, 0, 0));
     pa.add(openvdb::Vec3R( 10, 10, 10), 3, openvdb::Vec3R( 0, 1, 0));
     pa.add(openvdb::Vec3R(  0,  0,  0), 6, openvdb::Vec3R( 0, 0,-5));
-
-    openvdb::util::NullInterrupter boss;
-    typedef openvdb::tools::ParticlesToLevelSetAndId<GridT, MyParticleList> RasterT;
-    RasterT raster(grid, boss);
+    
+    typedef openvdb::tools::ParticlesToLevelSetAndId<openvdb::FloatGrid, MyParticleList> RasterT;
+    RasterT raster(*ls);
 
     const RasterT::IndxGridT::Ptr id =
         raster.rasterizeTrails(pa, 0.75);//scale offset between two instances
@@ -431,8 +411,8 @@ TestParticlesToLevelSet::testRasterizeTrailsAndId()
     CPPUNIT_ASSERT_EQUAL(1, min);//first particle is ignored because of its small rdadius!
     CPPUNIT_ASSERT_EQUAL(3, max);
 
-    //grid.tree().print(std::cout, 4);
-    //this->writeGrid(grid, "testRasterizeTrails");
+    //ls->tree().print(std::cout, 4);
+    //this->writeGrid(*ls, "testRasterizeTrails");
 }
 
 // Copyright (c) 2012 DreamWorks Animation LLC
