@@ -34,18 +34,16 @@
 #include <openvdb/Types.h>
 #include <openvdb/openvdb.h>
 #include <openvdb/tools/Cpt.h>
-#include <openvdb/tools/Gradient.h>
+#include <openvdb/tools/GridOperators.h>
 #include <openvdb/tools/Filter.h>
+#include <openvdb/tools/LevelSetUtil.h>
 #include <openvdb/tools/LevelSetSphere.h>
-#include <openvdb/tools/Magnitude.h>
+#include <openvdb/tools/LevelSetAdvect.h>
 #include <openvdb/tools/Morphology.h>
-#include <openvdb/tools/Normalize.h>
 #include <openvdb/tools/PointAdvect.h>
 #include <openvdb/tools/PointScatter.h>
 #include <openvdb/tools/ValueTransformer.h>
 #include "util.h" // for unittest_util::makeSphere()
-#include <boost/date_time/posix_time/posix_time.hpp>
-#include <boost/thread/thread.hpp>// for boost::this_thread::sleep
 
 #define ASSERT_DOUBLES_EXACTLY_EQUAL(expected, actual) \
     CPPUNIT_ASSERT_DOUBLES_EQUAL((expected), (actual), /*tolerance=*/0.0);
@@ -62,6 +60,7 @@ public:
     CPPUNIT_TEST(testFilter);
     CPPUNIT_TEST(testFloatApply);
     CPPUNIT_TEST(testLevelSetSphere);
+    CPPUNIT_TEST(testLevelSetAdvect);
     CPPUNIT_TEST(testMagnitude);
     CPPUNIT_TEST(testNormalize);
     CPPUNIT_TEST(testPointAdvect);
@@ -75,6 +74,7 @@ public:
     void testFilter();
     void testFloatApply();
     void testLevelSetSphere();
+    void testLevelSetAdvect();
     void testMagnitude();
     void testNormalize();
     void testPointAdvect();
@@ -102,7 +102,7 @@ TestTools::testDilateVoxels()
 
     const openvdb::Index leafDim = Tree543f::LeafNodeType::DIM;
     CPPUNIT_ASSERT_EQUAL(1 << 3, int(leafDim));
-
+    
     {
         // Set and dilate a single voxel at the center of a leaf node.
         tree->setValue(Coord(leafDim >> 1), 1.0);
@@ -174,17 +174,48 @@ TestTools::testDilateVoxels()
             openvdb::tools::dilateVoxels(*tree);
         }
     }
-    {
+    
+    {// dialte a narrow band of a sphere
         typedef openvdb::Grid<Tree543f> GridType;
         GridType grid(tree->getBackground());
         unittest_util::makeSphere<GridType>(/*dim=*/openvdb::Coord(64, 64, 64),
-            /*center=*/openvdb::Vec3f(0, 0, 0), /*radius=*/20, grid, /*dx=*/1.0f,
-            unittest_util::SPHERE_DENSE_NARROW_BAND);
-        tree = grid.treePtr();
-        const openvdb::Index64 count = tree->activeVoxelCount();
-        openvdb::tools::dilateVoxels(*tree);
-        CPPUNIT_ASSERT(tree->activeVoxelCount() > count);
+                                            /*center=*/openvdb::Vec3f(0, 0, 0),
+                                            /*radius=*/20, grid, /*dx=*/1.0f,
+                                            unittest_util::SPHERE_DENSE_NARROW_BAND);
+        const openvdb::Index64 count = grid.tree().activeVoxelCount();
+        openvdb::tools::dilateVoxels(grid.tree());
+        CPPUNIT_ASSERT(grid.tree().activeVoxelCount() > count);
     }
+    
+    {// dilate a fog volume of a sphere
+        typedef openvdb::Grid<Tree543f> GridType;
+        GridType grid(tree->getBackground());
+        unittest_util::makeSphere<GridType>(/*dim=*/openvdb::Coord(64, 64, 64),
+                                            /*center=*/openvdb::Vec3f(0, 0, 0),
+                                            /*radius=*/20, grid, /*dx=*/1.0f,
+                                            unittest_util::SPHERE_DENSE_NARROW_BAND);
+        openvdb::tools::sdfToFogVolume(grid);
+        const openvdb::Index64 count = grid.tree().activeVoxelCount();
+        //std::cerr << "\nBefore: active voxel count = " << count << std::endl;
+        //grid.print(std::cerr,5);
+        openvdb::tools::dilateVoxels(grid.tree());
+        CPPUNIT_ASSERT(grid.tree().activeVoxelCount() > count);
+        //std::cerr << "\nAfter: active voxel count = " << grid.tree().activeVoxelCount() << std::endl;
+    }
+//     {// Test a grid from a file that has proven to be challenging
+//         openvdb::initialize();
+//         openvdb::io::File file("/usr/home/kmuseth/Data/vdb/dilation.vdb");
+//         file.open();
+//         openvdb::GridBase::Ptr baseGrid = file.readGrid(file.beginName().gridName());
+//         file.close();
+//         openvdb::FloatGrid::Ptr grid = openvdb::gridPtrCast<openvdb::FloatGrid>(baseGrid);
+//         const openvdb::Index64 count = grid->tree().activeVoxelCount();
+//         //std::cerr << "\nBefore: active voxel count = " << count << std::endl;
+//         //grid->print(std::cerr,5);
+//         openvdb::tools::dilateVoxels(grid->tree());
+//         CPPUNIT_ASSERT(grid->tree().activeVoxelCount() > count);
+//         //std::cerr << "\nAfter: active voxel count = " << grid->tree().activeVoxelCount() << std::endl;
+//     }
 }
 
 void
@@ -306,91 +337,38 @@ TestTools::testFilter()
         }
         //std::cerr << "Successfully completed TestTools::testFilter mean test" << std::endl;
     }
-    {// test Filter::meanCurvatureFilter
-        openvdb::FloatGrid::Ptr filteredGrid = referenceGrid->deepCopy();
-        //openvdb::FloatTree& filteredTree = filteredGrid->tree();
-        openvdb::tools::Filter<openvdb::FloatGrid> filter(*filteredGrid);
-        filter.meanCurvature();///< @todo test!
-        /*
-        openvdb::math::CurvatureStencil<openvdb::FloatGrid> stencil(*referenceGrid, width);
-        //openvdb::tools::Filter<openvdb::FloatGrid> filter(filteredGrid);
-        //filter.mean(width, false);
-        for (int x=0; x<dim[0]; ++x) {
-            xyz[0]=x;
-            for (int y=0; y<dim[1]; ++y) {
-                xyz[1]=y;
-                for (int z=0; z<dim[2]; ++z) {
-                    xyz[2]=z;
-                    double sum =0.0, count=0.0;
-                    for (int i = xyz[0] - width, ie= xyz[0] + width; i <= ie; ++i) {
-                        openvdb::Coord ijk(i,0,0);
-                        for (int j = xyz[1] - width, je = xyz[1] + width; j <= je; ++j) {
-                            ijk.setY(j);
-                            for (int k = xyz[2] - width, ke = xyz[2] + width; k <= ke; ++k) {
-                                ijk.setZ(k);
-                                sum += sphere.getValue(ijk);
-                                count += 1.0;
-                            }
-                        }
-                    }
-                    stencil.moveTo(xyz);
-                    CPPUNIT_ASSERT_DOUBLES_EQUAL(
-                        sum/count, stencil.mean(), 0.0001);
-                    CPPUNIT_ASSERT_DOUBLES_EQUAL(
-                        stencil.mean(), filteredTree.getValue(xyz), 0.0001);
-                }
-            }
-        }
-        */
-        //std::cerr << "Successfully completed TestTools::testFilter mean curvature test\n";
-    }
-    {// test Filter::laplacian
-    }
-    {// test Filter::renormalize
-        openvdb::FloatGrid::Ptr filteredGrid = referenceGrid->deepCopy();
-        openvdb::tools::Filter<openvdb::FloatGrid> filter(*filteredGrid);
-        filter.renormalize<true/*=UseWeno*/>(2,1e-3);///< @todo test!
-        filter.renormalize<false/*=UseWeno*/>(2,1e-3);///< @todo test!
-    }
 }
 
 void
 TestTools::testLevelSetSphere()
 {
-    using namespace boost::posix_time;
     const float radius = 4.3f;
     const openvdb::Vec3f center(15.8f, 13.2f, 16.7f);
-    const float voxel_size = 1.5f, width = 3.25f;
-    const float gamma = voxel_size * width;
+    const float voxelSize = 1.5f, width = 3.25f;
     const int dim = 32;
 
-    //ptime t1 = microsec_clock::local_time();
-    //boost::this_thread::sleep( seconds(1) );
-    openvdb::FloatGrid::Ptr grid1 = openvdb::tools::makeSphere<openvdb::util::NullInterrupter>(
-        radius, center, voxel_size, width);
-    //size_t runTime = (microsec_clock::local_time() - t1).total_milliseconds();
-    //std::cerr << "\nComputed " << grid1->activeVoxelCount() << " voxels in "
-    //    << runTime << " milliseconds" << std::endl;
+    openvdb::FloatGrid::Ptr grid1 =
+        openvdb::tools::createLevelSetSphere<openvdb::FloatGrid>(radius, center, voxelSize, width);
 
     /// Also test ultra slow makeSphere in unittest/util.h
-    openvdb::FloatGrid::Ptr grid2 = openvdb::FloatGrid::create(gamma);
-    grid2->setTransform(openvdb::math::Transform::createLinearTransform(voxel_size));
+    openvdb::FloatGrid::Ptr grid2 = openvdb::createLevelSet<openvdb::FloatGrid>(voxelSize, width);
     unittest_util::makeSphere<openvdb::FloatGrid>(
         openvdb::Coord(dim), center, radius, *grid2, unittest_util::SPHERE_SPARSE_NARROW_BAND);
 
+    const float outside = grid1->background(), inside = -outside;
     for (int i=0; i<dim; ++i) {
         for (int j=0; j<dim; ++j) {
             for (int k=0; k<dim; ++k) {
-                const openvdb::Vec3f p(voxel_size*i,voxel_size*j,voxel_size*k);
+                const openvdb::Vec3f p(voxelSize*i,voxelSize*j,voxelSize*k);
                 const float dist = (p-center).length() - radius;
                 const float val1 = grid1->tree().getValue(openvdb::Coord(i,j,k));
                 const float val2 = grid2->tree().getValue(openvdb::Coord(i,j,k));
-                if (dist > gamma) {
-                    CPPUNIT_ASSERT_DOUBLES_EQUAL( gamma, val1, 0.0001);
-                    CPPUNIT_ASSERT_DOUBLES_EQUAL( gamma, val2, 0.0001);
-                } else if (dist < -gamma) {
-                    CPPUNIT_ASSERT_DOUBLES_EQUAL(-gamma, val1, 0.0001);
-                    CPPUNIT_ASSERT_DOUBLES_EQUAL(-gamma, val2, 0.0001);
+                if (dist > outside) {
+                    CPPUNIT_ASSERT_DOUBLES_EQUAL( outside, val1, 0.0001);
+                    CPPUNIT_ASSERT_DOUBLES_EQUAL( outside, val2, 0.0001);
+                } else if (dist < inside) {
+                    CPPUNIT_ASSERT_DOUBLES_EQUAL( inside, val1, 0.0001);
+                    CPPUNIT_ASSERT_DOUBLES_EQUAL( inside, val2, 0.0001);
                 } else {
                     CPPUNIT_ASSERT_DOUBLES_EQUAL(  dist, val1, 0.0001);
                     CPPUNIT_ASSERT_DOUBLES_EQUAL(  dist, val2, 0.0001);
@@ -402,6 +380,125 @@ TestTools::testLevelSetSphere()
     CPPUNIT_ASSERT_EQUAL(grid1->activeVoxelCount(), grid2->activeVoxelCount());
 }
 
+void
+TestTools::testLevelSetAdvect()
+{
+    // Uncomment sections below to run this (time-consuming) test
+    /*
+    const int dim = 64;//256
+    const openvdb::Vec3f center(0.35f, 0.35f, 0.35f);
+    const float radius = 0.15f, voxelSize = 1.0f/(dim-1);
+    
+    typedef openvdb::FloatGrid GridT;
+    typedef openvdb::Vec3fGrid VectT;
+
+    class FrameWriter {
+    public:
+        FrameWriter(int dim, GridT::Ptr grid) : mFrame(0), mDim(dim), mGrid(grid) {}
+        void operator()(const char* name, float time, size_t n) {
+            std::ostringstream ostr;
+            ostr << "/usr/pic1/tmp/" << name << "_" << mDim << "_" << mFrame << ".vdb";
+            openvdb::io::File file(ostr.str());
+            openvdb::GridPtrVec grids;
+            grids.push_back(mGrid);
+            file.write(grids);
+            std::cerr << "\nWrote \"" << ostr.str() << "\" with time = "
+                      << time << " after CFL-iterations = " << n << std::endl;
+            ++mFrame;
+        }
+    private:
+        int mFrame, mDim;
+        GridT::Ptr mGrid;
+        };
+    */
+    /*
+    {//test tracker
+        GridT::Ptr grid = openvdb::tools::createLevelSetSphere<GridT>(radius, center, voxelSize);
+        typedef openvdb::tools::LevelSetTracker<GridT>  TrackerT;
+        TrackerT tracker(*grid);
+        tracker.setSpatialScheme(openvdb::math::HJWENO5_BIAS);
+        tracker.setTemporalScheme(openvdb::math::TVD_RK1);
+
+        FrameWriter fw(dim, grid); fw("Tracker",0, 0);
+        //for (float t = 0, dt = 0.005f; !grid->empty() && t < 3.0f; t += dt) {
+        //    fw("Enright", t + dt, advect.advect(t, t + dt));
+        //}
+        for (float t = 0, dt = 0.5f; !grid->empty() && t < 1.0f; t += dt) {
+            tracker.track();
+            fw("Tracker", 0, 0);
+        }
+        }
+    */
+    /*
+    {//test EnrightField
+        GridT::Ptr grid = openvdb::tools::createLevelSetSphere<GridT>(radius, center, voxelSize);
+        typedef openvdb::tools::EnrightField<float> FieldT;
+        FieldT field;
+        
+        typedef openvdb::tools::LevelSetAdvection<GridT, FieldT>  AdvectT;
+        AdvectT advect(*grid, field);
+        advect.setSpatialScheme(openvdb::math::HJWENO5_BIAS);
+        advect.setTemporalScheme(openvdb::math::TVD_RK2);
+        advect.setTrackerSpatialScheme(openvdb::math::HJWENO5_BIAS);
+        advect.setTrackerTemporalScheme(openvdb::math::TVD_RK1);
+        
+        FrameWriter fw(dim, grid); fw("Enright",0, 0);
+        //for (float t = 0, dt = 0.005f; !grid->empty() && t < 3.0f; t += dt) {
+        //    fw("Enright", t + dt, advect.advect(t, t + dt));
+        //}
+        for (float t = 0, dt = 0.5f; !grid->empty() && t < 1.0f; t += dt) {
+            fw("Enright", t + dt, advect.advect(t, t + dt));
+        }
+        }
+    */
+    /*
+    {// test DiscreteGrid - Aligned
+        GridT::Ptr grid = openvdb::tools::createLevelSetSphere<GridT>(radius, center, voxelSize);
+        VectT vect(openvdb::Vec3f(1,0,0));
+        typedef openvdb::tools::DiscreteField<VectT> FieldT;
+        FieldT field(vect);
+        typedef openvdb::tools::LevelSetAdvection<GridT, FieldT>  AdvectT;
+        AdvectT advect(*grid, field);
+        advect.setSpatialScheme(openvdb::math::HJWENO5_BIAS);
+        advect.setTemporalScheme(openvdb::math::TVD_RK2);
+        
+        FrameWriter fw(dim, grid); fw("Aligned",0, 0);
+        //for (float t = 0, dt = 0.005f; !grid->empty() && t < 3.0f; t += dt) {
+        //    fw("Aligned", t + dt, advect.advect(t, t + dt));
+        //}
+        for (float t = 0, dt = 0.5f; !grid->empty() && t < 1.0f; t += dt) {
+            fw("Aligned", t + dt, advect.advect(t, t + dt));
+        }
+        }
+    */
+    /*
+    {// test DiscreteGrid - Transformed
+        GridT::Ptr grid = openvdb::tools::createLevelSetSphere<GridT>(radius, center, voxelSize);
+        VectT vect(openvdb::Vec3f(0,0,0));
+        VectT::Accessor acc = vect.getAccessor();
+        for (openvdb::Coord ijk(0); ijk[0]<dim; ++ijk[0])
+            for (ijk[1]=0; ijk[1]<dim; ++ijk[1])
+                for (ijk[2]=0; ijk[2]<dim; ++ijk[2])
+                    acc.setValue(ijk, openvdb::Vec3f(1,0,0));
+        vect.transform().scale(2.0f);
+        typedef openvdb::tools::DiscreteField<VectT> FieldT;
+        FieldT field(vect);
+        typedef openvdb::tools::LevelSetAdvection<GridT, FieldT>  AdvectT;
+        AdvectT advect(*grid, field);
+        advect.setSpatialScheme(openvdb::math::HJWENO5_BIAS);
+        advect.setTemporalScheme(openvdb::math::TVD_RK2);
+
+        FrameWriter fw(dim, grid); fw("Xformed",0, 0);
+        //for (float t = 0, dt = 0.005f; !grid->empty() && t < 3.0f; t += dt) {
+        //    fw("Xformed", t + dt, advect.advect(t, t + dt));
+        //}
+        for (float t = 0, dt = 0.5f; !grid->empty() && t < 1.0f; t += dt) {
+            fw("Xformed", t + dt, advect.advect(t, t + dt));
+        }
+        }
+    */
+}
+
 void
 TestTools::testFloatApply()
 {
@@ -482,7 +579,7 @@ TestTools::testMagnitude()
 void
 TestTools::testNormalize()
 {
-    openvdb::FloatGrid::Ptr grid = openvdb::FloatGrid::create(/*background=*/5.0);
+    openvdb::FloatGrid::Ptr grid = openvdb::FloatGrid::create(5.0);
     openvdb::FloatTree& tree = grid->tree();
 
     const openvdb::Coord dim(64,64,64);
@@ -506,7 +603,7 @@ TestTools::testNormalize()
         static inline void visit(const ValueIter& it) { it.setValue(op(*it)); }
     };
 
-    openvdb::tools::foreach(grad->beginValueOn(), Local::visit, /*threaded=*/true);
+    openvdb::tools::foreach(grad->beginValueOn(), Local::visit, true);
 
     openvdb::VectorGrid::ConstAccessor accessor = grad->getConstAccessor();
 
@@ -523,7 +620,6 @@ TestTools::testNormalize()
     CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, v.length(), 0.0001);
 }
 
-
 void
 TestTools::testPointAdvect()
 {
