@@ -80,7 +80,6 @@ public:
     CPPUNIT_TEST(testPruneLevelSet);
     CPPUNIT_TEST(testTouchLeaf);
     CPPUNIT_TEST(testProbeLeaf);
-    CPPUNIT_TEST(testLeafArray);
     CPPUNIT_TEST(testLeafManager);
     CPPUNIT_TEST(testProcessBBox);
     CPPUNIT_TEST_SUITE_END();
@@ -106,7 +105,6 @@ public:
     void testPruneInactive();
     void testTouchLeaf();
     void testProbeLeaf();
-    void testLeafArray();
     void testLeafManager();
     void testProcessBBox();
 
@@ -334,7 +332,7 @@ TestTree::testSetValue()
     ASSERT_DOUBLES_EXACTLY_EQUAL(2*background, tree.getValue(c0));
     ASSERT_DOUBLES_EXACTLY_EQUAL(15.0+background, tree.getValue(c1));
     CPPUNIT_ASSERT_EQUAL(2, int(tree.activeVoxelCount()));
-    
+
     // Test the extremes of the coordinate range
     ASSERT_DOUBLES_EXACTLY_EQUAL(background, tree.getValue(openvdb::Coord::min()));
     ASSERT_DOUBLES_EXACTLY_EQUAL(background, tree.getValue(openvdb::Coord::max()));
@@ -1055,8 +1053,8 @@ TestTree::testVoxelizeActiveTiles()
             tree.setValue(xyz[1], 1.0f);
         } else {
             const int n = tile_size[level];
-            tree.fill(CoordBBox(Coord(-n,-n,-n), n), 1.0f, true);
-            tree.fill(CoordBBox(Coord( 0, 0, 0), n), 1.0f, true);
+            tree.fill(CoordBBox::createCube(Coord(-n,-n,-n), n), 1.0f, true);
+            tree.fill(CoordBBox::createCube(Coord( 0, 0, 0), n), 1.0f, true);
         }
 
         CPPUNIT_ASSERT_EQUAL(3-level,tree.getValueDepth(xyz[0]));
@@ -1526,7 +1524,7 @@ TestTree::testProbeLeaf()
 {
     const float background=10.0f, value = 2.0f;
     const openvdb::Coord xyz(-20,30,10);
-    {// test tree
+    {// test Tree::probeLeaf
         openvdb::FloatTree::Ptr tree(new openvdb::FloatTree(background));
         CPPUNIT_ASSERT_EQUAL(-1, tree->getValueDepth(xyz));
         CPPUNIT_ASSERT_EQUAL( 0, int(tree->leafCount()));
@@ -1542,13 +1540,31 @@ TestTree::testProbeLeaf()
         CPPUNIT_ASSERT(tree->isValueOn(xyz));
         ASSERT_DOUBLES_EXACTLY_EQUAL(value, tree->getValue(xyz));
     }
-    {// test accessor
+    {// test Tree::probeConstLeaf
+        const openvdb::FloatTree tree1(background);
+        CPPUNIT_ASSERT_EQUAL(-1, tree1.getValueDepth(xyz));
+        CPPUNIT_ASSERT_EQUAL( 0, int(tree1.leafCount()));
+        CPPUNIT_ASSERT(tree1.probeConstLeaf(xyz)==NULL);
+        CPPUNIT_ASSERT_EQUAL(-1, tree1.getValueDepth(xyz));
+        CPPUNIT_ASSERT_EQUAL( 0, int(tree1.leafCount()));
+        openvdb::FloatTree tmp(tree1);
+        tmp.setValue(xyz, value);
+        const openvdb::FloatTree tree2(tmp);
+        CPPUNIT_ASSERT_EQUAL( 3, tree2.getValueDepth(xyz));
+        CPPUNIT_ASSERT_EQUAL( 1, int(tree2.leafCount()));
+        CPPUNIT_ASSERT(tree2.probeConstLeaf(xyz)!=NULL);
+        CPPUNIT_ASSERT_EQUAL( 3, tree2.getValueDepth(xyz));
+        CPPUNIT_ASSERT_EQUAL( 1, int(tree2.leafCount()));
+        CPPUNIT_ASSERT(tree2.isValueOn(xyz));
+        ASSERT_DOUBLES_EXACTLY_EQUAL(value, tree2.getValue(xyz));
+    }
+    {// test ValueAccessor::probeLeaf
         openvdb::FloatTree::Ptr tree(new openvdb::FloatTree(background));
         openvdb::tree::ValueAccessor<openvdb::FloatTree> acc(*tree);
         CPPUNIT_ASSERT_EQUAL(-1, acc.getValueDepth(xyz));
         CPPUNIT_ASSERT_EQUAL( 0, int(tree->leafCount()));
         CPPUNIT_ASSERT(acc.probeLeaf(xyz)==NULL);
-        CPPUNIT_ASSERT_EQUAL(-1, tree->getValueDepth(xyz));
+        CPPUNIT_ASSERT_EQUAL(-1, acc.getValueDepth(xyz));
         CPPUNIT_ASSERT_EQUAL( 0, int(tree->leafCount()));
         acc.setValue(xyz, value);
         CPPUNIT_ASSERT_EQUAL( 3, acc.getValueDepth(xyz));
@@ -1559,48 +1575,26 @@ TestTree::testProbeLeaf()
         CPPUNIT_ASSERT(acc.isValueOn(xyz));
         ASSERT_DOUBLES_EXACTLY_EQUAL(value, acc.getValue(xyz));
     }
-}
-
-void
-TestTree::testLeafArray()
-{
-    using openvdb::Coord;
-    using openvdb::Index32;
-    using openvdb::Index64;
-
-    const float background=10.0f, R=5.6f;
-    const openvdb::Vec3f C(12.3f, 15.5f, 10.0f);
-    const openvdb::Coord dim(32, 32, 32);
-    openvdb::FloatGrid grid(background);
-    unittest_util::makeSphere<openvdb::FloatGrid>(dim, C, R, grid,
-                                                  1.0f, unittest_util::SPHERE_SPARSE_NARROW_BAND);
-    openvdb::FloatTree& tree = grid.tree();
-
-    typedef openvdb::tree::LeafArray<openvdb::FloatTree,2> ArrayType;
-    ArrayType leafArray(tree, /*sync*/false);
-
-    CPPUNIT_ASSERT_EQUAL(tree.leafCount(), leafArray.size());
-
-    ArrayType::IterRangeType range = leafArray.getRange();
-    ArrayType::IterType i = range.begin();
-
-    const float first_val = i->leaf->getValue(0);
-
-    i->buffer[1].fill(first_val + 1.0);
-    CPPUNIT_ASSERT(!openvdb::math::isExactlyEqual(first_val, i->buffer[1].getValue(0)));
-
-    leafArray.syncBuffers(/*serial*/false);
-
-    ASSERT_DOUBLES_EXACTLY_EQUAL(first_val , i->buffer[1].getValue(0));
-
-    i->buffer[1].setValue(0, 1.45f);
-
-    CPPUNIT_ASSERT(!openvdb::math::isExactlyEqual(first_val, i->buffer[1].getValue(0)));
-
-    leafArray.swapBuffers(1, /*serial*/false);
-
-    ASSERT_DOUBLES_EXACTLY_EQUAL(first_val , i->buffer[1].getValue(0));
-    ASSERT_DOUBLES_EXACTLY_EQUAL(1.45f , i->leaf->getValue(0));
+    {// test ValueAccessor::probeConstLeaf
+        const openvdb::FloatTree tree1(background);
+        openvdb::tree::ValueAccessor<const openvdb::FloatTree> acc1(tree1);
+        CPPUNIT_ASSERT_EQUAL(-1, acc1.getValueDepth(xyz));
+        CPPUNIT_ASSERT_EQUAL( 0, int(tree1.leafCount()));
+        CPPUNIT_ASSERT(acc1.probeConstLeaf(xyz)==NULL);
+        CPPUNIT_ASSERT_EQUAL(-1, acc1.getValueDepth(xyz));
+        CPPUNIT_ASSERT_EQUAL( 0, int(tree1.leafCount()));
+        openvdb::FloatTree tmp(tree1);
+        tmp.setValue(xyz, value);
+        const openvdb::FloatTree tree2(tmp);
+        openvdb::tree::ValueAccessor<const openvdb::FloatTree> acc2(tree2);
+        CPPUNIT_ASSERT_EQUAL( 3, acc2.getValueDepth(xyz));
+        CPPUNIT_ASSERT_EQUAL( 1, int(tree2.leafCount()));
+        CPPUNIT_ASSERT(acc2.probeConstLeaf(xyz)!=NULL);
+        CPPUNIT_ASSERT_EQUAL( 3, acc2.getValueDepth(xyz));
+        CPPUNIT_ASSERT_EQUAL( 1, int(tree2.leafCount()));
+        CPPUNIT_ASSERT(acc2.isValueOn(xyz));
+        ASSERT_DOUBLES_EXACTLY_EQUAL(value, acc2.getValue(xyz));
+    }
 }
 
 struct BBoxOp
@@ -1632,8 +1626,8 @@ TestTree::testProcessBBox()
     for (int level=0; level<=3; ++level) {
         openvdb::FloatTree tree;
         const int n = size[level];
-        const CoordBBox bbox[]={CoordBBox(Coord(-n,-n,-n), n),
-                                CoordBBox(Coord( 0, 0, 0), n)};
+        const CoordBBox bbox[]={CoordBBox::createCube(Coord(-n,-n,-n), n),
+                                CoordBBox::createCube(Coord( 0, 0, 0), n)};
         if (level==0) {
             tree.setValue(Coord(-1,-2,-3), 1.0f);
             tree.setValue(Coord( 1, 2, 3), 1.0f);
@@ -1754,7 +1748,6 @@ TestTree::testLeafManager()
     }
 }
 
-
 // Copyright (c) 2012 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
