@@ -152,8 +152,14 @@ float sumf(float a, float b) { return a + b; }
 float mulf(float a, float b) { return a * b; }
 
 openvdb::Vec3f orderv(const openvdb::Vec3f& a, const openvdb::Vec3f& b) { return a + 100 * b; }
-openvdb::Vec3f maxv(const openvdb::Vec3f& a, const openvdb::Vec3f& b) { return std::max(a, b); }
-openvdb::Vec3f minv(const openvdb::Vec3f& a, const openvdb::Vec3f& b) { return std::min(a, b); }
+openvdb::Vec3f maxv(const openvdb::Vec3f& a, const openvdb::Vec3f& b) {
+    const float aMag = a.lengthSqr(), bMag = b.lengthSqr();
+    return (aMag > bMag ? a : (bMag > aMag ? b : std::max(a, b)));
+}
+openvdb::Vec3f minv(const openvdb::Vec3f& a, const openvdb::Vec3f& b) {
+    const float aMag = a.lengthSqr(), bMag = b.lengthSqr();
+    return (aMag < bMag ? a : (bMag < aMag ? b : std::min(a, b)));
+}
 openvdb::Vec3f sumv(const openvdb::Vec3f& a, const openvdb::Vec3f& b) { return a + b; }
 openvdb::Vec3f mulv(const openvdb::Vec3f& a, const openvdb::Vec3f& b) { return a * b; }
 
@@ -221,6 +227,7 @@ TestTreeCombine::testComp(const TreeComp& comp, const ValueComp& op)
     const ValueT
         zero = openvdb::zeroVal<ValueT>(),
         minusOne = zero + (-1),
+        minusTwo = zero + (-2),
         one = zero + 1,
         three = zero + 3,
         four = zero + 4,
@@ -238,7 +245,7 @@ TestTreeCombine::testComp(const TreeComp& comp, const ValueComp& op)
         TreeT bTree(five);
         bTree.setValueOn(openvdb::Coord(0, 0, 0), minusOne);
         bTree.setValueOn(openvdb::Coord(0, 1, 0), four);
-        bTree.setValueOn(openvdb::Coord(0, 1, 2), minusOne);
+        bTree.setValueOn(openvdb::Coord(0, 1, 2), minusTwo);
         bTree.setValueOff(openvdb::Coord(1, 0, 0), minusOne);
         bTree.setValueOff(openvdb::Coord(1, 1, 0), four);
 
@@ -256,8 +263,8 @@ TestTreeCombine::testComp(const TreeComp& comp, const ValueComp& op)
         CPPUNIT_ASSERT_EQUAL(op(one, five), aTree.getValue(openvdb::Coord(0, 0, 2)));
         CPPUNIT_ASSERT(aTree.isValueOn(openvdb::Coord(0, 0, 2)));
 
-        // a = 1 (On, = bg), b = -1 (On)
-        CPPUNIT_ASSERT_EQUAL(op(one, minusOne), aTree.getValue(openvdb::Coord(0, 1, 2)));
+        // a = 1 (On, = bg), b = -2 (On)
+        CPPUNIT_ASSERT_EQUAL(op(one, minusTwo), aTree.getValue(openvdb::Coord(0, 1, 2)));
         CPPUNIT_ASSERT(aTree.isValueOn(openvdb::Coord(0, 1, 2)));
 
         // a = 1 (bg), b = 4 (On)
@@ -294,7 +301,7 @@ TestTreeCombine::testComp(const TreeComp& comp, const ValueComp& op)
         TreeT bTree(five);
         bTree.setValueOn(openvdb::Coord(0, 0, 0), minusOne);
         bTree.setValueOn(openvdb::Coord(0, 1, 0), four);
-        bTree.setValueOn(openvdb::Coord(0, 1, 2), minusOne);
+        bTree.setValueOn(openvdb::Coord(0, 1, 2), minusTwo);
         bTree.setValueOff(openvdb::Coord(1, 0, 0), minusOne);
         bTree.setValueOff(openvdb::Coord(1, 1, 0), four);
 
@@ -312,8 +319,8 @@ TestTreeCombine::testComp(const TreeComp& comp, const ValueComp& op)
         CPPUNIT_ASSERT_EQUAL(op(five, one), bTree.getValue(openvdb::Coord(0, 0, 2)));
         CPPUNIT_ASSERT(bTree.isValueOn(openvdb::Coord(0, 0, 2)));
 
-        // a = 1 (On, = bg), b = -1 (On)
-        CPPUNIT_ASSERT_EQUAL(op(minusOne, one), bTree.getValue(openvdb::Coord(0, 1, 2)));
+        // a = 1 (On, = bg), b = -2 (On)
+        CPPUNIT_ASSERT_EQUAL(op(minusTwo, one), bTree.getValue(openvdb::Coord(0, 1, 2)));
         CPPUNIT_ASSERT(bTree.isValueOn(openvdb::Coord(0, 1, 2)));
 
         // a = 1 (bg), b = 4 (On)
