@@ -97,14 +97,37 @@ TestTreeIterators::testLeafIterator()
 }
 
 
+// Test the leaf iterator over a tree without any leaf nodes.
 void
 TestTreeIterators::testEmptyLeafIterator()
 {
+    using namespace openvdb;
+
     TreeType tree(/*fillValue=*/256.0);
 
-    // Test iterator without any leaf nodes.
+    std::vector<Index> dims;
+    tree.getNodeLog2Dims(dims);
+    CPPUNIT_ASSERT_EQUAL(4, int(dims.size()));
+
+    // Start with an iterator over an empty tree.
     TreeType::LeafCIter iter = tree.cbeginLeaf();
     CPPUNIT_ASSERT(!iter);
+
+    // Using sparse fill, add internal nodes but no leaf nodes to the tree.
+
+    // Fill the region subsumed by a level-2 internal node (assuming a four-level tree).
+    Index log2Sum = dims[1] + dims[2] + dims[3];
+    CoordBBox bbox(Coord(0), Coord((1 << log2Sum) - 1));
+    tree.fill(bbox, /*value=*/1.0);
+    iter = tree.cbeginLeaf();
+    CPPUNIT_ASSERT(!iter);
+
+    // Fill the region subsumed by a level-1 internal node.
+    log2Sum = dims[2] + dims[3];
+    bbox.reset(Coord(0), Coord((1 << log2Sum) - 1));
+    tree.fill(bbox, /*value=*/2.0);
+    iter = tree.cbeginLeaf();
+    CPPUNIT_ASSERT(!iter);
 }
 
 
@@ -231,7 +254,8 @@ TestTreeIterators::testValueAllIterator()
                     CPPUNIT_ASSERT(lvl < 4);
                     openvdb::CoordBBox bbox;
                     iter.getBoundingBox(bbox);
-                    CPPUNIT_ASSERT_EQUAL(bbox.extents(), openvdb::Coord(dim[lvl], dim[lvl], dim[lvl]));
+                    CPPUNIT_ASSERT_EQUAL(
+                        bbox.extents(), openvdb::Coord(dim[lvl], dim[lvl], dim[lvl]));
                 }
             }
         }
