@@ -28,17 +28,9 @@
 //
 ///////////////////////////////////////////////////////////////////////////
 
-#include <sstream>
 #include <cppunit/extensions/HelperMacros.h>
-#include <openvdb/Exceptions.h>
-#include <sys/types.h> // for stat()
-#include <sys/stat.h>
-#include <stdlib.h> // for system()
 #include <tbb/task.h>
-#include <openvdb/Types.h>
-#include <openvdb/tree/LeafNode.h>
-#include <openvdb/tree/Tree.h>
-#include <openvdb/tree/ValueAccessor.h>
+#include <boost/type_traits/remove_const.hpp>
 #include <openvdb/openvdb.h>
 
 #define ASSERT_DOUBLES_EXACTLY_EQUAL(expected, actual) \
@@ -46,9 +38,25 @@
 
 
 typedef float ValueType;
-typedef openvdb::tree::Tree4<ValueType, 5, 4, 3>::Type TreeType;
+typedef openvdb::tree::Tree<
+    openvdb::tree::RootNode<
+    openvdb::tree::LeafNode<ValueType, 3> > > Tree2Type;
+typedef openvdb::tree::Tree<
+    openvdb::tree::RootNode<
+    openvdb::tree::InternalNode<
+    openvdb::tree::LeafNode<ValueType, 3>, 4> > > Tree3Type;
+typedef openvdb::tree::Tree4<ValueType, 5, 4, 3>::Type Tree4Type;
+typedef openvdb::tree::Tree<
+    openvdb::tree::RootNode<
+    openvdb::tree::InternalNode<
+    openvdb::tree::InternalNode<
+    openvdb::tree::InternalNode<
+    openvdb::tree::LeafNode<ValueType, 3>, 4>, 5>, 5> > > Tree5Type;
+typedef Tree4Type TreeType;
 
 
+using namespace openvdb::tree;
+
 class TestValueAccessor: public CppUnit::TestFixture
 {
 public:
@@ -56,19 +64,85 @@ public:
     virtual void tearDown() { openvdb::uninitialize(); }
 
     CPPUNIT_TEST_SUITE(TestValueAccessor);
-    CPPUNIT_TEST(testValueAccessor);
-    CPPUNIT_TEST(testValueAccessorRW);
-    CPPUNIT_TEST(testConstValueAccessor);
-    CPPUNIT_TEST(testConstValueAccessorRW);
+
+    CPPUNIT_TEST(testTree2Accessor);
+    CPPUNIT_TEST(testTree2AccessorRW);
+    CPPUNIT_TEST(testTree2ConstAccessor);
+    CPPUNIT_TEST(testTree2ConstAccessorRW);
+
+    CPPUNIT_TEST(testTree3Accessor);
+    CPPUNIT_TEST(testTree3AccessorRW);
+    CPPUNIT_TEST(testTree3ConstAccessor);
+    CPPUNIT_TEST(testTree3ConstAccessorRW);
+
+    CPPUNIT_TEST(testTree4Accessor);
+    CPPUNIT_TEST(testTree4AccessorRW);
+    CPPUNIT_TEST(testTree4ConstAccessor);
+    CPPUNIT_TEST(testTree4ConstAccessorRW);
+
+    CPPUNIT_TEST(testTree5Accessor);
+    CPPUNIT_TEST(testTree5AccessorRW);
+    CPPUNIT_TEST(testTree5ConstAccessor);
+    CPPUNIT_TEST(testTree5ConstAccessorRW);
+
+    CPPUNIT_TEST(testTree3Accessor2);
+    CPPUNIT_TEST(testTree3ConstAccessor2);
+    CPPUNIT_TEST(testTree4Accessor2);
+    CPPUNIT_TEST(testTree4ConstAccessor2);
+    CPPUNIT_TEST(testTree4Accessor1);
+    CPPUNIT_TEST(testTree4ConstAccessor1);
+    CPPUNIT_TEST(testTree4Accessor0);
+    CPPUNIT_TEST(testTree4ConstAccessor0);
+    CPPUNIT_TEST(testTree5Accessor2);
+    CPPUNIT_TEST(testTree5ConstAccessor2);
+    CPPUNIT_TEST(testTree4Accessor12);//cache node level 2
+    CPPUNIT_TEST(testTree5Accessor213);//cache node level 1 and 3
+
     CPPUNIT_TEST(testMultithreadedAccessor);
     CPPUNIT_TEST(testAccessorRegistration);
     CPPUNIT_TEST(testGetNode);
-    CPPUNIT_TEST_SUITE_END();
 
-    void testValueAccessor();
-    void testValueAccessorRW();
-    void testConstValueAccessor();
-    void testConstValueAccessorRW();
+    CPPUNIT_TEST_SUITE_END();
+    // cache all node levels
+    void testTree2Accessor()        { accessorTest<ValueAccessor<Tree2Type> >(); }
+    void testTree2AccessorRW()      { accessorTest<ValueAccessorRW<Tree2Type> >(); }
+    void testTree2ConstAccessor()   { constAccessorTest<ValueAccessor<const Tree2Type> >(); }
+    void testTree2ConstAccessorRW() { constAccessorTest<ValueAccessorRW<const Tree2Type> >(); }
+    // cache all node levels
+    void testTree3Accessor()        { accessorTest<ValueAccessor<Tree3Type> >(); }
+    void testTree3AccessorRW()      { accessorTest<ValueAccessorRW<Tree3Type> >(); }
+    void testTree3ConstAccessor()   { constAccessorTest<ValueAccessor<const Tree3Type> >(); }
+    void testTree3ConstAccessorRW() { constAccessorTest<ValueAccessorRW<const Tree3Type> >(); }
+    // cache all node levels
+    void testTree4Accessor()        { accessorTest<ValueAccessor<Tree4Type> >(); }
+    void testTree4AccessorRW()      { accessorTest<ValueAccessorRW<Tree4Type> >(); }
+    void testTree4ConstAccessor()   { constAccessorTest<ValueAccessor<const Tree4Type> >(); }
+    void testTree4ConstAccessorRW() { constAccessorTest<ValueAccessorRW<const Tree4Type> >(); }
+    // cache all node levels
+    void testTree5Accessor()        { accessorTest<ValueAccessor<Tree5Type> >(); }
+    void testTree5AccessorRW()      { accessorTest<ValueAccessorRW<Tree5Type> >(); }
+    void testTree5ConstAccessor()   { constAccessorTest<ValueAccessor<const Tree5Type> >(); }
+    void testTree5ConstAccessorRW() { constAccessorTest<ValueAccessorRW<const Tree5Type> >(); }
+
+    // Test odd combinations of trees and ValueAccessors
+    // cache node level 0 and 1
+    void testTree3Accessor2()        { accessorTest<ValueAccessor<Tree3Type,2> >(); }
+    void testTree3ConstAccessor2()   { constAccessorTest<ValueAccessor<const Tree3Type,2> >(); }
+    void testTree4Accessor2()        { accessorTest<ValueAccessor<Tree4Type,2> >(); }
+    void testTree4ConstAccessor2()   { constAccessorTest<ValueAccessor<const Tree4Type,2> >(); }
+    void testTree5Accessor2()        { accessorTest<ValueAccessor<Tree5Type,2> >(); }
+    void testTree5ConstAccessor2()   { constAccessorTest<ValueAccessor<const Tree5Type,2> >(); }
+    // only cache leaf level
+    void testTree4Accessor1()        { accessorTest<ValueAccessor<Tree5Type,1> >(); }
+    void testTree4ConstAccessor1()   { constAccessorTest<ValueAccessor<const Tree5Type,1> >(); }
+    // disable node caching
+    void testTree4Accessor0()        { accessorTest<ValueAccessor<Tree5Type,0> >(); }
+    void testTree4ConstAccessor0()   { constAccessorTest<ValueAccessor<const Tree5Type,0> >(); }
+    //cache node level 2
+    void testTree4Accessor12()       { accessorTest<ValueAccessor1<Tree4Type,2> >(); }
+    //cache node level 1 and 3
+    void testTree5Accessor213()       { accessorTest<ValueAccessor2<Tree5Type,1,3> >(); }
+
     void testMultithreadedAccessor();
     void testAccessorRegistration();
     void testGetNode();
@@ -84,40 +158,16 @@ CPPUNIT_TEST_SUITE_REGISTRATION(TestValueAccessor);
 ////////////////////////////////////////
 
 
-void
-TestValueAccessor::testValueAccessor()
-{
-    accessorTest<openvdb::tree::ValueAccessor<TreeType> >();
-}
-
-void
-TestValueAccessor::testValueAccessorRW()
-{
-    accessorTest<openvdb::tree::ValueAccessorRW<TreeType> >();
-}
-
-void
-TestValueAccessor::testConstValueAccessor()
-{
-    constAccessorTest<openvdb::tree::ValueAccessor<const TreeType> >();
-}
-
-void
-TestValueAccessor::testConstValueAccessorRW()
-{
-    constAccessorTest<openvdb::tree::ValueAccessorRW<const TreeType> >();
-}
-
-
-////////////////////////////////////////
-
-
 template<typename AccessorT>
 void
 TestValueAccessor::accessorTest()
 {
+    typedef typename AccessorT::TreeType TreeType;
+    const int leafDepth = int(TreeType::DEPTH) - 1;
+    // subtract one because getValueDepth() returns 0 for values at the root
+
     const ValueType background = 5.0f, value = -9.345f;
-    const openvdb::Coord c0(5, 10, 20), c1(50000, 20000, 30000);
+    const openvdb::Coord c0(5, 10, 20), c1(500000, 200000, 300000);
 
     {
         TreeType tree(background);
@@ -151,7 +201,7 @@ TestValueAccessor::accessorTest()
         CPPUNIT_ASSERT(!acc.isVoxel(c0));
         CPPUNIT_ASSERT(!acc.isVoxel(c1));
 
-        tree.setValue(c0, value);
+        acc.setValue(c0, value);
 
         CPPUNIT_ASSERT(tree.isValueOn(c0));
         CPPUNIT_ASSERT(!tree.isValueOn(c1));
@@ -161,20 +211,21 @@ TestValueAccessor::accessorTest()
         ASSERT_DOUBLES_EXACTLY_EQUAL(value, v);
         CPPUNIT_ASSERT(!acc.probeValue(c1,v));
         ASSERT_DOUBLES_EXACTLY_EQUAL(background, v);
-        CPPUNIT_ASSERT_EQUAL( 3, acc.getValueDepth(c0));// leaf-level voxel value
-        CPPUNIT_ASSERT_EQUAL(-1, acc.getValueDepth(c1));// background value
-        CPPUNIT_ASSERT_EQUAL( 3, acc.getValueDepth(openvdb::Coord(7, 10, 20)));
-        CPPUNIT_ASSERT_EQUAL( 2, acc.getValueDepth(openvdb::Coord(8, 10, 20)));
-        CPPUNIT_ASSERT( acc.isVoxel(c0));// leaf-level voxel value
+        CPPUNIT_ASSERT_EQUAL(leafDepth, acc.getValueDepth(c0)); // leaf-level voxel value
+        CPPUNIT_ASSERT_EQUAL(-1, acc.getValueDepth(c1)); // background value
+        CPPUNIT_ASSERT_EQUAL(leafDepth, acc.getValueDepth(openvdb::Coord(7, 10, 20)));
+        const int depth = leafDepth == 1 ? -1 : leafDepth - 1;
+        CPPUNIT_ASSERT_EQUAL(depth, acc.getValueDepth(openvdb::Coord(8, 10, 20)));
+        CPPUNIT_ASSERT( acc.isVoxel(c0)); // leaf-level voxel value
         CPPUNIT_ASSERT(!acc.isVoxel(c1));
         CPPUNIT_ASSERT( acc.isVoxel(openvdb::Coord(7, 10, 20)));
         CPPUNIT_ASSERT(!acc.isVoxel(openvdb::Coord(8, 10, 20)));
 
         ASSERT_DOUBLES_EXACTLY_EQUAL(background, acc.getValue(c1));
         CPPUNIT_ASSERT(!acc.isCached(c1)); // uncached background value
-        CPPUNIT_ASSERT(!acc.isValueOn(c1));// inactive background value
+        CPPUNIT_ASSERT(!acc.isValueOn(c1)); // inactive background value
         ASSERT_DOUBLES_EXACTLY_EQUAL(value, acc.getValue(c0));
-        CPPUNIT_ASSERT(acc.isCached(c0)); // active, leaf-level voxel value
+        CPPUNIT_ASSERT((acc.numCacheLevels()>0) == acc.isCached(c0)); // active, leaf-level voxel value
         CPPUNIT_ASSERT(acc.isValueOn(c0));
 
         acc.setValue(c1, value);
@@ -182,13 +233,13 @@ TestValueAccessor::accessorTest()
         CPPUNIT_ASSERT(acc.isValueOn(c1));
         ASSERT_DOUBLES_EXACTLY_EQUAL(value, tree.getValue(c0));
         ASSERT_DOUBLES_EXACTLY_EQUAL(value, tree.getValue(c1));
-        CPPUNIT_ASSERT(acc.isCached(c1));
+        CPPUNIT_ASSERT((acc.numCacheLevels()>0) == acc.isCached(c1));
         ASSERT_DOUBLES_EXACTLY_EQUAL(value, acc.getValue(c1));
         CPPUNIT_ASSERT(!acc.isCached(c0));
         ASSERT_DOUBLES_EXACTLY_EQUAL(value, acc.getValue(c0));
-        CPPUNIT_ASSERT(acc.isCached(c0));
-        CPPUNIT_ASSERT_EQUAL( 3, acc.getValueDepth(c0));
-        CPPUNIT_ASSERT_EQUAL( 3, acc.getValueDepth(c1));
+        CPPUNIT_ASSERT((acc.numCacheLevels()>0) == acc.isCached(c0));
+        CPPUNIT_ASSERT_EQUAL(leafDepth, acc.getValueDepth(c0));
+        CPPUNIT_ASSERT_EQUAL(leafDepth, acc.getValueDepth(c1));
         CPPUNIT_ASSERT(acc.isVoxel(c0));
         CPPUNIT_ASSERT(acc.isVoxel(c1));
 
@@ -197,14 +248,14 @@ TestValueAccessor::accessorTest()
         ASSERT_DOUBLES_EXACTLY_EQUAL(value, tree.getValue(c0));
         ASSERT_DOUBLES_EXACTLY_EQUAL(value, tree.getValue(c1));
         CPPUNIT_ASSERT(!acc.isCached(c0));
-        CPPUNIT_ASSERT( acc.isCached(c1));
+        CPPUNIT_ASSERT((acc.numCacheLevels()>0) == acc.isCached(c1));
         CPPUNIT_ASSERT( acc.isValueOn(c0));
         CPPUNIT_ASSERT(!acc.isValueOn(c1));
 
         acc.setValueOn(c1);
 
         CPPUNIT_ASSERT(!acc.isCached(c0));
-        CPPUNIT_ASSERT( acc.isCached(c1));
+        CPPUNIT_ASSERT((acc.numCacheLevels()>0) == acc.isCached(c1));
         CPPUNIT_ASSERT( acc.isValueOn(c0));
         CPPUNIT_ASSERT( acc.isValueOn(c1));
 
@@ -213,13 +264,13 @@ TestValueAccessor::accessorTest()
         CPPUNIT_ASSERT(acc.isValueOn(c1));
         ASSERT_DOUBLES_EXACTLY_EQUAL(value, tree.getValue(c0));
         ASSERT_DOUBLES_EXACTLY_EQUAL(2*value, tree.getValue(c1));
-        CPPUNIT_ASSERT(acc.isCached(c1));
+        CPPUNIT_ASSERT((acc.numCacheLevels()>0) == acc.isCached(c1));
         ASSERT_DOUBLES_EXACTLY_EQUAL(2*value, acc.getValue(c1));
         CPPUNIT_ASSERT(!acc.isCached(c0));
         ASSERT_DOUBLES_EXACTLY_EQUAL(value, acc.getValue(c0));
-        CPPUNIT_ASSERT(acc.isCached(c0));
-        CPPUNIT_ASSERT_EQUAL( 3, acc.getValueDepth(c0));
-        CPPUNIT_ASSERT_EQUAL( 3, acc.getValueDepth(c1));
+        CPPUNIT_ASSERT((acc.numCacheLevels()>0) == acc.isCached(c0));
+        CPPUNIT_ASSERT_EQUAL(leafDepth, acc.getValueDepth(c0));
+        CPPUNIT_ASSERT_EQUAL(leafDepth, acc.getValueDepth(c1));
         CPPUNIT_ASSERT(acc.isVoxel(c0));
         CPPUNIT_ASSERT(acc.isVoxel(c1));
 
@@ -228,13 +279,13 @@ TestValueAccessor::accessorTest()
         CPPUNIT_ASSERT(acc.isValueOn(c1));
         ASSERT_DOUBLES_EXACTLY_EQUAL(value, tree.getValue(c0));
         ASSERT_DOUBLES_EXACTLY_EQUAL(3*value, tree.getValue(c1));
-        CPPUNIT_ASSERT(acc.isCached(c1));
+        CPPUNIT_ASSERT((acc.numCacheLevels()>0) == acc.isCached(c1));
         ASSERT_DOUBLES_EXACTLY_EQUAL(3*value, acc.getValue(c1));
         CPPUNIT_ASSERT(!acc.isCached(c0));
         ASSERT_DOUBLES_EXACTLY_EQUAL(value, acc.getValue(c0));
-        CPPUNIT_ASSERT(acc.isCached(c0));
-        CPPUNIT_ASSERT_EQUAL( 3, acc.getValueDepth(c0));
-        CPPUNIT_ASSERT_EQUAL( 3, acc.getValueDepth(c1));
+        CPPUNIT_ASSERT((acc.numCacheLevels()>0) == acc.isCached(c0));
+        CPPUNIT_ASSERT_EQUAL(leafDepth, acc.getValueDepth(c0));
+        CPPUNIT_ASSERT_EQUAL(leafDepth, acc.getValueDepth(c1));
         CPPUNIT_ASSERT(acc.isVoxel(c0));
         CPPUNIT_ASSERT(acc.isVoxel(c1));
 
@@ -249,8 +300,12 @@ template<typename AccessorT>
 void
 TestValueAccessor::constAccessorTest()
 {
+    typedef typename boost::remove_const<typename AccessorT::TreeType>::type TreeType;
+    const int leafDepth = int(TreeType::DEPTH) - 1;
+        // subtract one because getValueDepth() returns 0 for values at the root
+
     const ValueType background = 5.0f, value = -9.345f;
-    const openvdb::Coord c0(5, 10, 20), c1(50000, 20000, 30000);
+    const openvdb::Coord c0(5, 10, 20), c1(500000, 200000, 300000);
     ValueType v;
 
     TreeType tree(background);
@@ -268,8 +323,8 @@ TestValueAccessor::constAccessorTest()
     ASSERT_DOUBLES_EXACTLY_EQUAL(background, v);
     CPPUNIT_ASSERT_EQUAL(-1, acc.getValueDepth(c0));
     CPPUNIT_ASSERT_EQUAL(-1, acc.getValueDepth(c1));
-     CPPUNIT_ASSERT(!acc.isVoxel(c0));
-     CPPUNIT_ASSERT(!acc.isVoxel(c1));
+    CPPUNIT_ASSERT(!acc.isVoxel(c0));
+    CPPUNIT_ASSERT(!acc.isVoxel(c1));
 
     tree.setValue(c0, value);
 
@@ -284,15 +339,15 @@ TestValueAccessor::constAccessorTest()
     ASSERT_DOUBLES_EXACTLY_EQUAL(value, v);
     CPPUNIT_ASSERT(!acc.probeValue(c1,v));
     ASSERT_DOUBLES_EXACTLY_EQUAL(background, v);
-    CPPUNIT_ASSERT_EQUAL( 3, acc.getValueDepth(c0));
+    CPPUNIT_ASSERT_EQUAL(leafDepth, acc.getValueDepth(c0));
     CPPUNIT_ASSERT_EQUAL(-1, acc.getValueDepth(c1));
     CPPUNIT_ASSERT( acc.isVoxel(c0));
     CPPUNIT_ASSERT(!acc.isVoxel(c1));
 
     ASSERT_DOUBLES_EXACTLY_EQUAL(value, acc.getValue(c0));
-    CPPUNIT_ASSERT(acc.isCached(c0));
+    CPPUNIT_ASSERT((acc.numCacheLevels()>0) == acc.isCached(c0));
     ASSERT_DOUBLES_EXACTLY_EQUAL(background, acc.getValue(c1));
-    CPPUNIT_ASSERT(acc.isCached(c0));
+    CPPUNIT_ASSERT((acc.numCacheLevels()>0) == acc.isCached(c0));
     CPPUNIT_ASSERT(!acc.isCached(c1));
     CPPUNIT_ASSERT(acc.isValueOn(c0));
     CPPUNIT_ASSERT(!acc.isValueOn(c1));
@@ -301,11 +356,11 @@ TestValueAccessor::constAccessorTest()
 
     ASSERT_DOUBLES_EXACTLY_EQUAL(value, acc.getValue(c1));
     CPPUNIT_ASSERT(!acc.isCached(c0));
-    CPPUNIT_ASSERT(acc.isCached(c1));
+    CPPUNIT_ASSERT((acc.numCacheLevels()>0) == acc.isCached(c1));
     CPPUNIT_ASSERT(acc.isValueOn(c0));
     CPPUNIT_ASSERT(acc.isValueOn(c1));
-    CPPUNIT_ASSERT_EQUAL( 3, acc.getValueDepth(c0));
-    CPPUNIT_ASSERT_EQUAL( 3, acc.getValueDepth(c1));
+    CPPUNIT_ASSERT_EQUAL(leafDepth, acc.getValueDepth(c0));
+    CPPUNIT_ASSERT_EQUAL(leafDepth, acc.getValueDepth(c1));
     CPPUNIT_ASSERT(acc.isVoxel(c0));
     CPPUNIT_ASSERT(acc.isVoxel(c1));
 
@@ -324,9 +379,9 @@ TestValueAccessor::testMultithreadedAccessor()
 {
 #define MAX_COORD 5000
 
-    typedef openvdb::tree::ValueAccessorRW<TreeType> AccessorT;
+    typedef openvdb::tree::ValueAccessorRW<Tree4Type> AccessorT;
     // Substituting the following typedef typically results in assertion failures:
-    //typedef openvdb::tree::ValueAccessor<TreeType> AccessorT;
+    //typedef openvdb::tree::ValueAccessor<Tree4Type> AccessorT;
 
     // Task to perform multiple reads through a shared accessor
     struct ReadTask: public tbb::task {
@@ -375,7 +430,7 @@ TestValueAccessor::testMultithreadedAccessor()
         }
     };
 
-    TreeType tree(/*background=*/0.5);
+    Tree4Type tree(/*background=*/0.5);
     AccessorT acc(tree);
     // Populate the tree.
     for (int i = -MAX_COORD; i < MAX_COORD; ++i) {
@@ -430,6 +485,7 @@ TestValueAccessor::testAccessorRegistration()
 
     // Delete the tree and verify that the cache has been cleared.
     tree.reset();
+    CPPUNIT_ASSERT(acc.getTree() == NULL);
     CPPUNIT_ASSERT(acc.getNode<openvdb::FloatTree::RootNodeType>() == NULL);
     CPPUNIT_ASSERT(acc.getNode<openvdb::FloatTree::LeafNodeType>() == NULL);
 }
@@ -438,15 +494,15 @@ TestValueAccessor::testAccessorRegistration()
 void
 TestValueAccessor::testGetNode()
 {
-    typedef TreeType::LeafNodeType LeafT;
+    typedef Tree4Type::LeafNodeType LeafT;
 
     const ValueType background = 5.0f, value = -9.345f;
     const openvdb::Coord c0(5, 10, 20);
 
-    TreeType tree(background);
+    Tree4Type tree(background);
     tree.setValue(c0, value);
     {
-        openvdb::tree::ValueAccessor<TreeType> acc(tree);
+        openvdb::tree::ValueAccessor<Tree4Type> acc(tree);
         // Prime the cache.
         acc.getValue(c0);
         // Verify that the cache contains a leaf node.
@@ -460,7 +516,7 @@ TestValueAccessor::testGetNode()
     }
     {
         // As above, but with a const tree.
-        openvdb::tree::ValueAccessor<const TreeType> acc(tree);
+        openvdb::tree::ValueAccessor<const Tree4Type> acc(tree);
         acc.getValue(c0);
         const LeafT* node = acc.getNode<const LeafT>();
         CPPUNIT_ASSERT(node != NULL);
